/* GOES Radio Comments 
GOESProcessor.cpp is the class that does the data link layer processing for EMWIN-N 
and HRIT/LRIT.  
InitializeProcessor(), Initializes the memory buffers 
UninitializeProcessor(), Deletes the memory buffers

InitializeSocket(emwin socket flag, lrit socket flag, socket only flag) Creates
a socket connection for emwin and lrit data.  the socket flags are socket port numbers
that send data over localhost:flag.  the socket only flag prevents this application 
from writing the emwin products to disk, and just streams them over the socket interface.

UnitiliazeSocket() closes the socket connection

ProcessLritDataBLock(Buffer, Buffer size, x,y) is a wrapper for LRIT_main(x,y).  It 
takes the data from buffer, and puts it into a global array, CurrentDataPtr,  that 
LRIT_main can access.  This global array was initialized in InitializeProcessor()

LRIT_main(resynch threshold, BERFIleNAme) takes the data and performs frame synchronization.
If there is a 180 degree phase ambiguity, it resolves it by correlating to the inverted 
header sequence.  LRIT_main uses a state-machine with 4 levels of lock (0 to 3) to maintain frame
synchronization.  the user-defined resynch threshold is the threshold for lock-state 3 (total lock).
After Frame Synch, LRIT_main does Reed Solomon Decoding, LRITrs->Decode().  The RS-decoded 
CADU frame data is stored
in LRITframe.  From here, the function can write status messages like BER and FER to a file 
(BERFileName) and to the screen.  After this stage it sends the data over
a socket for further processing if that option is enabled.  If that option is not enabled it reads
the VCDU. It also reads the APIDs using GetLritPacketHeader();  During this function call, LRITFile[]
gets assigned to LritFileptr[].  When GetLritPacketHeader returns, it copies the processed LRIT data 
(LRITframe) into an 2-D array LritFilePtr[APIDindex].  LritFilePtr[0] would represent a pointer to one
LRIT data frame.  We store multiple file parts at a time, and index each one based on the APIDindex 
table.  Because LritFilePtr[] points to LRITFile[], LRITFile[] also gets the contents of the LRITframe(s). 
Frome here it calls PRocessLRITFile 

ProcessLRITFile()
This function associates a new global array, LpBufPtr to LRITFile[APIDindex], 
thus getting access to a single LRIT frame data.  It eventually writes the actual LRIT data products 
(Saved in another global array called FileBuffer) to file. Before it does this, signficant processing 
is done in CalcCRC(). 

CalcCRC()
CalcCRC grabs data from the global array lpBufPtr.  It also associates a new global array, filepointer, 
to FileBuffer.  FileBuffer is used by ProcessLRITFile() to write the LRIT products to disk. Using 
LpBufPtr, it checks validates the CRC and copies the contents of LpBufPtr into filepointer (which is
pointing to FileBuffer). It evaluates the contents of the LRIT data (LpBufPtr).  If its header data, 
it parses it for the following information:  Primary Header, Header Structure Record, NOAA LRIT Record,
Image Structure Record,Image Structure Record, Image Navigation Record, Image Data Function, 
Annotation Record, Time Stamp, Ancillary Text Record, Image Segment ID Record, Rice Compression Record, 
Data Filename, and EMWIN file marker
when it gets to the body of the data, based on the header, it either performs rice decompression or
sends the emwin data out to a emwin processor.  For the Rice Decompression, it copies copies the 
contents of LpBufPtr to riceptr, which is a global array that RiceDecompress() uses.  If 
RiceDecompress is successful, its output is copied into LRITImageptr.  Finalyl, LRITImageptr is 
copied into filepoint, which points to (FileBuffer).  CalcCRC() exits, and ProcessLRITFile() writes
the uncompressed LRIT data to a file.  If the packet contains EMWIN data, it copies the contents of 
LpBufPtr into a global array called EmwinPtr and then calls ProcessEmwinFile();  If the emwin socket
flag is set, it copies LpBufPtr into EmwinSocketFrame, then sends that data over a socket. 



For EMWIN-N processing
ProcessEmwinDataBLock(Buffer, Buffer size, x,y) is called.
It takes the data from buffer, and puts it into a global array, CurrentDataPtr, that EMWIN_main() can 
access.  This global array was initialized in InitializeProcessor().  

EMWIN_main(resynch threshold, BERFIleNAme) and LRIT_main() are identical, until after RS Solomon 
Decoding is performed.  The buffer that contains the decoded EMWIN-N CADU frame data is called LRITframe 
unfortunately due to copy/paste issues.  The APIDS for EMWIN-N are not read like they are for LRIT. The
VCDU header is checked to see if the current EMWIN frame is a fill or data packet.  If its a valid data 
packet, LRITframe is copied to a global array EmwinPtr, and then ProcessEmwinFile() is called.  If the 
emwin socket flag points to a valid port, the data is also copied to EmwinSocketFrame, and sent over the 
socket.

ProcessEmwinFile()
EmwinPtr is linked to the global array EmwinProcPtr.  EmwinProcPtr is evaluated for the six nulls and the 
file name.  There are checks throughout the code that makes sure there is enough data in the temporary 
buffers to finish processing.  Once the file name is found, the emwin packet is stored in a packet array 
that stores the temporary files via EmwinPtrtabl[][] = EmwinProcPtr.  When enough emwin frames are collected
to complete a file, emwindisplay() is called

Emwindisplay()
Parses the data, and writes zip, txt, gif, and jpeg files to disk. 
*/
#include <stdio.h>
#include <math.h>

#include "GoesProcessor.h"
#define RE_ACQ                          //Enables ReAcq Support
#define NO_SYNC_LIMIT     (600*1024)    //ReAcq is engaged after 600 frames are missed
#define MIN_DATA_SIZE        (2048)     //Minimum number of bytes before beginning packet processing
#define FILENAME_SIZE          (12)     //Size of the EMWIN File Names
#define HEADER_SIZE            (80)     
#define WAIT_TIL_DISCARD      (100)     //If its been 100 packets since a part of an incomplete file is received, then discard.
#define NUMBER_OF_FILES        (20)     //maximum number of partial files stored in memeory at once
#define TMP_BUFFER_SIZE   (1000000)     //Stores temporary data. The bigger, the more stable, but the more memory used. 
#define BIG_BUFFER_SIZE  (15000000)    //This buffer holds all processed data. The larger it is, the safer, but the more memory used.

CGoesProcessor::CGoesProcessor()
{

    DataBuffer      = NULL;
    //DataBuffer:: points, always to the begining of the allocated memory
    LastProcBytePtr   = NULL;
    //LastProcBytePtr (oldname: dispind) increases by 1020bytes,in LRIT_Main when a "good" block was
    // found. If a header is not found, it increases by 1, and the search for the header
    // restarts
    // After one is synched, LastProcBytePtr points to the last processed RS byte
    old_LastProcBytePtr   = NULL;
    // Keeps the old value of LastProcBytePtr. It is only updated on synched mode

    CurrentDataPtr        = NULL;
    //CurrentDataPtr: As data comes in from GNURadio, it is copied to the "Processing Buffer"
    // In the function, ProcessLRITDataBlock  , we check to see where the difference
    // (CurrentDataPtr - LastProcBytePtr)  > MIN_DATA_SIZE bytres. When this happens, we go ahead and process it.

    lpBufPtr        = NULL;
    LRITFirstsync   = TRUE;
    LRITsyncflag    = FALSE;
    Firstsyncflag   = TRUE;
    foundsyncflag   = FALSE;

    //------------------------------------------------
    //2.3	EMWIN-OQPSK CCSDS Data Format Summary
    //   CCSDS Data Format Summary
    //ASM	Hexadecimal 1ACFFC1D
    Ltreg           = 0xffffffff;
    lritsync        = 0x1acffc1d;
    //------------------------------------------------
    LRITDataInvert  = 0;
    InvByteCount    = 0;
    InvBadFrameCount= 0;
    LRITFrameCount  = 0;
    PendingAPID     = 2047;
    LRITrs          = NULL;
    VCDUcount       = 0;
    Fullheader      = FALSE;
    Extension       = FALSE;
    InvToggleFlag   = FALSE;
    PacketIndex     = 0;
    SrchHeaderFlag  = FALSE;
    LRIThavename    = FALSE;
    LRITfiletype    = 130;
    FileBuffer      = NULL;
    numLRITbytes    = 0;
    haveLRITfiles   = FALSE;
    FrameSyncCnt    = 0;
    EmwinDataSize   = BIG_BUFFER_SIZE; //JR Debug
    EmwinData       = NULL;
    EmwinBytes      = 0;
    EmwinNullCount  = 0;
    haveEMWINfiles  = FALSE;
    EmwinStart      = FALSE;
    EmwinPacketCount= 0;
    EmwinTempBuffer = NULL;
    EmwinProcBytes  = 0;
    viterbi_framerrors = 0; 
    RS_framerrors   = 0; 
    ErCountCum      = 0; 
    USE_SOCKET_GBL = 0;   // A emwin socket flag seen by all functions within this class
    USE_SOCKET_LRIT_GBL = 0; //An lrit socket flag seen by all functions within this class
    strcpy(statestring, "ALL");
    for (int i = 0; i < NUMBER_OF_FILES; i++)
    {
        LRITFile[i]     = NULL;
        EmwinFileName[i]   = NULL;
    }

    //EV: These Variables used to be Static
    // FRom LRIT_Main
    LRITFrameError =0;
    Missed_FrameCount = 0;
    VCDUmark1 =0;  //Marker to see if we missed a VCDU frame
    VCDUmark0 = 0;
    lock_status = -1; //lock status =-1,1,2,3 means completely_unsynched, somewhat_synched, pretty_synched, extremely_synched
    consecutive_good_frames = 0;
    consecutive_frames_missed = 0;
    consecutive_frame_errors = 0;
    this->bad_data_count =0;
    best_offset=0;
    best_offset_ready_signal = 1;
    MissedFrameInitialState = TRUE;

    /// EMWIN Variables
    EMWINFrameCount =0;
    EMWINFrameError =0;
    remain=0;
}

BOOL CGoesProcessor::InitializeProcessor()
{

    this-> only_socket =0;
    printf("\n\nGOES Processor Constructor Initialized \n\n");
    this->bad_data_count =0;
    char randm,feedbk;
    unsigned int i, j;
    InvToggleFlag = FALSE;

    LRITrs = new CReedSolomon(8,16,112,11,0,4,0,1);
    if (!LRITrs)
    {
        printf("Error allocating memory for reed solomon\n");
        return FALSE;
    }

    DataBuffer = new unsigned char[BIG_BUFFER_SIZE];
    if (!DataBuffer)
    {
        printf("Error allocating memory for DataBuffer\n");
        return FALSE;
    }

    lpBufPtr            = DataBuffer;
    LastProcBytePtr     = DataBuffer;
    invresetpointer     = DataBuffer;
    CurrentDataPtr      = DataBuffer;
    old_LastProcBytePtr = DataBuffer;

    EmwinData = new unsigned char[EmwinDataSize];
    if (!EmwinData)
    {
        printf("Error allocating memory for EmwinData\n");
        return FALSE;
    }
    EmwinBufEnd  = EmwinData + EmwinDataSize;
    EmwinPtr     = EmwinData;
    EmwinProcPtr = EmwinData;

    EmwinTempBuffer = new unsigned char[TMP_BUFFER_SIZE];
    if (!EmwinTempBuffer)
    {
        printf("Error allocating memory for EmwinTempBuffer\n");
        return FALSE;
    }

    FileBuffer = new unsigned char[TMP_BUFFER_SIZE];
    if (!FileBuffer)
    {
        printf("Error allocating memory for FileBuffer\n");
        return FALSE;
    }


    for (i = 0; i < NUMBER_OF_FILES; i++)  {
        //---------------- LRIT --------------------------------
        LRITFile[i] = new unsigned char[TMP_BUFFER_SIZE];
        if (!LRITFile[i]){
            printf("Error allocating memory for LRITFile\n");
            return FALSE;
        }

        LRITFilePtr[i] = LRITFile[i];
        //---------------- EMWIN --------------------------------
        // Last one is a null
        EmwinFileName[i] = new unsigned char[FILENAME_SIZE+1];
        if (!EmwinFileName[i])
        {
            printf("Error allocating memory for EmwinFileName\n");
            return FALSE;
        }
        //BUG *(EmwinFileName[i] + FILENAME_SIZE+1) = 0;	//NULL to make into string
        // The way it should be
        //      EmwinFileIndex[0,1,2,3,4,5,6,7] = Characters
        //      EmwinFileIndex[8] = "."
        //      EmwinFileIndex[9,10,11] = Extension
        //      EmwinFileIndex[12] = NULL
        // EmwinFileIndex is an array of "int" so be careful here ...
        //EmwinFileIndex[i][FILENAME_SIZE] = 0;	//NULL to make into string        // This is how it should be

        EmwinFileIndex[i][0] = 0;				//mark slot available
        EmwinFileIndex[i][3] = 0;				//initialize timeout counter ????
        //Added
        EmwinFileIndex[i][1] = 0;
        EmwinFileIndex[i][2] = 0;
    }

    //-----------------------------------------------------------------------------------
    // RANDOMIZER
    // char Random[1020]; = 8160 bits
    //Iniialize Shift Register to all 1's   --> Specified in standard
    randm = (char)0xff;
    // Original POlynomial is :  1 + x3 + x5 + x7 +x8
    for (i = 0; i < 1020; i++)  //1020bytes = 8160 bits in CADU packet
    {
        Random[i] = 0;
        for (j = 0; j <= 7; j++)
        {
            Random[i] = Random[i] << 1;
            if (randm & 0x80) //80h = 1000 0000b
                Random[i]++;

            //Bit-Wise AND between: Fixed shift register(95h) and the state of the
            // feedback register: randm
            feedbk = randm & 0x95;//95h = 1001 0101--> bits 1,3,5,8
            //feedback contains the contents of the registers masked by the polynomial
            //  1 + x3 + x5 + xt +x8 = 95 h
            randm = randm << 1;

            if ((((feedbk & 0x80)^(0x80 & feedbk << 3))^(0x80 & (feedbk << 5)))^(0x80 & (feedbk << 7)))
                randm++;
        }
    }

    for (i = 0; i < 64; i++)
    {
        for (j = 0; j < 9; j++)
        {
            VCIDtable[i][j] = 0;
            if (j == 3)
                VCIDtable[i][j] = 2047; //Set Pending Apid=2047
            if (j == 2)
                VCIDtable[i][j] = 2049; //Set APID index out of range

        }
    }

    for (i = 0; i < MAX_APID_TABLE; i++) //Initialize LRIT variables
    {
        APIDtable[i] = 2040;
        LengthTable[i] = 0;
        filesequence[i] = 0;
        Bytesremaining[i] = 0;
    }

    return TRUE;
}

void CGoesProcessor::InitializeSocket(int socket_flag, int socket_lrit_flag, int my_only_socket)
{
    this->only_socket = my_only_socket;
    
    USE_SOCKET_GBL = socket_flag;
    USE_SOCKET_LRIT_GBL = socket_lrit_flag;
    unsigned short port_number = socket_flag;
    unsigned short lrit_port_number = socket_lrit_flag;

    //Begin Windows Networking Connection if either the EMWIN or LRIT data sockets are invoked.
    if (USE_SOCKET_GBL > 0 | USE_SOCKET_LRIT_GBL > 0)
    {
        int errorCode = WSAStartup(0x0101, &wsaData);
        if (errorCode != 0) {
            printf("Unable to open connection to Windows networking system.\n");
        }
    }

    if (USE_SOCKET_GBL>0) //If the EMWIN socket is invoked
    {
        // Setup the server socket to listen on the bitrate control port
        wx_server_socket = socket(AF_INET, SOCK_STREAM, IPPROTO_TCP);
        struct sockaddr_in server_addr;
        server_addr.sin_family = AF_INET;
        server_addr.sin_port = htons(port_number);
        server_addr.sin_addr.s_addr = inet_addr("127.0.0.1");  //Local Host connection
        if (bind(wx_server_socket, (SOCKADDR*)&server_addr, sizeof(server_addr)) != 0) {
            printf("Unable to start/bind bit-rate request EMWIN server (%d). Port may already be in use.\n", WSAGetLastError());
            closesocket(wx_server_socket);
        }
        if (listen(wx_server_socket, SOMAXCONN) != 0) {
            printf("Unable to listen to EMWIN bitrate-control port(%d) Unknown cause.\n", WSAGetLastError());
            closesocket(wx_server_socket);
        }

        // Accept incoming connections until told to stop:
        //while(g_server_running) {
        printf("Waiting for Wx Weather Message Client to Connect to HRIT-R Server for EMWIN data....\n");
        new_connection = accept(wx_server_socket, NULL, NULL);

        if (new_connection == INVALID_SOCKET) {
            // If invalid, dump the socket and try again...
            printf("Invalid Socket\n");
            closesocket(new_connection);
            USE_SOCKET_GBL = 0;
        }
        else
        {
            printf("valid EMWIN socket now open on port %d\n", port_number);
        }
    }

    if (USE_SOCKET_LRIT_GBL>0) //If the LRIT socket is invoked
    {
        // Setup the server socket to listen on the bitrate control port
        wx_server_socket_lrit = socket(AF_INET, SOCK_STREAM, IPPROTO_TCP);
        struct sockaddr_in server_addr_lrit;
        server_addr_lrit.sin_family = AF_INET;
        server_addr_lrit.sin_port = htons(lrit_port_number);
        server_addr_lrit.sin_addr.s_addr = inet_addr("127.0.0.1");  //Local Host connection
        if (bind(wx_server_socket_lrit, (SOCKADDR*)&server_addr_lrit, sizeof(server_addr_lrit)) != 0) {
            printf("Unable to start/bind bit-rate request LRIT server (%d). Port may already be in use.\n", WSAGetLastError());
            closesocket(wx_server_socket_lrit);
        }
        if (listen(wx_server_socket_lrit, SOMAXCONN) != 0) {
            printf("Unable to listen to LRIT bitrate-control port(%d) Unknown cause.\n", WSAGetLastError());
            closesocket(wx_server_socket_lrit);
        }

        // Accept incoming connections until told to stop:
        //while(g_server_running) {
        printf("Waiting for LRIT Data Client to Connect to HRIT-R Server....\n");
        new_connection_lrit = accept(wx_server_socket_lrit, NULL, NULL);

        if (new_connection_lrit == INVALID_SOCKET) {
            // If invalid, dump the socket and try again...
            printf("Invalid Socket\n");
            closesocket(new_connection_lrit);
            USE_SOCKET_LRIT_GBL = 0;
        }
        else
        {
            printf("valid LRIT socket now open on port %d\n", lrit_port_number);
        }
    }
}

void CGoesProcessor::UnInitializeProcessor()
{

    printf(" GOES Processor Destructor \n");
    if (LRITrs)
        delete LRITrs;
    if (DataBuffer)
        delete [] DataBuffer;
    if (EmwinData)
        delete [] EmwinData;
    if  (EmwinTempBuffer)
        delete [] EmwinTempBuffer;
    if (FileBuffer)
        delete [] FileBuffer;
    for (int i = 0; i < NUMBER_OF_FILES; i++)
    {
        if (LRITFile[i])
            delete [] LRITFile[i];
        if (EmwinFileName[i])
            delete [] EmwinFileName[i];
    }
    this->bad_data_count=0;



}

void CGoesProcessor::UnInitializeSocket()
{
    if (wx_server_socket != NULL){ //JR socket
        closesocket(wx_server_socket);
        wx_server_socket = NULL;
    }
    if (new_connection != NULL){ //JR socket
        closesocket(new_connection);
        new_connection = NULL;
    }
    if (wx_server_socket_lrit != NULL){ //JR socket
        closesocket(wx_server_socket_lrit);
        wx_server_socket_lrit = NULL;
    }
    if (new_connection_lrit != NULL){ //JR socket
        closesocket(new_connection_lrit);
        new_connection_lrit = NULL;
    }
    WSACleanup();  //JR socket
    printf("Socket Closed\n");
}


BOOL CGoesProcessor::ProcessLritDataBlock(unsigned char *ReadBuffer, int next_block_size, int resynch_thresh, char * BERFileName)
{
    long unsigned int bytes_remaining;

    if ((CurrentDataPtr + next_block_size) > (DataBuffer + BIG_BUFFER_SIZE)){

        printf("CurrentDataPtr  = %i \n", CurrentDataPtr + 0);
        printf("next_block_size  = %i \n", next_block_size);
        printf("DataBuffer + BIG_BUFFER_SIZE = %i \n", DataBuffer + BIG_BUFFER_SIZE);
        printf("DataBuffer = %i \n", DataBuffer + 0 );
        printf("BIG_BUFFER_SIZE = %i \n\n", BIG_BUFFER_SIZE );


        //~~~~~~~~~~~~ ORIGINAL METHOD  ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

        //Step 1: Copy the unprocessed bytes to the begining of the buffer
        bytes_remaining   =  CurrentDataPtr - LastProcBytePtr;
        memcpy(DataBuffer, LastProcBytePtr, bytes_remaining);
        //Step 2: Make the pointer point to the begining of the buffer
        // We assume this information has already been processed
        lpBufPtr            = DataBuffer;
        LastProcBytePtr     = DataBuffer;
        invresetpointer     = DataBuffer;
        old_LastProcBytePtr = DataBuffer;

        CurrentDataPtr      = DataBuffer + bytes_remaining;
        printf( "\n\nBuffer Full \n Relocating unprocessed data \n ");

        FILE *BER_File;
        BER_File = fopen(BERFileName,"a");
        if (BER_File==NULL){
            printf("Could Not Open BER log file name\n");
            exit;
        }else{

            fprintf(BER_File,"Buffer Full \t Relocating unprocessed data\n Temp.Buffer Size %i\t Total Buf.Size=%i Bytes Transferred%i\n" ,
                    TMP_BUFFER_SIZE,
                    BIG_BUFFER_SIZE,
                    bytes_remaining);
            fclose(BER_File);
        }
    }

    memcpy(CurrentDataPtr, ReadBuffer, next_block_size*sizeof(unsigned char));
    CurrentDataPtr += next_block_size;

#ifdef RE_ACQ
    ReAcquire = 0;
    while ((CurrentDataPtr - LastProcBytePtr >= MIN_DATA_SIZE)&(!ReAcquire)){

        ReAcquire = LRIT_Main(resynch_thresh, BERFileName);
    }
    // Process exits when LRIT_Main() returns with an error
    if (!ReAcquire){
        return TRUE;
    }else{
        return FALSE;
    }
#else

    while (CurrentDataPtr - LastProcBytePtr >= MIN_DATA_SIZE)
        LRIT_Main(resynch_thresh, BERFileName);


    return TRUE;
#endif

}

BOOL CGoesProcessor::ProcessEmwinDataBlock(unsigned char *ReadBuffer, int next_block_size, int resynch_thresh, char * BERFileName)
{
    int bytes_remaining;
    if (CurrentDataPtr + next_block_size > DataBuffer + BIG_BUFFER_SIZE)
    {
        printf( "\n\nBuffer Full \n Relocating unprocessed data \n ");

        bytes_remaining = CurrentDataPtr - LastProcBytePtr;
        memcpy(DataBuffer, LastProcBytePtr, bytes_remaining);
        lpBufPtr = DataBuffer;
        LastProcBytePtr = DataBuffer;
        invresetpointer = DataBuffer;
        old_LastProcBytePtr = DataBuffer;
        CurrentDataPtr = DataBuffer + bytes_remaining;
    }

    memcpy(CurrentDataPtr, ReadBuffer, next_block_size);
    CurrentDataPtr += next_block_size;
  
/*#ifdef RE_ACQ
    ReAcquire = 0;
    while ((CurrentDataPtr - LastProcBytePtr >= MIN_DATA_SIZE)&(!ReAcquire)){
        ReAcquire = LRIT_Main(resynch_thresh, BERFileName);
    }
    if (!ReAcquire){
        return TRUE;
    }else{
        return FALSE;
    }
#else*/
    while (CurrentDataPtr - LastProcBytePtr >= MIN_DATA_SIZE)
        EMWIN_Main(resynch_thresh, BERFileName);
    return TRUE;    
//#endif
}

int CGoesProcessor::LRIT_Main(int resynch_thresh, char * BERFileName)
{

    UCHAR           hreg;
    int             offset;
    int             firstheaderpointer;
    BOOL            head = FALSE;
    int             bytestowrite = 0;
    char            xcor;
    unsigned int    i, j, k, m, n, ercount, ercountmax = 0;
    ULONG           synk;
    int             syncsum, syncthresh;
    WORD            temword;
    FILE            *BER_File;
    int SEARCH_DEPTH = 5;  //how far you can search for a header in bytes, before downgrading lock status
    int CONSECUTIVE_GOOD_THRESHOLD = 3; //How many consecutive frames I need to receiver in order to be in the highest lock state.
    int CONSECUTIVE_BAD_THRESHOLD = 3;

    int resynch_thresh3 = resynch_thresh;  //Found more than CONSECUTIVE THRESHOLD Frames in the Exact positions you expect to find headers
    int resynch_thresh2 = 6; //Had one missed frame or frame in error but looking in areas where you expect to find headers.
    int resynch_thresh1 = 2; //Looking for header in the middle of a packet
    if (resynch_thresh2 > resynch_thresh3)
        resynch_thresh2 = resynch_thresh3;

    unsigned char LritSocketFrame[1024];
    //===============================================
    // Eliminate GOTOs
    bool getAPID= false;
    bool BadData= false;

    //===============================================
    if ((bad_data_count %1024) ==1){
        //printf("LRIT_Main:: bad_data_count=%d \n",bad_data_count);
        printf("LRIT_Main::Detecting Unsynched Data=%d \r",(int)bad_data_count/1024);
    }

    //Process Every Byte that is stored in memory for processing
    if ((CurrentDataPtr - LastProcBytePtr) >= MIN_DATA_SIZE)  
    {
        if (LRITFirstsync)
        {
            for (i = 0; i < 4; i++)
                Ltreg = (Ltreg << 8) | ((*(LastProcBytePtr + i)) ^ LRITDataInvert);
            LastProcBytePtr += 4;
            LRITFirstsync = FALSE;
        }

        //LRITDataInvert is used to resolve the 180 degree phase ambiguity. 
        hreg = *LastProcBytePtr ^ LRITDataInvert;

        //Maintain byte alignment by checking all 8 bits within the byte for the header
        for (offset = 0; offset <= 7; offset++)  
        {
            //LRITsync = CADU Header = 32 bits
            //Ltreg is the 32 bit section of LRIT data being compared against the header
            synk = lritsync ^ Ltreg;    // Bit-wise XOR

            // If the header is found, then sync=32 "0" bits
            syncsum = 0;
            for (n = 0; n < 32; n++)
            {
                if (synk & 0x00000001)
                    syncsum++;
                synk = synk >> 1;
            }

            if ((consecutive_frames_missed + consecutive_frame_errors) > CONSECUTIVE_BAD_THRESHOLD){
                if (lock_status != -1)
                    printf("Lock Status Changed From %d to %d\n", lock_status, -1);
                lock_status = -1;
            }

            if (lock_status==1)
                syncthresh = resynch_thresh1;
            else if (lock_status==2)
                syncthresh = resynch_thresh2;
            else if (lock_status>=3 && (offset!=best_offset)) //Wrong expected bit alignment within bytesteam
                syncthresh = resynch_thresh2;
            else if (lock_status>=3 && (offset==best_offset)) //Correct expected bit alignment within bytesteam
                syncthresh = resynch_thresh3;
            else if (lock_status<0)
                syncthresh = 0;

            if (syncsum <= syncthresh)	 // if 32-bit header was found
            {
                FrameSyncCnt ++;
                best_offset_ready_signal = 1; //This is to allows you to estimate and print the syncsum for the undetected headers.
                if (consecutive_good_frames >= CONSECUTIVE_GOOD_THRESHOLD){
                    if (lock_status != 3)
                        //printf("Lock Status Changed From %d to %d\n", lock_status, 3);
                        lock_status = 3;  //upgrade lock status directly to 3
                    best_offset = offset;
                }

                InvByteCount = 0;
                InvBadFrameCount = 0;
                invresetpointer = LastProcBytePtr;
                foundsyncflag = TRUE;
                LRITFirstsync = TRUE;
                if (!LRITsyncflag)
                    LRITsyncflag = TRUE;

                if (!Firstsyncflag){
                    processbytes = (int)(LastProcBytePtr - old_LastProcBytePtr);
                    if (processbytes > 1024)
                        LRITsyncflag = FALSE;
                }else{
                    Firstsyncflag = FALSE;
                }
                old_LastProcBytePtr = LastProcBytePtr;

                //De-randomize the data
                for (j = 0; j < 1024; j++)
                {
                    LRITframe[j] = (*(LastProcBytePtr + j)) ^ LRITDataInvert;
                    if (offset != 0)
                    {
                        temword = (*(LastProcBytePtr + j)) ^ LRITDataInvert;
                        temword = (temword << 8) | (*(LastProcBytePtr + j + 1)) ^ LRITDataInvert;
                        LRITframe[j] = temword >> (8 - offset);
                    }
                    LRITframe[j] ^= Random[j];
                }

                LastProcBytePtr += 1020;  //CADU packet length is 8160bits = 1020 bytes
                LRITFrameCount++;

                //Make a copy of the CADU frame before Reed Solomon Decoding
                for (k = 0; k < 1024; k++)
                    LRIT1frame[k] = LRITframe[k];

                //RS Decode
                LRITrs->Decode((unsigned char *) &LRITframe[0]);
                viterbi_framerrors = LRITrs->framerrors;  
                RS_framerrors = LRITrs->uncorrected_framerrors;
                
                //Compare the LRIT Frame Before and after RS decoding to estimate the
                //Error Rate at the Output of Viterbi Decoding
                ercount=0;
                for (k = 0; k < 1020; k++)
                {
                    xcor = LRIT1frame[k] ^ LRITframe[k];
                    for (m = 0; m < 8; m++)
                    {
                        if ((xcor & 0x01) == 0x01) {
                            ercount++;
                            //ErrCount ++;
                        }
                        xcor = xcor >> 1;
                    }
                }
                ErCountCum = ErCountCum + ercount;  

                //If there is a Reed Solomon error, update error counters
                //Change Frame Synchronizatio Status
                // Try and recover the VCDU counter so you can detect a missed frame
                if (RS_framerrors>0)  
                {
                    LRITFrameError+=RS_framerrors;
                    consecutive_frame_errors++;
                    consecutive_good_frames = 0;
                    VCID = LRITframe[1] & 0x3f;
                    VCDUcount = LRITframe[2];
                    VCDUcount = (VCDUcount << 8) | LRITframe[3];
                    VCDUcount = (VCDUcount << 8) | LRITframe[4];
                    //printf(" \n Frame %d has Reed Solomon Error. Est. VCDU=%d Syncsum%d\n" ,LRITFrameCount,VCDUcount,syncsum);
                    if (lock_status==3){ //Strictly downgrade from 3 to 2 only.
                        //printf("Lock Status Changed From %d to %d\n", 3, 2);
                        lock_status=2;
                    }
                    return 0;
                }
                //The rest of code in [if(syncsun<sync_thresh)] represents a frame synch and a good received frame


                if (lock_status<2){ //If found and synched to good frame, upgrade from 0 or 1, up to 2.
                    //printf("Lock Status Changed From %d to %d\n", lock_status, 2);
                    lock_status = 2;
                }
                consecutive_frames_missed = 0;
                consecutive_good_frames++;

                //Get the VCID Market
                //Compare to to last VCID marker to make sure a frame wasn't missed
                VCID        = LRITframe[1] & 0x3f;
                VCDUcount   = LRITframe[2];
                VCDUcount   = (VCDUcount << 8) | LRITframe[3];
                VCDUcount   = (VCDUcount << 8) | LRITframe[4];
                VCDUmark1   = VCDUmark0;
                VCDUmark0   = VCDUcount;

                //Every 50 LRIT frames received, print status to screen. 
                if ((LRITFrameCount %50)==0 || syncsum>=8){
                    printf("RS FER=%g   Vit.BER=%4.3e  Viterbi Errs= %d  RS Frame Errors=%d Missed VCDU Frames= %d Total VCDU Frames=%d   \n",
                           ((double)LRITFrameError+(double)Missed_FrameCount) /((double)LRITFrameCount*4.0),
                           (double)ErCountCum/(LRITFrameCount*8192.0),
                           ErCountCum,
                           LRITFrameError,
                           Missed_FrameCount,
                           LRITFrameCount
                          );

                    //Every 500 LRIT frames, write status to a log file
                    if ((LRITFrameCount %500)==0){

                        BER_File = fopen(BERFileName,"a");
                        if (BER_File==NULL){
                            printf("Could Not Open BER log file name\n");
                            exit;
                        }else{
                            // During PLT testing, we modified the way we used to compute frame errors
                            fprintf(BER_File,"RS FER=%4.3e \t Vit.BER= %4.3e \tViterbi Errs= %d\tRS Frame Errors=%d Missed VCDU Frames= %d Total VCDU Frames=%d \n",
                                    ((double)LRITFrameError+(double)Missed_FrameCount) /((double)LRITFrameCount*4.0),
                                    (double)ErCountCum/(LRITFrameCount*8192.0),
                                    ErCountCum,
                                    LRITFrameError,
                                    Missed_FrameCount,
                                    LRITFrameCount );
                            fclose(BER_File);
                        }
                    }

                }

                //IF the VCDU markers do not agree, declare a missed frame.  
                //If there is a large burst of missed frames (more than 20), then use an alternate 
                //method to count missed frames. 
                if (((VCDUmark0 - VCDUmark1) > 1) && ((VCDUmark0 - VCDUmark1) <= 20)){
                    if (!MissedFrameInitialState){//EV
                        if (((VCDUmark0 - VCDUmark1)-1 - consecutive_frame_errors) > 0){
                            Missed_FrameCount = Missed_FrameCount + (VCDUmark0 - VCDUmark1)-1 - consecutive_frame_errors;
                            //printf("Missed %d VCDU Frames\t VCDUmark0 = %d\t VCDUmark1 = %d\n",(VCDUmark0 - VCDUmark1)-1, VCDUmark0,VCDUmark1);
                            //Don't double count frame misses and frame errors
                        }
                    }
                    else{
                        MissedFrameInitialState = FALSE;
                    }
                }

                
                //This alternate method counts how many bytes you processed without finding a frame synch flag
                else if ((this->bad_data_count >0) && ((VCDUmark0 - VCDUmark1) > 20)){
                    if (!MissedFrameInitialState){//EV
                        if ((int((double)this->bad_data_count/1020.0)- consecutive_frame_errors)>0){
                            Missed_FrameCount = Missed_FrameCount + int((double)this->bad_data_count/1020.0)- consecutive_frame_errors;
                            // printf("Missed %d Bad data Frames\n",int((double)this->bad_data_count/1020.0));
                            //Don't double count frame misses and frame errors;
                        }
                    }
                    else{
                        MissedFrameInitialState = FALSE;
                    }
                }
                consecutive_frame_errors = 0;
                this->bad_data_count=0;  //Reset this to zero since this is the if(syncsum <= syncthesh) code block.

                if (VCID == 63){
                    //printf("VDIC=63    RETURN \n");
                    return 0;
                }

                if (USE_SOCKET_LRIT_GBL>0) //Write LRIT BPDU FRame to a Socket including VCDU header
                {
                    memcpy(LritSocketFrame, LRITframe, 892); //Need to make an LRIT socket
                    send(new_connection_lrit, (char *)LritSocketFrame, 892, 0);
                }


                if (VCIDtable[VCID][8] == 0)
                    VCIDtable[VCID][8] = VCDUcount;
                else
                {
                    if (VCDUcount != VCIDtable[VCID][8] + 1)
                    {// Skipped a packet
                        if (VCIDtable[VCID][2] != 2049)
                            APIDtable[VCIDtable[VCID][2]] = 2040;
                        VCIDtable[VCID][0] = 0;
                        VCIDtable[VCID][1] = 0;
                        VCIDtable[VCID][2] = 2049;  // OUT OF RANGE
                        VCIDtable[VCID][3] = 2047;  // Packet contains data from previous block
                        VCIDtable[VCID][4] = 0;
                        VCIDtable[VCID][5] = 0;
                        VCIDtable[VCID][6] = 0;
                        VCIDtable[VCID][7] = 0;
                    }
                    VCIDtable[VCID][8] = VCDUcount;
                }


                Fullheader = VCIDtable[VCID][0];
                Extension = VCIDtable[VCID][1];
                APIDindex = VCIDtable[VCID][2];
                PendingAPID = VCIDtable[VCID][3];
                sequenceflag = VCIDtable[VCID][4];
                APID = VCIDtable[VCID][7];
                head = FALSE;
                firstheaderpointer = LRITframe[6];
                firstheaderpointer = ((firstheaderpointer << 8) | LRITframe[7]) & 0x7ff;
                framepointer = 8;
                remainingframebytes = 884;

                if (firstheaderpointer == 2047)
                {
                    if (PendingAPID == 2047){
                        VCIDtable[VCID][0] = 0;
                        VCIDtable[VCID][1] = 0;
                        VCIDtable[VCID][2] = 2049;//Set APID index out of range
                        VCIDtable[VCID][3] = 2047;//Set Pending Apid=2047
                        VCIDtable[VCID][4] = 0;
                        VCIDtable[VCID][5] = 0;
                        VCIDtable[VCID][6] = 0;
                        VCIDtable[VCID][7] = 0;
                        return 0;
                    }

                    if ((!Fullheader) && Extension)
                    {
                        GetLritPacketHeader();
                        if (APID == 2047){ // Pending Apid=2047
                            if (PacketLength < remainingframebytes)
                            {
                                framepointer += PacketLength;
                                remainingframebytes -= PacketLength;
                                //goto getapid;
                                getAPID = true;
                            }else if (PacketLength == remainingframebytes) {
                                VCIDtable[VCID][1] = 0;
                                return 0;
                            }
                            if (getAPID==false){
                                VCIDtable[VCID][1] = 1;
                                VCIDtable[VCID][3] = 2047;
                                Bytesremaining[2047] = PacketLength - remainingframebytes;
                                return 0;
                            }
                        }
                        if (getAPID==false){
                            PendingAPID = VCIDtable[VCID][2];
                            head = TRUE;
                        }
                    }

                    if (getAPID==false){
                        if (!Extension)
                            return 0;

                        APIDindex = PendingAPID;
                        if (Bytesremaining[APIDindex] < 884 - (framepointer - 8))
                            return 0;
                        if (head)
                        {
                            if (((TMP_BUFFER_SIZE - (LRITFile[APIDindex] - LRITFilePtr[APIDindex])) < 6) || (APIDindex > 14)){
                                printf( "TMP Buffer Overload or invalid APIDindex=%i \n",APIDindex);
                                return 0;
                            }
                            memcpy(LRITFilePtr[APIDindex], &LRIThead[0], 6);
                            LRITFilePtr[APIDindex] += 6;
                        }
                        if (((TMP_BUFFER_SIZE - (LRITFile[APIDindex] - LRITFilePtr[APIDindex])) < (884 - (framepointer - 8))) || (APIDindex > 14)){
                            printf( "TMP Buffer Overload or invalid APIDindex=%i \n",APIDindex);
                            return 0;
                        }
                        memcpy(LRITFilePtr[APIDindex], &LRITframe[framepointer], 884 - (framepointer - 8));
                        LRITFilePtr[APIDindex] += 884 - (framepointer - 8);
                        Bytesremaining[APIDindex] -= (884 - (framepointer - 8));
                        return 0;
                    }
                }

                if ((firstheaderpointer > 0)||(!getAPID)){
                    if ((!Fullheader) && Extension)
                    {
                        GetLritPacketHeader();
                        PendingAPID = VCIDtable[VCID][2];
                        head = TRUE;
                    }

                    if (Extension) {
                        APIDindex = PendingAPID;
                        if (head) {
                            if (((TMP_BUFFER_SIZE - (LRITFile[APIDindex] - LRITFilePtr[APIDindex])) < 6)||(APIDindex > 14)){
                                printf( "TMP Buffer Overload or invalid APIDindex=%i \n",APIDindex);
                                return 0;
                            }
                            memcpy(LRITFilePtr[APIDindex], &LRIThead[0], 6);
                            LRITFilePtr[APIDindex] += 6;
                        }

                        if (((TMP_BUFFER_SIZE - (LRITFile[APIDindex] - LRITFilePtr[APIDindex])) < Bytesremaining[APIDindex]) || (APIDindex > 14)){
                            printf( "TMP Buffer Overload or invalid APIDindex=%i \n",APIDindex);
                            return 0;
                        }

                        memcpy(LRITFilePtr[APIDindex], &LRITframe[framepointer], Bytesremaining[APIDindex]);
                        LRITFilePtr[APIDindex] += Bytesremaining[APIDindex];

                        if ((sequenceflag == 3) || (sequenceflag == 2)){
                            ProcessLRITfile(LRITFilePtr[APIDindex] - LRITFile[APIDindex]);
                            LRITFilePtr[APIDindex] = LRITFile[APIDindex];
                            APIDtable[APIDindex] = 2040;
                            filesequence[APID]++;
                        }
                        remainingframebytes -= Bytesremaining[APIDindex];
                        framepointer += Bytesremaining[APIDindex];
                        //goto getapid;
                    } else{
                        framepointer += firstheaderpointer;
                        remainingframebytes -= firstheaderpointer;
                        //goto getapid;
                    }
                    getAPID = true; // It's enable for both cases of (Extension)
                }

                //if (firstheaderpointer == 0)
                if ((firstheaderpointer == 0) ||(getAPID) ){ //will have a complete packet header available
                    //getapid:
                    getAPID = true;

                    while ( getAPID == true){
                        //0-Fullheader,1-extension,2-APIDindex,3-pendingapid,4-sequence,5-packetindex,6-packetlength
                        if (!GetLritPacketHeader())
                            return 0;

                        if (APID == 2047){//2047=Fill APID - ignore
                            if (PacketLength < remainingframebytes) {
                                framepointer += PacketLength;
                                remainingframebytes -= PacketLength;
                                getAPID = true;
                                //goto getapid;
                            }else{
                                //getAPID = false;
                                if (PacketLength == remainingframebytes){
                                    VCIDtable[VCID][1] = 0; //Extension=FALSE;
                                    return 0;
                                }
                                VCIDtable[VCID][1]  = 1;//Extension=TRUE;
                                VCIDtable[VCID][3]  = 2047;//PendingAPID=2047;
                                Bytesremaining[2047] = PacketLength - remainingframebytes;
                                return 0;
                            }
                        }else{
                            getAPID = false;
                        }

                        if (getAPID == false){
                            ////Add Bytes to File///
                            if (Bytesremaining[APIDindex] <= remainingframebytes) {
                                bytestowrite = Bytesremaining[APIDindex];
                                VCIDtable[VCID][1] = 0;
                            } else{
                                bytestowrite = remainingframebytes;
                                Bytesremaining[APIDindex] -= remainingframebytes;
                                PendingAPID = VCIDtable[VCID][2];
                                VCIDtable[VCID][3] = PendingAPID;
                                VCIDtable[VCID][1] = 1;
                            }
                            if (((TMP_BUFFER_SIZE - (LRITFile[APIDindex] - LRITFilePtr[APIDindex])) < bytestowrite + 6) || (APIDindex > 14)){
                                printf( "\n\n TMP Buffer Overload or invalid APIDindex=%i \n\n",APIDindex);
                                return 0;
                            }
                            //EV This is where LRIT usually crashed
                            //EV Add
                            if (APIDindex<NUMBER_OF_FILES){ //this doesn't fix it
                                memcpy(LRITFilePtr[APIDindex], &LRITframe[framepointer - 6] ,bytestowrite + 6);
                            }else{
                                printf("APID Index= %i exceeds array size (%i) \n EXIT \n",NUMBER_OF_FILES,APIDindex);
                                return 0;
                            }
                            LRITFilePtr[APIDindex] += bytestowrite + 6;
                            remainingframebytes -= bytestowrite;
                            framepointer += bytestowrite;
                            if (VCIDtable[VCID][1])
                                return 0;
                            if ((sequenceflag == 3) || (sequenceflag == 2)){
                                ProcessLRITfile(LRITFilePtr[APIDindex] - LRITFile[APIDindex]);
                                APIDtable[APIDindex] = 2040;
                                filesequence[APID]++;
                                LRITFilePtr[APIDindex] = LRITFile[APIDindex];
                            }
                            if (remainingframebytes <= 0){
                                return 0;
                            }
                            //goto getapid;
                            getAPID = true;
                        }
                    }
                }
            }

            //At the end an iteration of the bit offset loop
            //If this portion of code is reached, that means that no header was found during 
            //this particular bit marker 


            //Estimate and print what the missed synch header was.  Its not perfect, so disregard outliers reported
            if ((syncsum > syncthresh) && (offset==best_offset) && (best_offset_ready_signal==1))
            {
                // printf("Synched ? %d\n",NoSyncFound==0);
                printf("Header Errors Exceeded Threshold\tSync Sum = (%d / %d ) \tBit Offset=%d  \n",syncsum,syncthresh, best_offset);
                best_offset_ready_signal=0;
            }

//Update, 32 bit sequence to search Continue searching for frame sync...bit by bit
            Ltreg = Ltreg << 1; 
            if (hreg & 0x80)
                Ltreg++;
            hreg = hreg << 1;
        }

       //Checked every bit offset within for a given byte marker without finding a header
     
        LastProcBytePtr++; //No sync...get next byte, and search every offset for the header
        InvByteCount++;  //Update the total number of bytes processed without finding a header
        consecutive_good_frames = 0; 


        //If you are near the next frame header, relax the threshold
        if (((InvByteCount >= (1024-SEARCH_DEPTH)) && (InvByteCount <= (1024+SEARCH_DEPTH))) && ((consecutive_frames_missed+consecutive_frame_errors) < 2)){
            if (lock_status == 1){
                lock_status = 2;
            }
        }
        //If you are searching for a header in the middle of a packet, downgrade the lock
        //status 
        else if (lock_status>1){
            lock_status = 1;
        }

        BadData=false;
        if (InvByteCount > 1024){ //Checked all 1024 bytes, still no header found
            if (InvToggleFlag){
                LRITDataInvert = LRITDataInvert ^ 0xff;
                InvToggleFlag = FALSE;
                InvByteCount = 0;
                invresetpointer = LastProcBytePtr;
                InvBadFrameCount = 0;
                BadData = true;

            }
            if ((foundsyncflag)&&(!BadData)) {
                InvBadFrameCount++;
                if (InvBadFrameCount >= 3)
                {
                    LastProcBytePtr = invresetpointer;
                    LRITDataInvert = LRITDataInvert ^ 0xff;
                    InvToggleFlag = TRUE;
                    InvBadFrameCount = 0;
                    foundsyncflag = FALSE;
                }
                BadData = true;
            }
            if (!BadData){
                LastProcBytePtr = invresetpointer;
                LRITDataInvert = LRITDataInvert ^ 0xff;
                InvToggleFlag = TRUE;
                InvByteCount = 0;
            }
        }
        BadData=true;

#ifdef RE_ACQ

        if ((this->bad_data_count > (NO_SYNC_LIMIT)) ){
            printf(" GoesProcessor: Lost Synch \n");
            Missed_FrameCount =0;
            this->bad_data_count =0;
            return 1;  // "1" means Return= Reacquire . Normal exit is "0"
        }
#endif

        InvToggleFlag = InvToggleFlag;
        this->bad_data_count++;
        consecutive_frames_missed = consecutive_frames_missed + int((double)this->bad_data_count/1020.0);
    }

    return 0;
}

int CGoesProcessor::EMWIN_Main(int resynch_thresh, char * BERFileName)
{
    UCHAR hreg;
    char tempstr1[200];
    int offset;
    int EndofEmwinDataPtr;
    BOOL head = FALSE;
    int bytestowrite = 0;
    char xcor;
    unsigned int i, j, k, m, n, ercount, ercountmax = 0;
    int LRcount;
    ULONG synk;
    int syncsum,syncthresh;
    WORD temword;
    FILE *BER_File;
    int SEARCH_DEPTH = 5;  //how far you can search for a header in bytes, before downgrading lock status
    int CONSECUTIVE_GOOD_THRESHOLD = 3; //How many consecutive frames I need to receiver in order to be in the highest lock state.
    int CONSECUTIVE_BAD_THRESHOLD = 3;
    int resynch_thresh3 = resynch_thresh;  //Found more than CONSECUTIVE THRESHOLD Frames in the Exact positions you expect to find headers
    int resynch_thresh2 = 6; //Had one missed frame or frame in error but looking in areas where you expect to find headers.
    int resynch_thresh1 = 2; //Looking for header in the middle of a packet
    if (resynch_thresh2 > resynch_thresh3)
        resynch_thresh2 = resynch_thresh3;

    unsigned char EmwinSocketFrame[1000];
    //=======================================
    // Eliminate GOTOs
    bool BadData= false;
    //=======================================
    //                   EMWIN-OQPSK Coded VCDU
    //
    // ASM          VCDU         BPDU           BPDU Data Field   Reed-Solomon
    //0x1ACFFC1D	Header	     Header	               	          Check Symbols
    // 4 octets    6 octets      2 octets       884 octets         128 octets
    //<<<----------------------------  1024 bytes      ---------------------->>

    if ((CurrentDataPtr - LastProcBytePtr) >= MIN_DATA_SIZE)
    {
        //--------------------
        //      ASM HEADER
        //--------------------
        if (LRITFirstsync)
        {   //   ^ = OR
            // LRITDataInvert ; Alternates between all 0's, all 1's
            // Ltreg = FFFF FFFFxh
            for (i = 0; i < 4; i++)
                Ltreg = (Ltreg << 8) | ((*(LastProcBytePtr + i)) ^ LRITDataInvert);
            LastProcBytePtr += 4;
            LRITFirstsync = FALSE;
        }

        hreg = *LastProcBytePtr ^ LRITDataInvert;
        for (offset = 0; offset <= 7; offset++)
        {
            synk = lritsync ^ Ltreg;
            //-----------------------------------------------------
            // Bitwise comparison of lritsync  and Ltreg= rec.data
            // if they are both equal, then the XOR operation results in
            // synk ==0, which later makes suncsum =0, which implies that the
            // header was found

            syncsum = 0;
            for (n = 0; n < 32; n++)
            {
                if (synk & 0x00000001)
                    syncsum++;
                synk = synk >> 1;
            }

            if ((consecutive_frames_missed + consecutive_frame_errors) > CONSECUTIVE_BAD_THRESHOLD){
                if (lock_status != -1)
                    printf("Lock Status Changed From %d to %d\n", lock_status, -1);
                lock_status = -1;
            }

            if (lock_status==1)
                syncthresh = resynch_thresh1;
            else if (lock_status==2)
                syncthresh = resynch_thresh2;
            else if (lock_status>=3 && (offset!=best_offset)) //Wrong expected bit alignment within bytesteam
                syncthresh = resynch_thresh2;
            else if (lock_status>=3 && (offset==best_offset)) //Correct expected bit alignment within bytesteam
                syncthresh = resynch_thresh3;
            else if (lock_status<0)
                syncthresh = 0;

            if (syncsum <= syncthresh){
                //--------------------
                //  FOUND  ASM HEADER
                FrameSyncCnt ++;
                best_offset_ready_signal = 1; //This is to allows you to estimate and print the syncsum for the undetected headers.
                if (consecutive_good_frames >= CONSECUTIVE_GOOD_THRESHOLD){
                    if (lock_status != 3)
                        //printf("Lock Status Changed From %d to %d\n", lock_status, 3);
                        lock_status = 3;  //upgrade lock status directly to 3
                    best_offset = offset;
                }


                InvByteCount = 0;
                InvBadFrameCount = 0;
                invresetpointer = LastProcBytePtr;
                foundsyncflag = TRUE;
                LRITFirstsync = TRUE;
                if (!LRITsyncflag)
                    LRITsyncflag = TRUE;

                if (!Firstsyncflag)
                {
                    processbytes = (int)(LastProcBytePtr - old_LastProcBytePtr);
                    if (processbytes > 1024)
                        LRITsyncflag = FALSE;
                }
                else
                    Firstsyncflag = FALSE;

                old_LastProcBytePtr = LastProcBytePtr;

                //Randomizer: Frame size
                for (j = 0; j < 1024; j++)
                {
                    LRITframe[j] = (*(LastProcBytePtr + j)) ^ LRITDataInvert;
                    if (offset != 0)
                    {
                        temword = (*(LastProcBytePtr + j)) ^ LRITDataInvert;
                        temword = (temword << 8) | (*(LastProcBytePtr + j + 1)) ^ LRITDataInvert;
                        LRITframe[j] = temword >> (8 - offset);
                    }
                    LRITframe[j] ^= Random[j];
                }

                LastProcBytePtr += 1020;
                EMWINFrameCount++;

                for (k = 0; k < 1024; k++)
                    LRIT1frame[k] = LRITframe[k];

                //RS Decode
                LRITrs->Decode((unsigned char *) &LRITframe[0]);
                viterbi_framerrors = LRITrs->framerrors;
                RS_framerrors=-100;
                RS_framerrors = LRITrs->uncorrected_framerrors;
                ercount=0;
                for (k = 0; k < 1020; k++)
                {
                    xcor = LRIT1frame[k] ^ LRITframe[k];
                    for (m = 0; m < 8; m++)
                    {
                        if ((xcor & 0x01) == 0x01)
                        {
                            ercount++;
                        }
                        xcor = xcor >> 1;
                    }
                }
                ErCountCum = ErCountCum + ercount; 

                //If there is a Reed Solomon error, update error counters
                //Change Frame Synchronizatio Status
                // Try and recover the VCDU counter so you can detect a missed frame
                if (RS_framerrors>0) 
                {
                    EMWINFrameError+= RS_framerrors;
                    consecutive_frame_errors++;
                    consecutive_good_frames = 0;
                    VCID = LRITframe[1] & 0x3f;
                    VCDUcount = LRITframe[2];
                    VCDUcount = (VCDUcount << 8) | LRITframe[3];
                    VCDUcount = (VCDUcount << 8) | LRITframe[4];
                    //printf(" \n Frame %d has Reed Solomon Error. Est. VCDU=%d Syncsum%d\n" ,LRITFrameCount,VCDUcount,syncsum);
                    if (lock_status==3){ //Strictly downgrade from 3 to 2 only.
                        //printf("Lock Status Changed From %d to %d\n", 3, 2);
                        lock_status=2;
                    }
                    return 0;
                }
                //The rest of code in [if(syncsun<sync_thresh)] represents a frame synch and a good received frame

                consecutive_frames_missed = 0;
                if (lock_status<2){ //If found a and synched to good from, upgrade from 0 or 1, up to 2.
                    //printf("Lock Status Changed From %d to %d\n", lock_status, 2);
                    lock_status = 2;
                }

                /*
                if (USE_SOCKET_GBL>0){ //Write LRIT VCDU FRame to a Socket
                    send(new_connection, (char *)LRITframe, 1024, 0);
                }*/

                consecutive_good_frames++;

                //Table 2 2 VCDU Header Structure
                //Version	        VCDU-ID	       VCDU	     Signaling Field
                //Number	    SCID	VCID	 Counter	Replay Flag	  Spare
                //2 bits      8 bits   6 bits    24 bits    1 bit         7 bits

                // 1st Byte = 2 bits from Version Number, 6 from SCID
                // 2nd Byte = 2 bits from SCID, all 6 bits from VCID
                VCID = LRITframe[1] & 0x3f;     // Grab 6 bits of 8
                // 3rd byte = VCDU Counter
                VCDUcount = LRITframe[2];
                VCDUcount = (VCDUcount << 8) | LRITframe[3];
                VCDUcount = (VCDUcount << 8) | LRITframe[4];
                VCDUmark1= VCDUmark0;
                VCDUmark0 = VCDUcount;

                if ((EMWINFrameCount % 20)==0 || syncsum>=8){
                    /*printf("FER=%g\tVit.BER = % g\tViterbi Errs= %d\tFrame Errors=%d Missed Frames= %d Total Frames=%d VCDU=%d SyncSum=%d\n",
                        ((double)EMWINFrameError+(double)Missed_FrameCount) /(double)EMWINFrameCount, (double)ErCountCum/(double)(EMWINFrameCount*8192), ErCountCum,
                        EMWINFrameError, Missed_FrameCount, EMWINFrameCount,VCDUcount,syncsum);
                        */
                    printf("RS FER=%g  Vit.BER= %4.3e  Viterbi Errs= %d  RS Frame Errors=%d Missed VCDU Frames= %d Total VCDU Frames=%d\n",
                           ((double)EMWINFrameError+((double)Missed_FrameCount)*4.0) /((double)EMWINFrameCount*4.0), (double)ErCountCum/(double)(EMWINFrameCount*8192), ErCountCum,
                           EMWINFrameError, Missed_FrameCount, EMWINFrameCount);

                    if ((EMWINFrameCount %500)==0){
                        BER_File = fopen(BERFileName,"a");
                        if (BER_File==NULL){
                            printf("Could Not Open BER log file name\n");
                            exit;
                        }else{
                            fprintf(BER_File, "RS FER=%4.3e \t Vit.BER=%4.3e \tViterbi Errs= %d\tRS Frame Errors=%d Missed VCDU Frames= %d Total VCDU Frames=%d \n",
                                    ((double)EMWINFrameError+((double)Missed_FrameCount)*4.0) /((double)EMWINFrameCount*4.0), (double)ErCountCum/(double)(EMWINFrameCount*8192), ErCountCum,
                                    EMWINFrameError, Missed_FrameCount, EMWINFrameCount);

                            fclose(BER_File);
                        }
                    }
                }


                //IF the VCDU markers do not agree, declare a missed frame.  
                //If there is a large burst of missed frames (more than 20), then use an alternate 
                //method to count missed frames. 
                if (((VCDUmark0 - VCDUmark1) > 1) && ((VCDUmark0 - VCDUmark1) <= 20)){
                    if (!MissedFrameInitialState){//EV
                        if (((VCDUmark0 - VCDUmark1)-1 - consecutive_frame_errors) > 0){
                            Missed_FrameCount = Missed_FrameCount + (VCDUmark0 - VCDUmark1)-1 - consecutive_frame_errors;
                            //printf("Missed %d VCDU Frames\t VCDUmark0 = %d\t VCDUmark1 = %d\n",(VCDUmark0 - VCDUmark1)-1, VCDUmark0,VCDUmark1);
                            //Don't double count frame misses and frame errors
                        }
                    }
                    else{
                        MissedFrameInitialState = FALSE;
                    }
                }

               //This alternate method counts how many bytes you processed without finding a frame synch flag
                else if ((this->bad_data_count >0) && ((VCDUmark0 - VCDUmark1) > 20)){
                    if (!MissedFrameInitialState){//EV
                        if ((int((double)this->bad_data_count/1020.0)- consecutive_frame_errors)>0){
                            Missed_FrameCount = Missed_FrameCount + int((double)this->bad_data_count/1020.0)- consecutive_frame_errors;
                            // printf("Missed %d Bad data Frames\n",int((double)this->bad_data_count/1020.0));
                            //Don't double count frame misses and frame errors;
                        }
                    }
                    else{
                        MissedFrameInitialState = FALSE;
                    }
                }

                consecutive_frame_errors = 0;
                this->bad_data_count=0;  //Reset this to zero since this is the if(syncsum <= syncthesh) code block.


                // VCID	Binary 001111 specifying channel 15
                
                if (VCID != 15) 
                    return 0;

                // BPDU Data Field = 884 octets = 7071 bits
                EndofEmwinDataPtr = LRITframe[6] & 0x3f;
                //BUG:: The spec eludes to using 0x07, but this causes errors!  0x3f works better for some reason?!

                EndofEmwinDataPtr = (EndofEmwinDataPtr << 8) | LRITframe[7];//JR [Grab remaining 8 bits (in the 11 bit Header Pointer field)
                if ((EndofEmwinDataPtr == 16383) || (EndofEmwinDataPtr == 7071)) {

                    if (USE_SOCKET_GBL>0) //Write LRIT BPDU FRame to a Socket
                    {
                        if ((EmwinBufferLeft = (EmwinBufEnd - EmwinPtr)) >= 884)
                        {
                            memcpy(EmwinSocketFrame, &LRITframe[8], 884); //Skip 6 byte VCDU primary header, and 2 byte M_PDU header within VCDU zone.  (884 M_PDU payload left)
                            send(new_connection, (char *)EmwinSocketFrame, 884, 0);
                        }
                        else
                        {
                            memcpy(EmwinSocketFrame, &LRITframe[8], EmwinBufferLeft); //Skip 6 byte VCDU primary header, and 2 byte M_PDU header within VCDU zone.  (884 M_PDU payload left)
                            send(new_connection, (char *)EmwinSocketFrame, EmwinBufferLeft, 0);
                        }
                    }

                    if ((EmwinBufferLeft = (EmwinBufEnd - EmwinPtr)) >= 884)
                    {
                        memcpy(EmwinPtr, &LRITframe[8], 884);
                        EmwinPtr += 884;
                        EmwinBytes += 884;
                    }
                    else  //emwindata[10000000] full, start over
                    {
                        memcpy(EmwinPtr, &LRITframe[8], EmwinBufferLeft);
                        EmwinPtr = EmwinData;
                        EmwinBytes += EmwinBufferLeft;
                        memcpy(EmwinPtr, (&LRITframe[8]) + EmwinBufferLeft, (884 - EmwinBufferLeft));
                        EmwinBufferLeft = (884 - EmwinBufferLeft);
                        EmwinPtr += (EmwinBufferLeft);
                        EmwinBytes += EmwinBufferLeft;
                    }
                }
                else
                {
                    if ((EmwinBufferLeft = (EmwinBufEnd - EmwinPtr)) >= EndofEmwinDataPtr)
                    {
                        memcpy(EmwinPtr, &LRITframe[8], EndofEmwinDataPtr);
                        EmwinPtr += EndofEmwinDataPtr;
                        EmwinBytes += EndofEmwinDataPtr;
                    }
                    else
                    {
                        memcpy(EmwinPtr, &LRITframe[8], EmwinBufferLeft);
                        EmwinPtr = EmwinData;
                        EmwinBytes += EmwinBufferLeft;
                        memcpy(EmwinPtr, (&LRITframe[8]) + EmwinBufferLeft, (EndofEmwinDataPtr - EmwinBufferLeft));
                        EmwinBufferLeft = (EndofEmwinDataPtr - EmwinBufferLeft);
                        EmwinPtr += (EmwinBufferLeft);
                        EmwinBytes += EmwinBufferLeft;
                    }
                }

                ProcessEMWINfile();
                return 0;
            }

            
            //At the end an iteration of the bit offset loop
            //If this portion of code is reached, that means that no header was found during 
            //this particular bit marker.  

            //Update, 32 bit sequence to search Continue searching for frame sync...bit by bit
            Ltreg = Ltreg << 1;
            if (hreg & 0x80)
                Ltreg++;
            hreg = hreg << 1;
        }

        //Checked every bit offset within for a given byte marker without finding a header
     
        
        LastProcBytePtr++;
        InvByteCount++;
        consecutive_good_frames = 0; //Could not find header in proper location, check
        //If you are near the next frame header, relax the threshold
        if (((InvByteCount >= (1024-SEARCH_DEPTH)) && (InvByteCount <= (1024+SEARCH_DEPTH))) && ((consecutive_frames_missed+consecutive_frame_errors) < 2)){
            if (lock_status == 1){
                lock_status = 2;
            }
        }
        else if (lock_status>1){
            lock_status = 1;
        }

        BadData= false;
        if (InvByteCount > 1024){
            if (InvToggleFlag){
                LRITDataInvert = LRITDataInvert ^ 0xff;
                InvToggleFlag = FALSE;
                InvByteCount = 0;
                invresetpointer = LastProcBytePtr;
                InvBadFrameCount = 0;
                //goto baddata;
                BadData= true;
            }
            if ((foundsyncflag)&&(!BadData)) {
                InvBadFrameCount++;
                if (InvBadFrameCount >= 3){
                    LastProcBytePtr = invresetpointer;
                    LRITDataInvert = LRITDataInvert ^ 0xff;
                    InvToggleFlag = TRUE;
                    InvBadFrameCount = 0;
                    foundsyncflag = FALSE;
                }
                //goto baddata;
                BadData= true;
            }

            if (!BadData){
                LastProcBytePtr = invresetpointer;
                LRITDataInvert = LRITDataInvert ^ 0xff;
                InvToggleFlag = TRUE;
                InvByteCount = 0;
            }
        }

// baddata:
        BadData= true;
        InvToggleFlag = InvToggleFlag;
        this->bad_data_count++;
        consecutive_frames_missed = consecutive_frames_missed + int((double)this->bad_data_count/1020.0);
    }

    return 0;
}

BOOL CGoesProcessor::GetLritPacketHeader()
{
    int i;
    bool Done_SaveAPID=false;
    // Grab data from CP_PDU User Data
    for (PacketIndex = VCIDtable[VCID][5]; PacketIndex < 6; PacketIndex++)
    {
        packetheader[VCID][PacketIndex]=LRITframe[framepointer];
        framepointer+=1;
        remainingframebytes-=1;
        if (remainingframebytes <= 0)
        {
            VCIDtable[VCID][5]=PacketIndex+1;
            VCIDtable[VCID][0]=0;
            VCIDtable[VCID][1]=1;
            return FALSE;
        }
    }

    for (i = 0; i < 6; i++)
        LRIThead[i] = packetheader[VCID][i];
    VCIDtable[VCID][5] = 0;

    //EV: this is not really the APID
    // 1st byte has: Version No(3bits), Type(1bit),Secondary Header(1bit), 3 bits from APID
    // 2nd byte has: remaining 8 bits from APID. APID has a total of 11 bits
    APID = packetheader[VCID][0];
    if (APID && 0x08)
        SrchHeaderFlag = TRUE;
    else
        SrchHeaderFlag = FALSE;
    //EV: This is the "real" APID
    APID = ((APID << 8) | packetheader[VCID][1]) & 0x7ff;

    //SequenceFlag = 2bits from 3rd byte
    sequenceflag = packetheader[VCID][2];
    sequenceflag = (sequenceflag >> 6) & 0x03;
    VCIDtable[VCID][4] = sequenceflag;
    VCIDtable[VCID][7] = APID;

    // Packet Sequence Counter
    Seqcount = (((packetheader[VCID][2]) & 0x3f) << 8) | packetheader[VCID][3];

    // Packet Length
    PacketLength = packetheader[VCID][4];
    PacketLength = (PacketLength << 8) | packetheader[VCID][5];
    PacketLength++;   //EV Why this addition ?
    VCIDtable[VCID][6] = PacketLength;
    VCIDtable[VCID][0] = 1;

    if (APID == 2047)
        return TRUE;

    for (APIDindex = 0; APIDindex < MAX_APID_TABLE; APIDindex++)
    {
        if (APID == APIDtable[APIDindex])
        {
            Bytesremaining[APIDindex] = PacketLength;
            VCIDtable[VCID][2] = APIDindex;
            return TRUE;
        }
    }

    if ((sequenceflag != 3) && (sequenceflag != 1))
        return FALSE;

    Done_SaveAPID=false;
    for (APIDindex = 0; APIDindex < MAX_APID_TABLE; APIDindex++) {
        if (APIDtable[APIDindex]==2040) {
            APIDtable[APIDindex]=(WORD)APID;
            Bytesremaining[APIDindex]=PacketLength;
            VCIDtable[VCID][2]=APIDindex;
            if (APIDindex>19) return FALSE;
            LRITFilePtr[APIDindex]=LRITFile[APIDindex];
            //goto donesaveAPID;
            Done_SaveAPID=true;
        }
        if (Done_SaveAPID){
            break;
        }
    }
//donesaveAPID:
    Done_SaveAPID=true;
    return TRUE;
}

void CGoesProcessor::ProcessLRITfile(int bytes)
{
    char tempstr[200], tempstr1[20], tempstr2[20];
    DWORD dwBytesRead;
    int Error;
    HANDLE LFile;
    LRIThavename = FALSE;
    itoa(APID, tempstr1, 10);
    itoa(filesequence[APID], tempstr2, 10);
    sprintf(tempstr, "C:/LRITtemp/LRIT%s_%s", tempstr1, tempstr2);

    lpBufPtr = LRITFile[APIDindex];

if (only_socket ==0) {

#ifndef DISABLE_WRITE
    if (CalcCRC(bytes))
    {
        if (LRIThavename)
            sprintf(tempstr,"C:/LRITtemp/%s",LRITfilename);

        //printf("\nExtracting LRIT File \n APID Indx= %i \t FilteType=%i\t File=%s\n", APIDindex,LRITfiletype,tempstr);
        // Some of them are type DCS ???
        if (LRITfiletype != 0){
            //Intermediate files ??
            //TYPE 214: Intermediate LRIT
            //TYPE 130: DCS File
            return;
        }


        LFile = CreateFile((LPCTSTR)tempstr,
                           GENERIC_WRITE|GENERIC_READ,
                           FILE_SHARE_WRITE|FILE_SHARE_READ,
                           NULL,
                           CREATE_ALWAYS,
                           FILE_ATTRIBUTE_NORMAL,
                           (HANDLE)NULL);

        if (!WriteFile(LFile,FileBuffer,numLRITbytes,&dwBytesRead,NULL))
        {
            Error=GetLastError();
            sprintf(tempstr," Failed Write. Last error=%d APIDIndex=%d",Error,APIDindex);
            printf(tempstr);
        }else{
            printf("Extracted %s\n", tempstr);
        }
        CloseHandle(LFile);
        haveLRITfiles = TRUE;
    }
#endif
}
    return;
}

void CGoesProcessor::ProcessEMWINfile()
{
    
if (only_socket ==0) {    
    PUCHAR Headerptr;
    int seq, MaxBlks, index;
    char filename[FILENAME_SIZE+1];
    char headerbuff[HEADER_SIZE];
    char temstr[20];

    //======================================
    // Eliminate GOTOs
    bool ContinueStartSearch=false;
    bool DoneBlock = false;
    //======================================
    //EmwinStart is initialized as false
    if (!EmwinStart){
        while ( (EmwinProcBytes < EmwinBytes)&&(!EmwinStart)){
            if (NULL != *EmwinProcPtr){
                EmwinProcPtr++;
                if (EmwinProcPtr > EmwinBufEnd)
                    EmwinProcPtr = EmwinData;
                EmwinProcBytes++;
                EmwinNullCount = 0;
                //continue may only be used inside a loop, such as a for statement. It terminates the current iteration of the loop and proceeds directly to the next. In the case of a for loop it jumps to its increment-expr.
                continue;
            }

            EmwinNullCount++;
            EmwinProcPtr++;
            if (EmwinProcPtr > EmwinBufEnd){
                EmwinProcPtr = EmwinData;
            }
            EmwinProcBytes++;
            if (EmwinProcBytes >= EmwinBytes){
                return;
            }

            ContinueStartSearch = false;
            while ((EmwinNullCount < 6)&&(!ContinueStartSearch)){//6 null characters point to start of packet
                if (NULL == *EmwinProcPtr){ //Found another one. Increment count
                    EmwinNullCount++;
                    EmwinProcPtr++;
                    if (EmwinProcPtr > EmwinBufEnd){
                        //Wrap back to beginning of buffer
                        EmwinProcPtr = EmwinData;
                    }
                    EmwinProcBytes++;
                    if (EmwinProcBytes >= EmwinBytes){
                        //need more data
                        return;
                    }
                    ContinueStartSearch = false;
                }else{
                    // Found a null by chance, but not 6 in a row.
                    // Restart Search procedure
                    EmwinNullCount = 0;
                    EmwinProcPtr++;
                    if (EmwinProcPtr > EmwinBufEnd){
                        //Wrap back to beginning of buffer
                        EmwinProcPtr = EmwinData;
                    }
                    EmwinProcBytes++;
                    //goto continuestartsearch;
                    ContinueStartSearch = true;
                }
            }

            if (!ContinueStartSearch){
                while ('/' != *EmwinProcPtr){
                    EmwinProcPtr++;
                    if (EmwinProcPtr > EmwinBufEnd)
                        EmwinProcPtr = EmwinData;
                    EmwinProcBytes++;
                    if (EmwinProcBytes >= EmwinBytes)
                        return;
                }
                EmwinStart = TRUE;
                EmwinNullCount = 0;
                EmwinPacketCount = 0;
                //goto foundstart;
                //FoundStart=true;  // ---> This is actuall the same as EmwinStart ???
            }

            //continuestartsearch:
            ContinueStartSearch=true;
            //Not used  i = 0;
        }
    }

//foundstart:
    // EMWIN Packet: 1116 bytes
    //   6 Header =(NULL ) bytes

    if ((1110) > (EmwinBytes - EmwinProcBytes)) //Save 1110
        return;

    static int PrevRemain = 0;
    PrevRemain = remain;

    if ((remain = EmwinBufEnd - EmwinProcPtr) >= (1110+MIN_DATA_SIZE))	//Ensure that remain > 1110
    {// Get FILENAME
        // PF+ 12 bytes for filename
        // HEADER_SIZE bytes worth of header fields, sections 2 - 8 in EMWIN spec - winpro.htm
        memcpy(headerbuff, EmwinProcPtr, HEADER_SIZE);
        if (('P' != *(headerbuff+1)) || ('F' != *(headerbuff + 2))){
            //goto doneblock;
            DoneBlock=true;
        }else{
            EmwinProcPtr += HEADER_SIZE;
            EmwinProcBytes += HEADER_SIZE;
        }

    }else{
        memcpy(headerbuff, EmwinProcPtr, HEADER_SIZE);
        memcpy(EmwinData, EmwinProcPtr+HEADER_SIZE, remain-HEADER_SIZE);
        EmwinProcPtr = EmwinData+remain-HEADER_SIZE;
        if (('P' != *(headerbuff+1)) || ('F' != *(headerbuff + 2))){
            DoneBlock=true;
            //goto doneblock;
        }else{
            EmwinProcBytes += HEADER_SIZE;
        }
    }
    if (!DoneBlock){
        Headerptr = (PUCHAR)headerbuff;
        memcpy(filename, Headerptr + 3, FILENAME_SIZE);
        Headerptr += 18;
        index = strcspn((const char*)Headerptr, "/");
        if (index>19){
            printf("EMWIN Header is Corrupted\n");
            index=19;
            return;
        }
        memcpy(temstr, Headerptr, index);
        temstr[index] = 0;
        seq = atoi(temstr);
        Headerptr += index + 3;

        index = strcspn((const char*)Headerptr, "/");
        if (index>19){
            printf("EMWIN Header is Corrupted\n");
            index=19;
            return;
        }
        memcpy(temstr, Headerptr, index);
        temstr[index]   = 0;
        MaxBlks         = atoi(temstr);   // This tells how many parts the file has

        if (seq == 1) { // File has only one part --> usually txt and small ZIS files
            for (int i = 0; i < NUMBER_OF_FILES; i++)
            {// See which FileIndex position is available to store the file
                if (EmwinFileIndex[i][0] == 0){//found available slot
                    EmwinFileIndex[i][0] = 1;//mark in use
                    EmwinFileIndex[i][1] = 1;//block counter=1, 1st part
                    EmwinFileIndex[i][2] = MaxBlks;//total blocks in file
                    EmwinFileIndex[i][3] = 0;//reset use counter

                    memcpy(EmwinFileName[i], filename, FILENAME_SIZE);
                    EmwinPtrTable[0][i] = EmwinProcPtr;   //save pointer to data block
                    // BUG: Should this be [i][0] ?

                    if (seq == MaxBlks){// only 1 block.  Go process
                        EmwinDisplay(i);
                        EmwinFileIndex[i][0] = 0;  // Clean the File Index Array
                        EmwinFileIndex[i][3] = 0; //reset use counter
                        //goto doneblock;  //Not needed
                        //DoneBlock=true;
                    }
                    DoneBlock=true;
                    break;
                    //goto doneblock;
                }
                // If you are here, it's because  EmwinFileIndex[i][0] != 0
                // The paramater WAIT_TIL_DISCARD says that if "WAIT_TIL_DISCARD" pieces
                // have gone by and still this file is inuse, then it's probably lost
                // so discard it !
                if (!DoneBlock){
                    EmwinFileIndex[i][3]++;
                    if (EmwinFileIndex[i][3] > WAIT_TIL_DISCARD){
                        memcpy(temstr, EmwinFileName[i], FILENAME_SIZE);
                        printf("TIMEOUT: File %s has missing portions: Discard File \n",temstr);
                        EmwinFileIndex[i][0] = 1;
                        EmwinFileIndex[i][1] = 1;//block counter=1, 1st part
                        EmwinFileIndex[i][2] = MaxBlks;
                        EmwinFileIndex[i][3] = 0;
                        memcpy(EmwinFileName[i], filename, FILENAME_SIZE);
                        EmwinPtrTable[0][i] = EmwinProcPtr;
                        if (seq == MaxBlks){
                            EmwinDisplay(i);
                            EmwinFileIndex[i][0] = 0;
                            EmwinFileIndex[i][3] = 0;
                            //goto doneblock; Not needed
                        }
                        //goto doneblock;
                        DoneBlock=true;
                        break;
                    }
                }
            }
        }else{
            for (int i = 0; i < NUMBER_OF_FILES; i++)
            {// Check to see if it's a pre-existing file
                if (0 == memcmp(filename, EmwinFileName[i], FILENAME_SIZE)) {
                    if (0 == EmwinFileIndex[i][0]){
                        //goto doneblock;//invalid file block
                        DoneBlock=true;
                        break;
                    }

                    if ((seq == EmwinFileIndex[i][2])&&(!DoneBlock)){ //last block:  go process
                        EmwinPtrTable[seq - 1][i] = EmwinProcPtr; //save pointer to data block
                        EmwinDisplay(i);
                        EmwinFileIndex[i][0] = 0;//mark available
                        EmwinFileIndex[i][3] = 0;//reset use counter
                        //goto doneblock;
                        DoneBlock=true;
                        break;
                    }

                    if (  ((seq != (1 + EmwinFileIndex[i][1])) || (seq > 200)) && (!DoneBlock)){
                        //missed some data or file too large.  abort
                        memcpy(temstr, EmwinFileName[i], FILENAME_SIZE);
                        printf("LOST Portion of File: File %s has missing portions\n",temstr);
                        printf("\t\t Last Part: %i of %i\n",EmwinFileIndex[i][1], EmwinFileIndex[i][2] );
                        printf("\t\t Current Part: %i of %i\n",seq, EmwinFileIndex[i][2] );
                        EmwinFileIndex[i][0] = 0; //mark available
                        EmwinFileIndex[i][3] = 0; //reset use counter
                        //goto doneblock;
                        DoneBlock=true;
                        break;
                    }
                    if (!DoneBlock){
                        EmwinFileIndex[i][1] = seq;
                        EmwinPtrTable[seq - 1][i] = EmwinProcPtr;
                        //goto doneblock;
                        DoneBlock=true;
                        break;
                    }
                }
            }
        }
    }
//doneblock:
    EmwinStart = FALSE;
    
} // end if "only_socket
    return;
}

void CGoesProcessor::EmwinDisplay(int indx)
{
    PUCHAR CurrentDataPtr,Emwinstopptr;
    int MaxBlks, remain = 0, filesize = 0;
    char temstr[1000], filename[FILENAME_SIZE+1];
    char srch[3] = {0x0d, 0x0a, 0};
    int cmp = 3, i, lng;
    int length;
    int index;
    int Error;
    HANDLE Emfile;
    char tempstr[200], tempstring[200];
    EmwinTempBufferPtr = EmwinTempBuffer;
    MaxBlks = EmwinFileIndex[indx][2];
    filesize = 0;

    for (i = 0; i < MaxBlks; i++)
    {
        //printf("%d\n", &EmwinPtrTable[i][indx]);  //JR Debug to counter Heap corruption
        CurrentDataPtr = EmwinPtrTable[i][indx];
        if ((remain = EmwinBufEnd-CurrentDataPtr) >= 1024)
        {
            memcpy(EmwinTempBufferPtr, CurrentDataPtr, 1024);
            EmwinTempBufferPtr += 1024;
        }
        else
        {
            memcpy(EmwinTempBufferPtr, CurrentDataPtr, remain);
            EmwinTempBufferPtr += remain;
            CurrentDataPtr = EmwinData;
            memcpy(EmwinTempBufferPtr, CurrentDataPtr, 1024 - remain);
            EmwinTempBufferPtr += (1024 - remain);
        }

        filesize += 1024;
    }

    *EmwinTempBufferPtr = 0;
    Emwinstopptr = EmwinTempBufferPtr;
    EmwinTempBufferPtr = EmwinTempBuffer;
    memcpy(filename, EmwinFileName[indx], FILENAME_SIZE);
    filename[FILENAME_SIZE] = NULL;

    if (0 == _strnicmp((const char*)filename + 9, "TXT", 3)){
        cmp = 0;
    }
    if (0 == _strnicmp((const char*)filename + 9, "JPG", 3)){
        cmp = 0;
    }
    if (0 == _strnicmp((const char*)filename + 9, "GIF", 3)){
        cmp = 0;
    }
    //Esteban
    if (0 == _strnicmp((const char*)filename + 9, "ZIS", 3)){
        cmp = 1;
        // This case is different because we rename
    }
    if (0 == _strnicmp((const char*)filename + 9, "UID", 3)){
        cmp = 0;
    }

    // SAVE EVERY FILE, APPEND COUNTER AT THE END
    char alphabet[]="0123456789";

#ifndef DISABLE_WRITE

    switch (cmp)
    {
    case 0:
        // TXT File
        sprintf(tempstr, "C:/EMWINtemp/%s", filename);
        //------------------------------------------------------
        // SAVE EVERY FILE, APPEND COUNTER AT THE END
        // Esteban
        //FILE * pFile;
        sprintf(tempstr, "C:/EMWINtemp/%s", filename);
        //pFile = fopen (tempstr,"r");
        //EV: BUG ? when do we close this ????

        Emfile = CreateFile((LPCTSTR)tempstr,
                            GENERIC_WRITE|GENERIC_READ,
                            FILE_SHARE_WRITE|FILE_SHARE_READ,
                            NULL,
                            CREATE_ALWAYS,
                            FILE_ATTRIBUTE_NORMAL,
                            (HANDLE)NULL);
        Error = GetLastError();
        //------------------------------------------------------

        if (!WriteFile(Emfile, EmwinTempBuffer, filesize, &dwBytesRead, NULL)){
            Error = GetLastError();
            sprintf(tempstr," Failed EMWINWrite. Last error=%d", Error);
            printf(tempstr);
        }else{
            printf("Extracted %s\n", tempstr);
        }
        CloseHandle(Emfile);

        if (0 != _strnicmp(statestring, "ALL", 3))
        {
            if (0 != _strnicmp((const char*)filename + 6, statestring, 2))
                break;
        }
        while (EmwinTempBufferPtr < Emwinstopptr)
        {
            index = strcspn((const char*)EmwinTempBufferPtr, srch);
            if (index > 0)
            {
                *(EmwinTempBufferPtr + index) = NULL;
                length = strlen((const char*)EmwinTempBufferPtr);
                if (length <= 90)
                {
                    sprintf(tempstring,"        ");
                    strcat(tempstring, (char*)EmwinTempBufferPtr);
                }
                else
                {
                    while (length > 0)
                    {
                        memcpy(temstr, EmwinTempBufferPtr, lng=__min(90, length));
                        temstr[lng] = NULL;
                        sprintf(tempstring, "        ");
                        strcat(tempstring, (char*)temstr);
                        EmwinTempBufferPtr += lng;
                        length -= lng;
                    }
                }
            }

            EmwinTempBufferPtr += index + 1;
        }

        break;

    case 1:
        //      ZIS
        //------------------------------------------------------
        // SAVE EVERY FILE, APPEND COUNTER AT THE END
        // Esteban
        //FILE * pFile;
        sprintf(tempstr, "C:/EMWINtemp/");
        strncat(tempstr, filename,11);
        strncat(tempstr, "P",1);
        //pFile = fopen (tempstr,"r");

        //printf("ZIS file: %s \n", tempstr);
        Emfile = CreateFile((LPCTSTR)tempstr,
                            GENERIC_WRITE|GENERIC_READ,
                            FILE_SHARE_WRITE|FILE_SHARE_READ,
                            NULL,
                            CREATE_ALWAYS,
                            FILE_ATTRIBUTE_NORMAL,
                            (HANDLE)NULL);
        Error = GetLastError();
//------------------------------------------------------

        if (!WriteFile(Emfile, EmwinTempBuffer, filesize, &dwBytesRead, NULL)){
            Error = GetLastError();
            sprintf(tempstr," Failed EMWINWrite. Last error=%d", Error);
            printf(tempstr);
        }else{
            printf("Extracted %s\n", tempstr);
        }
        CloseHandle(Emfile);

        if (0 != _strnicmp(statestring, "ALL", 3))
        {
            if (0 != _strnicmp((const char*)filename + 6, statestring, 2))
                break;
        }
        while (EmwinTempBufferPtr < Emwinstopptr)
        {
            index = strcspn((const char*)EmwinTempBufferPtr, srch);
            if (index > 0)
            {
                *(EmwinTempBufferPtr + index) = NULL;
                length = strlen((const char*)EmwinTempBufferPtr);
                if (length <= 90)
                {
                    sprintf(tempstring,"        ");
                    strcat(tempstring, (char*)EmwinTempBufferPtr);
                }
                else
                {
                    while (length > 0)
                    {
                        memcpy(temstr, EmwinTempBufferPtr, lng=__min(90, length));
                        temstr[lng] = NULL;
                        sprintf(tempstring, "        ");
                        strcat(tempstring, (char*)temstr);
                        EmwinTempBufferPtr += lng;
                        length -= lng;
                    }
                }
            }

            EmwinTempBufferPtr += index + 1;
        }

        break;


    default:
        printf ( "\nNon-Standard File Found:%s \n", tempstr);
        break;
    }

#endif  // ENABLE / DISABLE write 2 file.

    return;
}

BOOL CGoesProcessor::RiceDecompress()
{
    // THis library has a memory leak. Memory increases by ~16 kb every other time you execute this
    //                                                            ----------------  
    // SOLVED: By deleting the pointer before you return
    //
    //CRiceDecompression::CRiceDecompression	( 	int 	  Mask,
    //    int 	  BitsPerPixel,
    //    int 	  PixelsPerBlock,
    //    int 	  PixelsPerScanline,
    //    int 	  ScanLinesPerPacket
    //) 	
    CRiceDecompression *rice = new CRiceDecompression(49, 8, 16, NC, 1);
    if (!rice->Decompress(riceptr, ricelength)){
        delete rice;
        return FALSE;
    }else{
        LRITimageptr = rice->Ptr();
        RiceSize = rice->Size();
        delete rice;
        return TRUE;
    }

   // delete rice;


}

BOOL CGoesProcessor::CalcCRC(int filebytes)
{
    PUCHAR filepointer, tempptr;
    unsigned short APID, Seq, Length, scounter;
    unsigned short Crc, CRCByte;
    unsigned char s1byte;
    WORD s2byte;
    int headlength,  totalheader = 1;
    int IMID, MaxSeg, MaxCol, MaxRow;
    int Riceflags, PPB, SLPP;
    int Param, Compress;
    char chardat[5000];
    int CFAC, LFAC, COFF, LOFF;
    numLRITbytes = 0;
    LRITfiletype = 130;
    filepointer = FileBuffer;
    unsigned char EmwinSocketFrame[1000];

    while (lpBufPtr < LRITFile[APIDindex] + filebytes)
    {
        APID = *lpBufPtr;
        APID = ((APID<<8) | *(lpBufPtr+1)) & 0x7ff;
        lpBufPtr += 2;
        Seq = *lpBufPtr;
        Seq = Seq>>6;
        scounter = (*lpBufPtr) & 0x37;
        scounter = (scounter << 8) | *(lpBufPtr + 1);
        lpBufPtr += 2;
        Length = *lpBufPtr;
        Length = (Length << 8) | *(lpBufPtr + 1);
        Length++;
        lpBufPtr += 2;
        Crc = CRC(lpBufPtr, Length - 2);
        CRCByte = *(lpBufPtr + Length - 2);
        CRCByte = (CRCByte << 8) | *(lpBufPtr + Length - 1);
        if (Crc != CRCByte)
            return FALSE;

        riceptr = lpBufPtr;

        if ((Seq == 1) || (Seq == 3))
        {
            if ((TMP_BUFFER_SIZE - (filepointer - FileBuffer)) < Length - 2){
                printf( "TMP Buffer Overload \n");
                return FALSE;
            }
            tempptr = lpBufPtr + Length;
            lpBufPtr += 10;
            memcpy(filepointer, lpBufPtr, Length-12);
            numLRITbytes += Length - 12;
            filepointer += Length - 12;
            totalheader = 1;

            while (totalheader > 0)
            {
                s1byte = *lpBufPtr;
                lpBufPtr++;
                s2byte = *lpBufPtr;
                headlength = (s2byte << 8) | *(lpBufPtr + 1);
                lpBufPtr += 2;
                switch (s1byte)
                {
                case 0:  //Primary Header
                    LRITfiletype = Get(1);
                    totalheader = Get(4);
                    riceptr += totalheader;
                    datafieldlength = Get(8);
                    ricelength = totalheader;
                    totalheader = totalheader-headlength;
                    break;
                case 130: //Header Structure Record
                    if ((headlength - 3) > 5000)
                        return FALSE;
                    memcpy(chardat, lpBufPtr, (headlength - 3));
                    lpBufPtr += (headlength - 3);
                    totalheader = totalheader - headlength;
                    break;
                case 129: //NOAA LRIT Record
                    memcpy(chardat, lpBufPtr, 4);
                    chardat[4] = NULL;
                    lpBufPtr += 4;
                    SPID = Get(2);
                    SubPID = Get(2);
                    Param = Get(2);
                    Compress = Get(1);
                    totalheader = totalheader - headlength;
                    break;
                case 1:  //Image Structure Record
                    NB = Get(1);
                    NC = Get(2);
                    NL = Get(2);
                    CFLG = Get(1);
                    totalheader = totalheader - headlength;
                    break;
                case 2:  //Image Navigation Record
                    memcpy(chardat, lpBufPtr, 32);
                    chardat[32] = NULL;
                    lpBufPtr += 32;
                    CFAC = Get(4);
                    LFAC = Get(4);
                    COFF = Get(4);
                    LOFF = Get(4);
                    totalheader = totalheader - headlength;
                    break;
                case 3:  //Image Data Function
                    if ((headlength - 3) > 5000)
                        return FALSE;
                    memcpy(chardat, lpBufPtr, (headlength - 3));
                    chardat[(headlength - 3)] = NULL;
                    lpBufPtr += (headlength - 3);
                    totalheader = totalheader - headlength;
                    break;
                case 4: //Annotation Record
                    if ((headlength - 3) > 5000)
                        return FALSE;
                    memcpy(LRITfilename, lpBufPtr, (headlength - 3));
                    LRITfilename[(headlength - 3)] = NULL;
                    LRIThavename = TRUE;
                    lpBufPtr += (headlength - 3);
                    totalheader = totalheader - headlength;
                    break;
                case 5: //Time Stamp
                    lpBufPtr += 7;
                    totalheader = totalheader - headlength;
                    break;
                case 6: //Ancillary Text Record
                    if ((headlength - 3) > 5000)
                        return FALSE;
                    memcpy(chardat, lpBufPtr, (headlength - 3));
                    lpBufPtr += (headlength - 3);
                    totalheader = totalheader - headlength;
                    break;
                case 128:  //Image Segment ID Record
                    IMID = Get(2);
                    SegNum = Get(2);
                    StartCol = Get(2);
                    StartLine = Get(2);
                    MaxSeg = Get(2);
                    MaxCol = Get(2);
                    MaxRow = Get(2);
                    totalheader = totalheader - headlength;
                    break;
                case 131:  //Rice Compression Record
                    Riceflags = Get(2);
                    PPB = Get(1);
                    SLPP = Get(1);
                    totalheader = totalheader - headlength;
                    break;
                case 132: //Data Filename
                    if ((headlength - 3) > 5000)
                        return FALSE;
                    memcpy(chardat, lpBufPtr, (headlength - 3));
                    chardat[(headlength - 3)] = NULL;
                    lpBufPtr += (headlength - 3);
                    totalheader = totalheader - headlength;
                    break;
                case 214: //EMWIN file
                    break;
                default:
                    lpBufPtr += (headlength - 3);
                    totalheader = totalheader - headlength;
                    break;
                }
            }

            if ((Seq == 3) && (LRITfiletype == 214)) //EMWIN file
            {
                if (USE_SOCKET_GBL>0) //Write LRIT BPDU FRame to a Socket
                {
                    if ((EmwinBufferLeft = (EmwinBufEnd - EmwinPtr)) >= 836)
                    {
                        memcpy(EmwinSocketFrame, lpBufPtr, 836);
                        send(new_connection, (char *)EmwinSocketFrame, 836, 0);
                    }
                    else
                    {
                        memcpy(EmwinSocketFrame, lpBufPtr, EmwinBufferLeft);
                        send(new_connection, (char *)EmwinSocketFrame, EmwinBufferLeft, 0);
                    }
                }


                if ((EmwinBufferLeft = (EmwinBufEnd - EmwinPtr)) >= 836)
                {
                    memcpy(EmwinPtr, lpBufPtr, 836);
                    EmwinPtr += 836;
                    EmwinBytes += 836;
                }
                else
                {
                    memcpy(EmwinPtr, lpBufPtr, EmwinBufferLeft);
                    EmwinPtr = EmwinData;
                    EmwinBytes += EmwinBufferLeft;
                    memcpy(EmwinPtr, lpBufPtr + EmwinBufferLeft, (836 - EmwinBufferLeft));
                    EmwinPtr += (836 - EmwinBufferLeft);
                    EmwinBytes += (836 - EmwinBufferLeft);
                }

                ProcessEMWINfile();
            }
            lpBufPtr = tempptr;
        }

        if ((Seq == 0) || (Seq == 2))
        {
            ricelength = Length - 2;
            if (LRITfiletype == 0)
            {
               
              if (RiceDecompress())
                {
                    memcpy(filepointer, LRITimageptr, RiceSize);
                    filepointer += RiceSize;
                    numLRITbytes += RiceSize;
                }
                else
                    return FALSE;
            }
            else
            {
                memcpy(filepointer, riceptr, ricelength);
                filepointer += ricelength;
                numLRITbytes += ricelength;
            }
            lpBufPtr += Length;
        }
    }
    return TRUE;
}

unsigned short CGoesProcessor::CRC(const unsigned char* buf, int length)
{
    unsigned short Crc = 0xffff, ind = 0;

    while (length--)
        Crc = (Crc << 8) ^ crcTable[(Crc >> 8) ^ (unsigned short)buf[ind++]];

    return Crc;
}

int CGoesProcessor::Get(int bytes)
{
    int j, result = 0;

    for (j = 0; j < bytes; j++)
    {
        result = (result << 8) | *(lpBufPtr + j);
    }
    lpBufPtr += bytes;

    return result;
}