#pragma once

#include "Reed Solomon.hpp"
#include "PacketDecompression.h"
#include <windows.h>
#include <winsock.h>

#define MAX_APID_TABLE		2048
//#define DATA_BUFFER_SIZE	10240000

#define NUMBER_OF_FILES        (20)

class CGoesProcessor
{
public:
	CGoesProcessor();
	~CGoesProcessor() {UnInitializeProcessor();UnInitializeSocket();};
	BOOL InitializeProcessor();
    void InitializeSocket(int socket_flag, int socket_lrit_flag, int my_only_socket);
	//--------------------------------------------------
	// EV: Backwards Compatibility
    BOOL ProcessLritDataBlock(unsigned char *ReadBuffer, int next_block_size);
	BOOL ProcessEmwinDataBlock(unsigned char *ReadBuffer, int next_block_size);
	//--------------------------------------------------
	BOOL ProcessLritDataBlock(unsigned char *ReadBuffer, int next_block_size, int resynch_thresh, char * BERFileName);
	BOOL ProcessEmwinDataBlock(unsigned char *ReadBuffer, int next_block_size, int resynch_thresh,char * BERFileName);
    
    //--------------------------------------------------
    //EV: RS Debuggin
    BOOL ProcessRSDataBlock(unsigned char *ReadBuffer, int next_block_size);    
	//--------------------------------------------------
    

private:
    void UnInitializeProcessor();
    void UnInitializeSocket();
    //--------------------------------------------------
    //EV: Changes needed for Backwards Compatibility:
    //void LRIT_Main();
	//void EMWIN_Main();
    void ProcessFSK();
    void ProcessReedSolomon();
    //--------------------------------------------------
    int LRIT_Main(int resych_thresh, char * BERFileName);
	int EMWIN_Main(int resych_thresh, char * BERFileName);
	BOOL GetLritPacketHeader();
	void ProcessLRITfile(int bytes);
	void ProcessEMWINfile();
	void EmwinDisplay(int indx);
	BOOL RiceDecompress();
	BOOL CalcCRC(int filebytes);
	unsigned short CRC(const unsigned char* buf, int length);
	int Get(int bytes);

	unsigned char *lpBufPtr;
	unsigned char *CurrentDataPtr;
	unsigned char *LastProcBytePtr;
	unsigned char *old_LastProcBytePtr;
	unsigned char *DataBuffer;
	BOOL LRITFirstsync;
	BOOL LRITsyncflag;
	BOOL Firstsyncflag;
	BOOL foundsyncflag;
	/*
	BOOL connectstatus;
	BOOL Viterbisync;
	BOOL Timersync;
	*/
	unsigned int Ltreg;
	unsigned int lritsync;
	unsigned char LRITDataInvert;
	int InvByteCount;
	int InvBadFrameCount;
	unsigned char *invresetpointer;
	int processbytes;
	char LRIThead[6];
	unsigned char LRITframe[1024];
	unsigned char LRIT1frame[1024];
	char Random[1020];
	int VCIDtable[64][9];
	unsigned short VCID;
	unsigned short APID;
	int LengthTable[MAX_APID_TABLE];
	int TotalAPIDS;
	int APIDindex;
	unsigned int PendingAPID;
	int Bytesremaining[MAX_APID_TABLE];
	unsigned short APIDtable[MAX_APID_TABLE];
	char filesequence[MAX_APID_TABLE];
	CReedSolomon *LRITrs;
	unsigned long VCDUcount;
	unsigned long framerrors;
	BOOL            Fullheader;
	BOOL            Extension;
	char            sequenceflag;
	int             framepointer;
	int             remainingframebytes;
	unsigned int PacketLength;
	unsigned char packetheader[64][8];
	unsigned short Seqcount;
	unsigned char   *LRITFile[NUMBER_OF_FILES];
	unsigned char   *LRITFilePtr[NUMBER_OF_FILES];
	BOOL            InvToggleFlag;
	int             PacketIndex;
	BOOL            SrchHeaderFlag;
	BOOL            LRIThavename;
	char            LRITfilename[80];
	int             LRITfiletype;
	BOOL            showhead;
	unsigned short SubPID, SPID;
	int             NB, NC, NL, CFLG, SegNum, StartCol, StartLine;
	unsigned char   *FileBuffer;
	int             numLRITbytes;	
	BOOL haveLRITfiles;
	unsigned char *riceptr;
	int datafieldlength, ricelength;
	unsigned char   *LRITimageptr;
	int             RiceSize;
	unsigned int    FrameSyncCnt;
	int             EmwinDataSize;
	unsigned char   *EmwinData;
	unsigned char   *EmwinBufEnd;
	unsigned char   *EmwinProcPtr;
	unsigned char   *EmwinPtr;
	int             EmwinBytes;
	int             EmwinNullCount;
	unsigned char   *EmwinFileName[NUMBER_OF_FILES];
	int             EmwinFileIndex[NUMBER_OF_FILES][4];
	unsigned long   dwBytesRead;
	BOOL            haveEMWINfiles;
	char            statestring[10];
	unsigned char   *EmwinStartPtr;
	unsigned char  *EmwinPtrTable[200][NUMBER_OF_FILES];
	BOOL EmwinStart;
	int EmwinPacketCount;
	unsigned char *EmwinTempBuffer;
	unsigned char *EmwinTempBufferPtr;
	int EmwinProcBytes;
	long EmwinBufferLeft;
    int viterbi_framerrors; //JR added variable
    int RS_framerrors; // JR added variable
    int ErCountCum;  //JR Cummalitive Error Count for BER computations
    int USE_SOCKET_GBL;
    int USE_SOCKET_LRIT_GBL;
    SOCKET wx_server_socket; //JR added to write to a socket
    SOCKET new_connection;  //JR added to write to a socket
    SOCKET new_connection_lrit;
    SOCKET wx_server_socket_lrit; 
    WSADATA wsaData;
    int only_socket;
    //===============================================
    //EV Want to avoid static variables, so we do things right and we define them here
    //int bad_data_count;
    int best_offset;
    int best_offset_ready_signal;
    bool MissedFrameInitialState;
    
    long int LRITFrameCount ;
    long int LRITFrameError ;
    //long int LRITVitFrameError;
    long int Missed_FrameCount;
    long int VCDUmark1;  //Marker to see if we missed a VCDU frame
    long int VCDUmark0;
    //===============================================
    // State Machine Variables
    int      lock_status; //lock status =-1,1,2,3 means completely_unsynched, somewhat_synched, pretty_synched, extremely_synched
    long int consecutive_good_frames;
    long int consecutive_frames_missed;
    long int consecutive_frame_errors;
    //EMWIn Variables
    long int EMWINFrameCount ;
    long int EMWINFrameError ;   
    int remain;
    int bad_data_count;
    //===============================================
    //Re -ACQ
    int ReAcquire;

};

const unsigned short crcTable[] = {
   0x0000, 0x1021, 0x2042, 0x3063, 0x4084, 0x50A5, 0x60C6, 0x70E7,
   0x8108, 0x9129, 0xA14A, 0xB16B, 0xC18C, 0xD1aD, 0xE1cE, 0xF1eF,
   0x1231, 0x0210, 0x3273, 0x2252, 0x52B5, 0x4294, 0x72F7, 0x62D6,
   0x9339, 0x8318, 0xB37B, 0xA35A, 0xD3bD, 0xC39C, 0xF3fF, 0xE3dE,
   0x2462, 0x3443, 0x0420, 0x1401, 0x64E6, 0x74C7, 0x44A4, 0x5485,
   0xA56A, 0xB54B, 0x8528, 0x9509, 0xE5eE, 0xF5cF, 0xC5aC, 0xD58D,
   0x3653, 0x2672, 0x1611, 0x0630, 0x76D7, 0x66F6, 0x5695, 0x46B4,
   0xB75B, 0xA77A, 0x9719, 0x8738, 0xF7dF, 0xE7fE, 0xD79D, 0xC7bC,
   0x48C4, 0x58E5, 0x6886, 0x78A7, 0x0840, 0x1861, 0x2802, 0x3823,
   0xC9cC, 0xD9eD, 0xE98E, 0xF9aF, 0x8948, 0x9969, 0xA90A, 0xB92B,
   0x5AF5, 0x4AD4, 0x7AB7, 0x6A96, 0x1A71, 0x0A50, 0x3A33, 0x2A12,
   0xDbfD, 0xCbdC, 0xFbbF, 0xEb9E, 0x9b79, 0x8b58, 0xBb3B, 0xAb1A,
   0x6CA6, 0x7C87, 0x4CE4, 0x5CC5, 0x2C22, 0x3C03, 0x0C60, 0x1C41,
   0xEdaE, 0xFd8F, 0xCdeC, 0xDdcD, 0xAd2A, 0xBd0B, 0x8d68, 0x9d49,
   0x7E97, 0x6EB6, 0x5ED5, 0x4EF4, 0x3E13, 0x2E32, 0x1E51, 0x0E70,
   0xFf9F, 0xEfbE, 0xDfdD, 0xCffC, 0xBf1B, 0xAf3A, 0x9f59, 0x8f78,
   0x9188, 0x81a9, 0xB1cA, 0xA1eB, 0xD10C, 0xC12D, 0xF14E, 0xE16F,
   0x1080, 0x00A1, 0x30C2, 0x20E3, 0x5004, 0x4025, 0x7046, 0x6067,
   0x83b9, 0x9398, 0xA3fB, 0xB3dA, 0xC33D, 0xD31C, 0xE37F, 0xF35E,
   0x02B1, 0x1290, 0x22F3, 0x32D2, 0x4235, 0x5214, 0x6277, 0x7256,
   0xB5eA, 0xA5cB, 0x95a8, 0x8589, 0xF56E, 0xE54F, 0xD52C, 0xC50D,
   0x34E2, 0x24C3, 0x14A0, 0x0481, 0x7466, 0x6447, 0x5424, 0x4405,
   0xA7dB, 0xB7fA, 0x8799, 0x97b8, 0xE75F, 0xF77E, 0xC71D, 0xD73C,
   0x26d3, 0x36F2, 0x0691, 0x16B0, 0x6657, 0x7676, 0x4615, 0x5634,
   0xD94C, 0xC96D, 0xF90E, 0xE92F, 0x99c8, 0x89e9, 0xB98A, 0xA9aB,
   0x5844, 0x4865, 0x7806, 0x6827, 0x18C0, 0x08E1, 0x3882, 0x28A3,
   0xCb7D, 0xDb5C, 0xEb3F, 0xFb1E, 0x8bf9, 0x9bd8, 0xAbbB, 0xBb9A,
   0x4A75, 0x5A54, 0x6A37, 0x7A16, 0x0AF1, 0x1AD0, 0x2AB3, 0x3A92,
   0xFd2E, 0xEd0F, 0xDd6C, 0xCd4D, 0xBdaA, 0xAd8B, 0x9de8, 0x8dc9,
   0x7C26, 0x6C07, 0x5C64, 0x4C45, 0x3CA2, 0x2C83, 0x1CE0, 0x0CC1,
   0xEf1F, 0xFf3E, 0xCf5D, 0xDf7C, 0xAf9B, 0xBfbA, 0x8fd9, 0x9ff8,
   0x6E17, 0x7E36, 0x4E55, 0x5E74, 0x2E93, 0x3EB2, 0x0ED1, 0x1EF0,
};

