/* GOES Radio Comments 
ar_emwin_image_sink is a gr_sync_block type that takes in inputs, but does not
generate an output for another gr_block.  This particular project creates and instantiates 
the GOESprocessor class with the name emwin_processor.  Upon starting up, this block takes
instantiates Goesprocessor::InitializeProcessor(); and GOESProcessor::InitialzeSocket(socket port)
During successive calls, this the emwin_image_sink class calls GOESProcessor::processEMWINDatablock
Process EMWINDatablock is the data link layer processor for EMWIN-N 
*/
#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <ar_emwin_image_sink.h>
#include <gr_io_signature.h>
#include <cstdio>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <stdexcept>

// win32 (mingw/msvc) specific
#ifdef HAVE_IO_H
#include <io.h>
#endif
#ifdef O_BINARY
#define	OUR_O_BINARY O_BINARY
#else
#define	OUR_O_BINARY 0
#endif

// should be handled via configure
#ifdef O_LARGEFILE
#define	OUR_O_LARGEFILE	O_LARGEFILE
#else
#define	OUR_O_LARGEFILE 0
#endif



ar_emwin_image_sink::ar_emwin_image_sink(size_t itemsize, const char *filename,int resynch_thresh, char * BERFileName, int socket_flag, int socket_lrit_flag, int only_socket)
  : gr_sync_block ("file_sink", gr_make_io_signature(1, 1, itemsize), gr_make_io_signature(0, 0, 0)),
    d_itemsize(itemsize), d_fp(0), d_new_fp(0), d_resynch_thresh(resynch_thresh), d_updated(false), d_BERFileName(BERFileName),
    d_socket_flag(socket_flag), d_socket_lrit_flag(socket_lrit_flag), d_only_socket(only_socket)
{
  if (!open(filename))
    throw std::runtime_error ("can't open file");

  if (!emwin_processor.InitializeProcessor())
	  throw std::runtime_error ("can't initialize goes processor");

  emwin_processor.InitializeSocket(d_socket_flag, -1,d_only_socket);
  Synched = TRUE;
    
}

ar_emwin_image_sink_sptr
ar_make_emwin_image_sink (size_t itemsize, const char *filename, int resynch_thresh, char * BERFileName)
{
  return ar_emwin_image_sink_sptr (new ar_emwin_image_sink (itemsize, filename, resynch_thresh, BERFileName, 0,0,0));
}

ar_emwin_image_sink_sptr
ar_make_emwin_image_sink (size_t itemsize, const char *filename, int resynch_thresh, char * BERFileName, int socket_flag,int socket_lrit_flag, int only_socket)
{
  return ar_emwin_image_sink_sptr (new ar_emwin_image_sink (itemsize, filename, resynch_thresh, BERFileName, socket_flag,socket_lrit_flag, only_socket));
}


ar_emwin_image_sink::~ar_emwin_image_sink ()
{
  close();
  if (d_fp){
    fclose((FILE *) d_fp);
    d_fp = 0;
  }
}

bool
ar_emwin_image_sink::open(const char *filename)
{
  omni_mutex_lock	l(d_mutex);	// hold mutex for duration of this function

  // we use the open system call to get access to the O_LARGEFILE flag.
  int fd;
  if ((fd = ::open (filename,
		    O_WRONLY|O_CREAT|O_TRUNC|OUR_O_LARGEFILE|OUR_O_BINARY, 0664)) < 0){
    perror (filename);
    return false;
  }

  if (d_new_fp){		// if we've already got a new one open, close it
    fclose((FILE *) d_new_fp);
    d_new_fp = 0;
  }

  if ((d_new_fp = fdopen (fd, "wb")) == NULL){
    perror (filename);
    ::close(fd);		// don't leak file descriptor if fdopen fails.
  }

  d_updated = true;
  return d_new_fp != 0;
}

void
ar_emwin_image_sink::close()
{
  omni_mutex_lock	l(d_mutex);	// hold mutex for duration of this function

  if (d_new_fp){
    fclose((FILE *) d_new_fp);
    d_new_fp = 0;
  }
  d_updated = true;
}

int 
ar_emwin_image_sink::work (int noutput_items,
		    gr_vector_const_void_star &input_items,
		    gr_vector_void_star &output_items)
{
  unsigned char *inbuf = (unsigned char *) input_items[0];
  int  nwritten = 0;

  if (d_updated){
    omni_mutex_lock	l(d_mutex);	// hold mutex for duration of this block
    if (d_fp)
      fclose((FILE *)d_fp);
    d_fp = d_new_fp;			// install new file pointer
    d_new_fp = 0;
    d_updated = false;
  }
  
  if (!d_fp)
    return noutput_items;		// drop output on the floor

  emwin_processor.ProcessEmwinDataBlock(inbuf, noutput_items,d_resynch_thresh, d_BERFileName);

  return noutput_items;
}
