#include <usrp1_source_c.h>
#include <string>

/*
    Gain choices are as follows:
    LNA(high) MIXER(low)  -> GAIN = 01; 12dB
    LNA(low)  MIXER(high) -> GAIN = 02; 13dB
    LNA(high) MIXER(high) -> GAIN = 03; 25dB
    LNA(low)  MIXER(low)  -> GAIN = 04; 00dB
	Returns false if unsuccessful!
*/
float set_frend_gain(usrp1_source_c_sptr usrp_source, int which_board, int gain );
/*
	Provide freq, range from 1.6GHz to 1.95GHz minium step is 200kHz, example 1831.2e6 etc...
	Returns false if unsuccessful!
*/
bool set_frend_dfs(usrp1_source_c_sptr usrp_source, double freq );


#define bm_BIT0  1
#define bm_BIT1  2
#define bm_BIT2  4
#define bm_BIT3  8
#define bm_BIT4  16
#define bm_BIT5  32
#define bm_BIT6  64
#define bm_BIT7  128
#define bm_BIT8  256
#define bm_BIT9  512
#define bm_BIT10 1024
#define bm_BIT11 2048
#define bm_BIT12 4096
#define bm_BIT13 8192
#define bm_BIT14 16348
#define bm_BIT15 32768
#define bm_BIT16 65536
#define bm_BIT17 131072
#define bm_BIT18 262144
#define bm_BIT19 524288
#define bm_BIT20 1048576
#define bm_BIT21 2097152
#define bm_BIT22 4194304
#define bm_BIT23 8388608



//SPI COMMANDS
// least signficant bit first
#define	SPI_DFS_LSB	        (1 << 7)	
// most significant bit first 
#define	SPI_DFS_MSB	        (0 << 7)	
// select d'board TX A
#define	SPI_ENABLE_DFS		 0x10	 
//24 bits
#define BUFFER_MASK          0xffffff   
// select d'board TX B
#define	SPI_ENABLE_TX_B		0x40	


//CONTROL latch
#define SELECT_C_L           0

//5mA
#define CORE_POW_05mA        0
//10mA
#define CORE_POW_10mA        bm_BIT2  
//15mA
#define CORE_POW_15mA        bm_BIT3 
//20mA
#define CORE_POW_20mA        (bm_BIT3|bm_BIT2)  

//00 - NORMAL 
//RESETS ALL COUNTERS
#define COUNTER_RESET        bm_BIT4  

//Three state out
#define MUX_OUT_CTRL_1       0 
//Digital lock detect
#define MUX_OUT_CTRL_2       bm_BIT5 
//N divider out
#define MUX_OUT_CTRL_3       bm_BIT6  
//DVdd
#define MUX_OUT_CTRL_4       (bm_BIT6|bm_BIT5)  
//R divider out
#define MUX_OUT_CTRL_5       bm_BIT7  
//N-channel open drain
#define MUX_OUT_CTRL_6       (bm_BIT7|bm_BIT5)  
//Serial data out
#define MUX_OUT_CTRL_7       (bm_BIT7|bm_BIT6)   
//DVdd
#define MUX_OUT_CTRL_8       (bm_BIT7|bm_BIT6|bm_BIT5)   
//negative polarity
#define PHASE_DET_POL        bm_BIT8 
//normal is 0
#define CP_NORM_N            bm_BIT9  
//setting 1 and 2
#define CP_GAIN_N            bm_BIT10 
//MUTE_TILL-LOCK detect
#define MUL_LEV_LOCK_EN      bm_BIT11 


//50 Omhs -12dBm
#define OUT_POW_12dBm        0 
//50 Omhs -9dBm
#define OUT_POW_9dBm         bm_BIT12 
//50 Omhs -6dBm
#define OUT_POW_6dBm         bm_BIT13 
//50 Omhs -3dBm
#define OUT_POW_3dBm         (bm_BIT13|bm_BIT12)  


#define CP_CUR_LEV_0_31mA    0  
#define CP_CUR_LEV_0_62mA    (bm_BIT17|bm_BIT14)
#define CP_CUR_LEV_0_93mA    (bm_BIT18|bm_BIT15)
#define CP_CUR_LEV_1_25mA    (bm_BIT18|bm_BIT17|bm_BIT15|bm_BIT14)
#define CP_CUR_LEV_1_56mA    (bm_BIT19|bm_BIT16)
#define CP_CUR_LEV_1_87mA    (bm_BIT19|bm_BIT17|bm_BIT16|bm_BIT14)
#define CP_CUR_LEV_2_18mA    (bm_BIT19|bm_BIT18|bm_BIT16|bm_BIT15)
#define CP_CUR_LEV_2_50mA    (bm_BIT19|bm_BIT18|bm_BIT17|bm_BIT16|bm_BIT15|bm_BIT14)

#define POW_DOWN1_EN         bm_BIT20
#define POW_DOWN2_EN         bm_BIT21
#define PRESCALER_08_09      0
#define PRESCALER_16_17      bm_BIT22
#define PRESCALER_32_33      (bm_BIT23|bm_BIT22)

//N COUNTER latch
#define SELECT_N_L           bm_BIT1

#define CP_GAIN_ON           bm_BIT21


#define DIV_BY_2_ON          bm_BIT22
#define DIV_BY_2_SEL_ON      bm_BIT23


//R COUNTER latch
#define SELECT_R_L           bm_BIT0

//3nsec
#define ANTI_BACK_PW_3ns     0
//1.3nsec
#define ANTI_BACK_PW_1_3ns   bm_BIT16
//6nsec
#define ANTI_BACK_PW_6ns     bm_BIT17
//3nsec
#define ANTI_BACK_PW_3_2ns   (bm_BIT17|bm_BIT16)

//lock detect after 3 cycles or 5
#define LOCK_DET_5_CYC       bm_BIT18

//TEST mode should be normal 
#define TMB_NORMAL_N         bm_BIT19

//band select clock divider
#define BAND_SEL_1           0 
//band select clock divider
#define BAND_SEL_2           bm_BIT20
//band select clock divider 
#define BAND_SEL_3           bm_BIT21
//band select clock divider
#define BAND_SEL_4           (bm_BIT21|bm_BIT20) 

//don't care
#define RESERVED             (bm_BIT23|bm_BIT22)




