
#ifndef INCLUDED_GR_LRIT_IMAGE_SINK_H
#define INCLUDED_GR_LRIT_IMAGE_SINK_H

#include <gr_sync_block.h>
#include <omnithread.h>

#include "GoesProcessor.h"

class ar_lrit_image_sink;
typedef boost::shared_ptr<ar_lrit_image_sink> ar_lrit_image_sink_sptr;

ar_lrit_image_sink_sptr ar_make_lrit_image_sink(size_t itemsize, const char *filename, int resynch_thresh, char * BERFileName, int socket_flag, int socket_lrit_flag, int only_socket);
ar_lrit_image_sink_sptr ar_make_lrit_image_sink(size_t itemsize, const char *filename, int resynch_thresh, char * BERFileName, int socket_flag, int socket_lrit_flag);
ar_lrit_image_sink_sptr ar_make_lrit_image_sink(size_t itemsize, const char *filename, int resynch_thresh, char * BERFileName);
ar_lrit_image_sink_sptr ar_make_lrit_image_sink(size_t itemsize, const char *filename, int resynch_thresh);
ar_lrit_image_sink_sptr ar_make_lrit_image_sink(size_t itemsize, const char *filename);

/*!
 * \brief Write stream to file.
 * \ingroup sink
 */

class ar_lrit_image_sink : public gr_sync_block
{
  friend ar_lrit_image_sink_sptr ar_make_lrit_image_sink(size_t itemsize, const char *filename, int resynch_thresh, char * BERFileName, int socket_flag, int socket_lrit_flag, int only_socket);
  friend ar_lrit_image_sink_sptr ar_make_lrit_image_sink(size_t itemsize, const char *filename, int resynch_thresh, char * BERFileName, int socket_flag, int socket_lrit_flag);
  friend ar_lrit_image_sink_sptr ar_make_lrit_image_sink(size_t itemsize, const char *filename, int resynch_thresh, char * BERFileName);
  friend ar_lrit_image_sink_sptr ar_make_lrit_image_sink(size_t itemsize, const char *filename, int resynch_thresh);
  friend ar_lrit_image_sink_sptr ar_make_lrit_image_sink(size_t itemsize, const char *filename);

 private:
    size_t	    d_itemsize;
    void	       *d_fp;		// current FILE pointer
    void	       *d_new_fp;	// new FILE pointer
    bool		    d_updated;	// is there a new FILE pointer?
    omni_mutex	    d_mutex;    
    CGoesProcessor  lrit_processor;
    int             d_resynch_thresh;
    FILE *          d_BERFILE;
    char *          d_BERFileName;
    int             d_socket_flag;
    int             d_socket_lrit_flag;
    int             d_only_socket;

 protected:
    ar_lrit_image_sink(size_t itemsize, const char *filename, int resynch_thresh, char * BERFileName, int socket_flag, int socket_lrit_flag, int only_socket);
    ar_lrit_image_sink(size_t itemsize, const char *filename, int resynch_thresh, char * BERFileName, int socket_flag, int socket_lrit_flag);
    ar_lrit_image_sink(size_t itemsize, const char *filename, int resynch_thresh, char * BERFileName);
	ar_lrit_image_sink(size_t itemsize, const char *filename, int resynch_thresh);
    ar_lrit_image_sink(size_t itemsize, const char *filename);

 public:
    ~ar_lrit_image_sink();
    //EV
    bool get_synch();
    bool  Synched;

  /*! 
   * \brief Open filename and begin output to it.
   */
    bool open(const char *filename);

  /*!
   * \brief Close current output file.
   *
   * Closes current output file and ignores any output until
   * open is called to connect to another file.
   */
    void close();

    int work(int noutput_items,
	   gr_vector_const_void_star &input_items,
	   gr_vector_void_star &output_items);
};


#endif /* INCLUDED_GR_LRIT_IMAGE_SINK_H */
