/*
GOES Radio Comments
This is the toplevel file.   It calls usrp_source_c to create an interface to the hardware,
freq_acq, bpsk_rx, and viterbi_rx.   This file works in either single threaded mode or 
multithreaded mode.  In single threaded mode, viterbi_rx is not called explicitly from this
file. Instead it is implemented inside of bprsk_rx.  In multithreaded mode, two separate 
block spaces are made, one for bpsk_rx and one for viterbi_rx.  
*/
#pragma once

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <ar_fir_filter_ccf.h>
#include "gnuradio.h"
#include "bpsk_modem.h"
//#include "ControllerIF.h"
#include "freq_acq.h"
//GNU Radio
#include <gr_single_threaded_scheduler.h>
#include <gr_block_detail.h>
#include <gr_file_sink.h>
#include <gr_file_source.h>
#include <gr_head.h>
#include <usrp1_source_c.h>
// Multithreading
#include <ar_ring_buffer_source2.h>
#include <ar_ring_buffer_sink2.h>
#include "circular_buffer.h"
#include <conio.h>
#include <time.h>
using namespace std;

//=============================================
// For Memory Leak Detection
#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif
#define DEBUG_NEW new(_NORMAL_BLOCK, __FILE__, __LINE__)

//using namespace std;
#define _CRTDBG_MAP_ALLOC
#include <stdlib.h>
#include <crtdbg.h>
//=============================================

//RE-ACQ only works multithreaded for the time being

#define MULTI_THREAD
//#undef  MULTI_THREAD

int run_LRIT(const lrit_config &config);

int main(int argc, char* argv[])
{

// -+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+--+-+-+-+-+-+-+-+-+--++-+-+-+-+-+-+-+-+-+--+-+
// HANDLE SOCKETS
    HANDLE hLogFile;
    hLogFile = CreateFile("c:\\temp\\usrp_log\\mem_leak_log.txt", GENERIC_WRITE, FILE_SHARE_WRITE,
                          NULL, CREATE_ALWAYS, FILE_ATTRIBUTE_NORMAL, NULL);
    //_CrtSetReportMode(_CRT_WARN, _CRTDBG_MODE_FILE);
    _CrtSetReportMode(_CRT_WARN, _CRTDBG_MODE_FILE | _CRTDBG_MODE_DEBUG);
    _CrtSetReportFile(_CRT_WARN, hLogFile);
    _RPT0(_CRT_WARN,"file message\n");
    //CloseHandle(hLogFile);
    
    //Check for memory leaks
    _CrtSetDbgFlag ( _CRTDBG_ALLOC_MEM_DF | _CRTDBG_LEAK_CHECK_DF);
    //	 _CrtSetReportMode(_CRT_WARN, _CRTDBG_MODE_FILE | _CRTDBG_MODE_DEBUG);
    // _CrtSetReportFile(_CRT_WARN, _CRTDBG_FILE_STDOUT);
    
    char *LeakBucket;
    LeakBucket = (char*) malloc(10000);
    int * p1 = new int[5];

    //-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+--+-+-+-+-+-+-+-+-+--++-+-+-+-+-+-+-+-+-+--+-+
    //--------------------------------------------------------------

    // Read Command Prompt Input Arguments

    //---------------------------------------------------------------
    //-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+--+-+-+-+-+-+-+-+-+--++-+-+-+-+-+-+-+-+-+--+-+

    int l;

    //help function
    for (l = 1; l < argc; l++ ) {
        if (!strcmp(argv[l],"help")) {
            printf("\n\n\nThis Application Demodulates the BPSK LRIT Signal from GOES Using the Following Options\n\nflags\t\t default\t description\n------\t\t --------\t -------------\n\n-socket\t\t 0\t\t Socket port number for which EMWIN data is sent out over localhost\n");
            printf("-socket_lrit\t 0\t\t Socket port number for which LRIT data is sent out over localhost\n");
            printf("-wr\t\t 0\t\t Flag that writes input USB data to a file temp\\usrp_log\\usrp_in.txt\n");
            printf("-rec\t\t Null\t\t Processes data from the file [FileName] instead of using the USB\n");
            printf("-repeat\t\t 1\t\t Flag that specifies if the input data file will be looped back, or processed just once\n");
            printf("-sink\t\t 0\t\t Flag that writes BPSK tracked constellation complex data points to temp\\usrp_log\\timing_tracked.dat\n");
            printf("-fc\t\t 137.5e6\t Input IF Center Frequency of the LRIT Signal at the input to the IF Digitizer\n");
            printf("-br\t\t 293883\t\t Baud rate of the Signal\n");
            printf("-dr\t\t 108\t\t Decimation ratio for the IF Digitizer\n");
            printf("-pbw\t\t 100\t\t Phase bandwidth set as the (symbol rate)/pbw.  pbw=100 implies 293883/100 = 2.938 KHz\n");
            printf("-tbw\t\t 3000\t\t Timing loop bandwidth set as the (symbol rate)/tbw.  tbw=3000 implies 293883/3000 = 97.96 Hz\n");
            printf("-lfilter\t 1\t\t Flag that specifies if an input rejection filter is applied to the signal before demodulation\n");
            printf("-agc\t\t 10\t\t Level for the software AGC\n");
            printf("-roll\t\t 0.5\t\t Root Raised Cosine Receiver Pulse Shaping Filter roll-off (alpha) factor\n");
            printf("-acq\t\t 1\t\t Flag that turns the coarse frequency acquisition engine off/on\n");
            printf("-foff\t\t 0\t\t Manual Initial Frequency Offset Adjustment in Hz\n");
            printf("-fa_numfft\t 16\t\t Number of FFT averages on spectrum during frequency acquisition\n");
            printf("-fa_res\t\t 50\t\t Spectral Resolution in Hz during frequency acquisition process\n");
            printf("-fa_range\t 50000\t\t Range of frequencies to search over during frequency acquisition\n");
            printf("-fa_1\t\t 0\t\t Flag that uses the 1st order frequency acquisition algorithm\n");
            printf("-fa_2\t\t 1\t\t Flag that uses the 2nd order frequency acquisition algorithm\n");
            printf("-fa_4\t\t 1\t\t Flag that uses the 4th order frequency acquisition algorithm (4th order requires 2nd order to be 1)\n");
            printf("-fa_fc\t\t 0\t\t Frequency Offset in Hz performed before conducting frequency acquisition\n");
            printf("-vperiod\t 5\t\t The number of Viterbi packets processed before verifying it is still locked\n");
            printf("-vthresh\t 0.125\t\t Output Viterbi BER to declare the FEC block is Locked\n");
            printf("-thresh\t\t 22\t\t The number of errors allowed in the CADU frame header to declare that lock is maintained\n");
            return 0;
        }
    }


 
    char *in_socket = "0";  //Option to dump data out to a socket
    for (l = 1; l < argc; l++ ) {
        if (!strcmp(argv[l],"-socket")) {
            in_socket                = argv[l+1];
        }
    }

    char *in_socket_lrit = "0";  //Option to dump data out to a lrit socket
    for (l = 1; l < argc; l++ ) {
        if (!strcmp(argv[l],"-socket_lrit")) {
            in_socket_lrit                = argv[l+1];
        }
    }

    char *in_only_socket   = "0"; //Option to dump data out to only socket and not write products to disk
    for (l = 1; l < argc; l++ ) {
        if (!strcmp(argv[l],"-onlysocket")) {
            in_only_socket = argv[l+1];
        }
    }

    //This flag writes USB data direclty to a file
    char *file_sink_on    = "0";
    for (l = 1; l < argc; l++ ) {
        if (!strcmp(argv[l],"-wr")) {
            file_sink_on = argv[l+1];
        }
    }
    
    //Phase bandwidth ratio defined as Symbol_Rate/PHaseBW = BW in Hz. 
    char *PhaseBW    = "100";
    for (l = 1; l < argc; l++ ) {
        if (!strcmp(argv[l],"-pbw")) {
            PhaseBW = argv[l+1];
        }
    }

    //Timing bandwidth ratio defined as Symbol_Rate//TimeBW = BW in Hz. 
    char *TimeBW    = "3000";
    for (l = 1; l < argc; l++ ) {
        if (!strcmp(argv[l],"-tbw")) {
            TimeBW = argv[l+1];
        }
    }

    //USRP FPGA decimation rate
    char *in_decim_rate = "108";
    for (l = 1; l < argc; l++ ) {
        if (!strcmp(argv[l],"-dr")) {
            in_decim_rate = argv[l+1];
        }
    }

    //USRP FPGA DDFS center frequency
    char *in_fc = "137.5e6";
    for (l = 1; l < argc; l++ ) {
        if (!strcmp(argv[l],"-fc")) {
            in_fc                = argv[l+1];
        }
    }

    //Manual Frequency Offset
    char *in_foff = "0.0";
    for (l = 1; l < argc; l++ ) {
        if (!strcmp(argv[l],"-foff")) {
            in_foff                = argv[l+1];
        }
    }

    //Activate Low Pass LRIT filter
    char *in_lrit_filter_on = "1";
    for (l = 1; l < argc; l++ ) {
        if (!strcmp(argv[l],"-lfilter")) {
            in_lrit_filter_on    = argv[l+1];
        }
    }


    //Read data from a file source instead a USB
    char *in_rec_data_file  ="c:/temp/usrp_log/SampleRecordedData/win_usrp_in_Bits2_DR108_v01.txt";
    int  using_rec_data       =0;
    for (l = 1; l < argc; l++ ) {
        if (!strcmp(argv[l],"-rec")) {
            using_rec_data           =1;
            in_rec_data_file         = argv[l+1];
        }
    }

    char *repeat    = "1";    //Option to Repeat File Source
    for (l = 1; l < argc; l++ ) {
        if (!strcmp(argv[l],"-replay")) {
            repeat = argv[l+1];
        }
    }

    char *in_shaping_rolloff = "0.5"; 	     // Rolloff for RRC Pulse
    for (l = 1; l < argc; l++ ) {
        if (!strcmp(argv[l],"-roll")) {
            in_shaping_rolloff   = argv[l+1];
        }
    }

     //Flag to turn the Initial Coarse Frequency Acquisiton Engine on/off
    char *ACQEngine = "1";     
    for (l = 1; l < argc; l++ ) {
        if (!strcmp(argv[l],"-acq")) {
            ACQEngine= argv[l+1];
        }
    }

    //Set the bit rate for the bpsk_rx class
    char *in_bitrate = "293883";     
    for (l = 1; l < argc; l++ ) {
        if (!strcmp(argv[l],"-br")) {
            in_bitrate= argv[l+1];
        }
    }

    //Set the AGC level for the bpsk_rx class
    char *in_AGC = "10";      // AGC
    for (l = 1; l < argc; l++ ) {
        if (!strcmp(argv[l],"-agc")) {
            in_AGC= argv[l+1];
        }
    }

    //Set the buffer size between the ring_buffer_source (thread 1)
    //and ring_buffer_sink (thread 2) 
    char *buff_size = "65536";      // BITRATE
    for (l = 1; l < argc; l++ ) {
        if (!strcmp(argv[l],"-buff")) {
            buff_size= argv[l+1];
        }
    }

    //Obsolete Option
    char      *using_icy      ="0";
    double    icyFreqMult     =1.0;;
    for (l = 1; l < argc; l++ ) {
        if (!strcmp(argv[l],"-icy")) {
            using_icy= argv[l+1];
        }
    }

    //This sets the maximum error rate to determine that the Viterbi decoder is locked
    char      *vit_thresh     = "0.125"; 
    for (l = 1; l < argc; l++ ) {
        if (!strcmp(argv[l],"-vthresh")) {
            vit_thresh = argv[l+1];
        }
    }

    //This sets the frequency with wich Viterbi verifies that it is still locked
    char      *resynch_period     ="5000";
    for (l = 1; l < argc; l++ ) {
        if (!strcmp(argv[l],"-vperiod")) {
            resynch_period= argv[l+1];
        }
    }

    //This sets the CADU Frame Synchronization error threshold in the data link layer
    char      *resynch_thresh     ="22";  
    for (l = 1; l < argc; l++ ) {
        if (!strcmp(argv[l],"-thresh")) {
            resynch_thresh = argv[l+1];
        }
    }



    // FRequency Acquisition: number off FFT averages on spectrum
    //The more averaging, the better the result, but the longer it takes
    char *in_num_fft_avg = "16";      
    for (l = 1; l < argc; l++ ) {
        if (!strcmp(argv[l],"-fa_numfft")) {
            in_num_fft_avg= argv[l+1];
        }
    }

    // FRequency Acquisition: Spectral Resolution in hz
    char *in_resolution = "50";      
    for (l = 1; l < argc; l++ ) {
        if (!strcmp(argv[l],"-fa_res")) {
            in_resolution= argv[l+1];
        }
    }

    // FRequency Acquisition: Range of Frequencies to search over
    char *in_freq_range = "50000";      
    for (l = 1; l < argc; l++ ) {
        if (!strcmp(argv[l],"-fa_range")) {
            in_freq_range= argv[l+1];
        }
    }

     // FRequency Acquisition: Square Law Detector
    char *in_square_law = "1";     
    for (l = 1; l < argc; l++ ) {
        if (!strcmp(argv[l],"-fa_2")) {
            in_square_law= argv[l+1];
        }
    }

    // FRequency Acquisition: Fourth Law Detector
    char *in_fourth_law = "0";      
    for (l = 1; l < argc; l++ ) {
        if (!strcmp(argv[l],"-fa_4")) {
            in_fourth_law= argv[l+1];
        }
    }

    // FRequency Acquisition: Internal Mixer (Separate from built in Usrp Mixer!) ALWAYS ZERO!
    char *in_freq_acq_fc = "0";      
    for (l = 1; l < argc; l++ ) {
        if (!strcmp(argv[l],"-fa_fc")) {
            in_freq_acq_fc= argv[l+1];
        }
    }

    if (using_rec_data==0){
        printf( "==========================\n");
        printf("  Tracking LIVE SATELLITE    \n");
        if (atoi(using_icy)==0)
            printf("  Using 64 MHZ BOX    \n");
        else
            printf("  Using 48 MHZ BOX    \n");
        printf("===========================\n\n");
    }else{
        printf("===========================\n\n");
        printf("  Tracking RECORDED SATELLITE    \n");
        if (atoi(using_icy)==0)
            printf("  Using 64 MHZ BOX    \n");
        else
            printf("  Using 48 MHZ BOX    \n");
        printf("===========================\n\n");
    }

    //--------------------------------------------------------------
    //             ---  BPSK - LRIT/EMWIN     ----
    // EMWIN-N:    Fc                     = 137.5Mhz.
    //   	     Data rate                = 293883 ksps.
    //
    //URSP BOX
    //--------
    // Sampling at 64 MHz --> 3rd Harmonic at 9.5  Mhz
    // Signal comes in at 293883 ksps.  ADC samples a 64 Msps.  Thus I hav 217.7 sps
    // decimate by 108  gives 2.0164 samples per symbol
    // decimate by 72  gives 3.0246 samples per symbol
    // decimate by 54  gives 4.03 samples per symbol
    //
    // ICY BOX
    //---------
    // Sampling at 48 MHz --> 3rd Harmonic at  6.5 Mhz
    // Signal comes in at 293883 ksps.  ADC samples a 48 Msps.  Thus I hav 163.33 sps
    // decimate by 80  gives 2.0416 samples per symbol
    // decimate by 54  gives 3.0246 samples per symbol
    // decimate by 40  gives 4.08 samples per symbol
    //
    //--------------------------------------------------------------

    // --------------------- Create the transmitter ------------------
    // USRP: RX
    unsigned int	decim_rate	    = atoi(in_decim_rate);   // 2samples-> 108, 3 samples-->72, 4 samples--> 54
    double		    fc			    = atof(in_fc);		     // Center freq. for USRP
    int			    lrit_filter_on	= atoi(in_lrit_filter_on);
    //==============================================
    // RX Front End Variables
    // USRP EMWIN


    
    //Create a configuration file that possess ALL of the configuration data to do 
    //usrp_source, freq_acq, bpsk_rx, and viterbi decoding. 
    lrit_config config;

    //Convert the IF frequency (for example 137.5 MHz)
    //into the DDFS frequency needed to bring the signal down to baseband based
    //on the IF-sampling principle.  For 137.5, mirror_fc would be 9.5 MHz
    double mirror_fc =fc;
    if (atoi(using_icy) ==0){ //USRP
        config.Fs		 = 64e6/decim_rate;
        for  (int ii=0; ii< 5;ii++){
            if (fabs(fc - ii*64e6) < mirror_fc)
                mirror_fc = fabs(fc - ii*64e6);
        }
    }else{// ICY Obsolete
        icyFreqMult     = 64.0/48.0;
        config.Fs		= 48e6/decim_rate;
        for  (int ii=0; ii< 5;ii++){
            if (fabs(fc - ii*48e6)*icyFreqMult < mirror_fc)
                mirror_fc = fabs(fc - ii*48e6)*icyFreqMult;
        }
    }



    //Populate all of the system options into the lrit_config data structure 

    fc                          = mirror_fc;

    config.Fc				    = fc;
    config.samples_per_symbol   = float(config.Fs) /atof(in_bitrate);
    config.bit_rate		        = atoi(in_bitrate);
    config.shaping_rolloff	    = atof(in_shaping_rolloff);  //unkown
    config.icyFreqMult          = icyFreqMult;
    config.lrit_filter_on       = lrit_filter_on;
    config.using_rec_data       = using_rec_data;
    config.file_sink_on         = (atoi(file_sink_on)==1);
    config.RunACQEngine         = bool(atoi(ACQEngine));
    config.decim_rate           = decim_rate;
    config.foff                 = atof(in_foff);
    config.buff_size            = atoi(buff_size);
    config.vit_thresh           = atof(vit_thresh);
    config.PBW					= atof(PhaseBW);
    config.TBW					= atof(TimeBW);
    config.AGC				    = atof(in_AGC);
    config.vit_thresh           = atof(vit_thresh);
    config.resynch_thresh       = atoi(resynch_thresh);
    config.resynch_period       = atoi(resynch_period);
    config.square_law           = (bool)atoi(in_square_law);
    config.fourth_law           = (bool)atoi(in_fourth_law);
    config.num_fft_avg          = atof(in_num_fft_avg);
    config.resolution           = atof(in_resolution);
    config.freq_range           = atof(in_freq_range);
    config.repeat               = atoi(repeat);
    config.socket_flag          = atoi(in_socket);  
    config.socket_lrit_flag     = atoi(in_socket_lrit);
    config.only_socket   = atoi(in_only_socket);

    strncpy(config.in_rec_data_file,in_rec_data_file,200);


    //Facilitate Re-Acquisition by calling run_LRIT(config) successively.
    //If run_LRIT gets into a troubled state, it is restarted using the same
    //User option as specified above.  

    int LRIT_synch  =   0;   // Not Synched
    while (TRUE){
        if (LRIT_synch ==0 ){
            LRIT_synch = run_LRIT(config);
        }else if (LRIT_synch ==-1 ){
            //QUIT PROGRAM
            return 0;
        }
    }
    return 0;
}



int run_LRIT(const lrit_config &config){

/* This function carrier out all of the functions needed to receive the LRIT/HRIT signal.  
//It calls usrp_source_c to create an interface to the hardware,
freq_acq, bpsk_rx, and viterbi_rx.  If it gets into a troubled state, main() restarts it. 
*/

    //----------- Set some of the variables for the USRP board ---------//
    int			    which_board	= 0;
    int			    nchan		= 1;
    int			    mux			= 1;
    int			    mode		= 0;
    int			    NoSamples	= 250e6;  // No. USRP Samples
    float           foff        =config.foff;

    // Container for the transmitter and receiver blocks
    vector<gr_block_sptr> bpsk_blocks;
    // Copy configuration
#ifdef MULTI_THREAD
    vector<gr_block_sptr> viterbi_blocks;
#endif
    // =======================================================================
    //                  Create frequency acquisition block
    // =======================================================================
    freq_acq acq;
    freq_acq_config                    freq_acq_config0;
    if (config.RunACQEngine) {
        // FIXME If there's ever a chance that the planning functions
        // will be called in multiple threads, we've got to ensure single
        // threaded access.  They are not thread-safe.
        printf( "======================\n");
        printf( "  ACQ-Engine ENABLED  \n");
        printf( "======================\n");
        freq_acq_config0.Fs                =  config.Fs	;
        freq_acq_config0.Fc                = 0;
        freq_acq_config0.delay             = 0;  //Delay by one symbol
        freq_acq_config0.resolution        = config.resolution;
        freq_acq_config0.freq_range        = config.freq_range;
        freq_acq_config0.num_fft_avg       = config.num_fft_avg;
        freq_acq_config0.square_law        = config.square_law;
        freq_acq_config0.fourth_law        = config.fourth_law;
        // USRP SETUP
        freq_acq_config0.usrp_which_board  = which_board;
        freq_acq_config0.usrp_decim_rate   = config.decim_rate;
        freq_acq_config0.usrp_nchan        = nchan;
        freq_acq_config0.usrp_mux          = mux;
        freq_acq_config0.usrp_mode         = mode;
        freq_acq_config0.usrp_fc           = config.Fc;
        freq_acq_config0.rec_data          = config.using_rec_data;
        freq_acq_config0.repeat            = config.repeat;
        if (config.using_rec_data)
            strcpy(freq_acq_config0.filename, config.in_rec_data_file);
        freq_acq acq(freq_acq_config0, false); // no Nallatech hardware

        // Begin ACQ
        acq.run();
        Sleep(2000);
        freq_acq_result result;
        // CAll get_peak_freq() and wait until the acquisition class returns
        // The result. 
        while (!acq.get_peak_freq(result)) Sleep(1);

        printf("Acquired Peak: Freq = %f [Delta=%f], pwr = %f\n", result.freq,
               result.freq - freq_acq_config0.Fc ,result.relative_power);

        //Stop the Acquisition engine, and realese the hardware
        acq.stop();

        printf("Input Freq.Offset = %4.2e \n",foff);
        // Update detected Frequency offset
        foff = -result.freq ;

    }else {
        foff=config.foff;
    }
    // =======================================================================
    //                  BEGIN TRACKING & PROCESSING
    // =======================================================================

    //Define an interface to the hardware, usrp_source 
    usrp1_source_c_sptr           usrp_source;
    //--------------------------
    //   ICY BOX (Obsolete) 
    //--------------------------
    foff = foff* config.icyFreqMult; 
    //------------------------------------------------------------------------------------
    // HEAD: File Sink: Input Bits
    unsigned int	     InputDataSize	=sizeof(gr_complex);
    gr_block_sptr	     my_head;
    gr_block_sptr		 my_head2;  //plot filtered signal
    my_head				 = gr_make_head(InputDataSize,NoSamples);
    my_head2			 = gr_make_head(InputDataSize,NoSamples);   // plot filtered signal

   
    gr_file_sink_sptr   file_sink ;
    gr_file_source_sptr recorded_data_source;
    //Create a File Source to Read Input Data
    if (config.using_rec_data==1){
        recorded_data_source = gr_make_file_source(InputDataSize, config.in_rec_data_file, config.repeat);
    }
    else //If not using a file source, create an interface to the IF Digitzer
    {
        printf("----------------------\n  Reset DIGITIZER PARAMETERS \n");
        printf("Frequency %5.3e =(%4.3e + %4.3f) \n ",config.Fc+foff,config.Fc,foff);
        printf("----------------------\n");

        usrp_source= usrp1_make_source_c ( which_board, config.decim_rate,nchan, mux,mode, 0,0,"","");
        usrp_source->set_rx_freq (0,config.Fc+foff);
        usrp_source->set_pga (which_board,1);
    }

    if (config.file_sink_on== 1){
        file_sink   = gr_make_file_sink(InputDataSize, "c:/temp/usrp_log/win_usrp_in.txt");
        connect_blocks(usrp_source,   0,   my_head,  0, bpsk_blocks);
        connect_blocks(my_head,       0,   file_sink,0, bpsk_blocks);
    }
    // --------------------- Create the receiver ---------------------
    bpsk_config BPSKconfig;
    BPSKconfig.Fc                   =0.0;       // Not Used
    BPSKconfig.Fs                   =config.Fs;
    BPSKconfig.samples_per_symbol   =config.samples_per_symbol;
    BPSKconfig.bit_rate             =config.bit_rate;
    BPSKconfig.shaping_rolloff      =config.shaping_rolloff;
    BPSKconfig.PBW					= config.PBW;
    BPSKconfig.TBW					= config.TBW;
    BPSKconfig.AGC					= config.AGC;
    // VITERBI CONFIGURATION
    viterbi_config configV;
    configV.AGC				       = config.AGC;
    configV.resynch_thresh         = config.resynch_thresh;
    configV.vthresh                = config.vit_thresh;
    configV.resynch_period         = config.resynch_period;
    configV.socket_flag            = config.socket_flag;  
    configV.socket_lrit_flag       = config.socket_lrit_flag;
    configV.only_socket            = config.only_socket;

    //--------------------------
    //   Bit Error Rate Log
    //--------------------------
    char dateStr [9];
    char timeStr [9];
    _strdate( dateStr);
    printf( "The current date is %s \n", dateStr);
    _strtime( timeStr );
    printf( "The current time is %s \n", timeStr);

    FILE		* BERpFile;
    char 	    *BERDataFile;
    BERDataFile         =(char *) calloc(200,sizeof(char));
    strcpy(BERDataFile,"c:/LRITTEMP/BER_LRITlog.txt");
    BERpFile = fopen(BERDataFile , "a+" );
    if ( (BERpFile==NULL) ) {
        printf("\n\nUnable to open a BER log file!!!!\n This run will not be logged!!!!");
    }
    else{
        fprintf(BERpFile,"========================================================\n");
        fprintf(BERpFile,"This is an Error Log for EMWIN/LRIT BPSK transmission\n");
        fprintf(BERpFile, "Date: %s \t\t Time: %s \n\n", dateStr,timeStr);
        if (config.using_rec_data){
            fprintf(BERpFile," RECORDED DATA \n");
        }else{
            fprintf(BERpFile," LIVE Satellite DATA \n");
        }
        fprintf(BERpFile, "Simulation Data:\n");
        printf("\t Clock.Frequency: %4.1f MHz\n",64.0/config.icyFreqMult);
        fprintf(BERpFile, "\t Clock.Frequency: %4.1f MHz\n",64.0/config.icyFreqMult);
        fprintf(BERpFile, "\t ACQ Enable? %i \n \t Square Law=%i \t 4th Power=%i\n",config.RunACQEngine,config.square_law,config.fourth_law);
        fprintf(BERpFile, "\t Frequency Offset=%4.3e\n",foff);
        fprintf(BERpFile, "\t Tracking Constant:%4.3e \t Phase Constant=%4.3e\n",config.TBW,config.PBW);
        fprintf(BERpFile, "\t Viterbi: Threshold:%4.3e \t Resynch Period=%i\n\n",configV.vthresh,configV.resynch_period);

    }
    fclose(BERpFile);
    configV.BERFileName            = BERDataFile;
    BPSKconfig.BERFileName            = BERDataFile;
    //--------------------------


    //Instantiate the bpsk_demodulator. 
    bpsk_rx     bpsk_demod(bpsk_blocks, BPSKconfig);

    // ----------------- FILTER front end LRIT SATELLITE SIGNAL  ---------------------
    vector<float>           usrp_lowpass,icy_lowpass;
    gr_fir_filter_ccf_sptr  lrit_filter;

    if (config.lrit_filter_on){
        if (config.icyFreqMult==1.0){
            if (config.decim_rate == 54){
                grab(usrp_lowpass, "c:/temp/usrp_log/input_data/lowpass_54decim_50taps.coef"); // Load coefs from a file
            }else if (config.decim_rate == 32){
                grab(usrp_lowpass, "c:/temp/usrp_log/input_data/lowpass_32decim_50taps.coef"); // Load coefs from a file
            }else if (config.decim_rate == 72){
                grab(usrp_lowpass, "c:/temp/usrp_log/input_data/lowpass_72decim_50taps.coef"); // Load coefs from a file
            }else{
                grab(usrp_lowpass, "c:/temp/usrp_log/input_data/lowpass_108decim_50taps.coef"); // Load coefs from a file
            }
            lrit_filter =  gr_make_fir_filter_ccf (1, usrp_lowpass);
        }else{ //IF Digitizer
            printf("Loading ICY Filter Coeffs \n");
            if (config.decim_rate == 40){ // 4 samples per symbol
                grab(icy_lowpass, "c:/temp/usrp_log/input_data/icy_40decim_35taps.coef"); // Load coefs from a file
            }else{
                grab(icy_lowpass, "c:/temp/usrp_log/input_data/icy_80decim_35taps.coef"); // Load coefs from a file
            }
            lrit_filter =  gr_make_fir_filter_ccf (1, icy_lowpass);
        }

        if (config.using_rec_data){
            connect_blocks(recorded_data_source,    0, lrit_filter, 0, bpsk_blocks);
        }else{
            connect_blocks(usrp_source,             0, lrit_filter, 0, bpsk_blocks);
        }

        // SEND INPUT DATA TO BPSK RECEIVER
        connect_blocks(lrit_filter, 0, bpsk_demod.get_input_block(), 0, bpsk_blocks);
    }
    else{
        if (config.using_rec_data){
            connect_blocks(recorded_data_source,    0, bpsk_demod.get_input_block(), 0, bpsk_blocks);
        }else{
            connect_blocks(usrp_source,             0, bpsk_demod.get_input_block(), 0, bpsk_blocks);
        }
    }
    // --------------------------------------------------------------------------------
    printf("==================================\n");
    printf("  SOFTWARE RADIO CONFIGURATION    \n");
    printf("==================================\n\n");
    printf("Input Parameters: \n");
    printf("Using Recorded Data ? (0,1)=\t %i \n" , config.using_rec_data );
    if (config.using_rec_data){
        printf("\tRec Data File   \t %s \n"  ,config.in_rec_data_file);
    }
    printf("Using LRIT Filter? (0,1)=\t %i\n", config.lrit_filter_on);
    printf("Circular Buffer Size= \t\t %i \n"  ,config.buff_size);
    printf("\n");
    printf(" USRP Information: \n");
    printf("\t Decimation Rate = %i \n",config.decim_rate);
    printf("\t Sampling Freq. = %5.4e \n" , config.Fs );
    printf("\t\t Center Freq. = %5.4e \n" , config.Fc);
    printf("\t\t Offset Freq. = %5.4f \n" , config.foff);
    printf("\t Pulse-Shaping Rolloff = %4.2f \n" , config.shaping_rolloff);
    printf("\t Samples per symbol = %4.2f \n",config.samples_per_symbol);
#ifdef MULTI_THREAD
    printf("\t MULTI-THREADED PROCESS \n" );
#else
    printf("\t SINGLE-THREAD PROCESS \n" );
#endif
    printf("\n\n");
    //-------------------------------------------------------------------

    // True:	Multi thread
    // False:	Single Process
#ifdef MULTI_THREAD

    // Create Viterbi Process
    //viterbi_rx  viterbi_dec(viterbi_blocks,config.Vit_Resynch_Period);
    viterbi_rx  viterbi_dec(viterbi_blocks, configV);

    // Create PIPES
    gr_ring_buffer_sink2_r_sptr        buff_sink =      gr_make_ring_buffer_sink2_r (config.buff_size);
    gr_ring_buffer_source2_r_sptr      buff_source =    gr_make_ring_buffer_source2_r(buff_sink->get_buffer());

    //Take the output of BPSK demodulator (thread 1) and feed it into the input of the Viterbi decoder (thread 2) 
    connect_blocks(bpsk_demod.get_softout(), 0, buff_sink,                      0, bpsk_blocks);
    connect_blocks(buff_source,              0, viterbi_dec.get_input_block(),	0, viterbi_blocks);

    //Begin a separate scheduler on each thread 
    gr_single_threaded_scheduler_sptr       scheduler =        start_scheduler_thread(bpsk_blocks, true);
    gr_single_threaded_scheduler_sptr       scheduler_viterbi =        start_scheduler_thread(viterbi_blocks, true);

#else
    //   ----------------
    //   SINGLE THREAD
    //   ----------------
    //gr_single_threaded_scheduler_sptr   scheduler =        start_scheduler_thread(bpsk_blocks, false);
    gr_single_threaded_scheduler_sptr   scheduler =        start_scheduler_thread(bpsk_blocks, true);

#endif

TempFixScheduler:

    bool rx_synch ;
    rx_synch = TRUE;

    char c;

    printf("TestBench: Test Synchronism \n");
    while (TRUE){
        if (_kbhit()){
            printf("Enter a new command: q =  QUIT \n");
            c = _getch();
            if (c == 'q') return -1;
        }

        Sleep(5000); // If this is too short, the program exits directly without giving it a chance to lock
#ifdef MULTI_THREAD
        rx_synch = viterbi_dec.get_packet_lock();
#else
        rx_synch = bpsk_demod.get_packet_lock();
#endif
        if (!rx_synch){
            printf("TestBench: Lost Frequency Synch \n");
            break;
        }
    }

#ifdef MULTI_THREAD

    FILE *BER_File;
    BER_File = fopen(BERDataFile,"a+");
    if (BER_File==NULL){
        printf("Could Not Open BER log file name\n");
        exit;
    }else{

        fprintf(BER_File,"---------------------------\n Frequency Resync Required \n"  );
        _strdate( dateStr);
        fprintf(BER_File, "The current date is %s \n", dateStr);
        _strtime( timeStr );
        fprintf(BER_File, "The current time is %s \n", timeStr);
        fprintf(BER_File,"---------------------------\n"  );
        fclose(BER_File);
    }

    //=============================================
    /*
    //WORKS IN DEBUG MODE
    printf("--> Stop Viterbi Scheduler\n");
    scheduler_viterbi->stop();
    while(scheduler_viterbi->running());
    printf("<-- Viterbi Scheduler Stopped\n");
    Sleep(100);

    printf("--> Stop BPSK Scheduler\n");
    scheduler->stop();

    while(scheduler->running()){
        Sleep(100);
    }
    printf("<-- BPSK Scheduler Stopped\n ");
    Sleep(1000);
    */
    //=============================================

    //RELEASE MODE
    // Stop Circular Buffer
    buff_source->gr_buffer_prepare2stop(); // Stop Reading from buffer
    Sleep(100);
    buff_sink->gr_buffer_prepare2stop();    // Stop Enqueuing
    Sleep(2000);


    printf("======================================= \n");
    printf("\t\t Restarting Schedulers \n");
    printf("======================================= \n");

    scheduler_viterbi->stop();
    scheduler->stop();



    //-----------------------------------------------
    //      BPSK SCHEDULER
    printf("--> Stop BPSK Scheduler\n");
    int  timeout1=0;
    while (scheduler->running()){
        timeout1++;
        Sleep(100);
        buff_sink->gr_buffer_prepare2stop();    // Stop Enqueuing
        if (timeout1 > 100){
            Sleep(1000);
            printf("ERROR: BPSK Stop TIMEOUT \r");
        }
    }
    printf("<-- BPSK Scheduler Stopped\n ");
    //Sleep(100);

//-----------------------------------------------
    //scheduler_viterbi->stop();
    printf("--> Stop Viterbi Scheduler\n");
    //for (int ii=0;ii< 10; ii++){
    //    scheduler->stop();
    //    scheduler_viterbi->stop();
    //}
    scheduler_viterbi->stop();
    int timeout2=0;
    while (scheduler_viterbi->running()){
        timeout2++;
        //scheduler_viterbi->stop();
        Sleep(100);
        if (timeout2 > 100){
            Sleep(1000);
            printf("ERROR: Viterbi Stop TIMEOUT \r");
        };
    }
    printf("<-- Viterbi Scheduler Stopped\n\n");



    viterbi_blocks.clear();
    bpsk_blocks.clear();

    scheduler = gr_single_threaded_scheduler_sptr(); // Assign to NULL
    scheduler_viterbi = gr_single_threaded_scheduler_sptr(); // Assign to NULL
#else
    // STOP, CLEAR & RETURN
    scheduler->stop();
    Sleep(100);
    while (scheduler->running() ) ; // Wait for it to quit
    Sleep(1000);
    scheduler = gr_single_threaded_scheduler_sptr(); // Assign to NULL
    bpsk_blocks.clear();
#endif
    usrp_source = usrp1_source_c_sptr(); // Assign to NULL
    if (c == 'q'){
        return -1;
    }else{
        return 0;
    }
}