/*
GOES Radio Comments
This file implements the data link layer processor for FSK
ar_fsk_image_sink sends an array of demodulated bits (readbuffer), to processFSKdatablock() for processing.
processFSKdatablock stores Readbuffer into a local array called stored_bits.  The bits go through the 
ITU V.35 descrambler.  After that, the data is synched to six nulls.  After that the data is parsed for 
File Name /PF, Block Number, /PN, Total blocks for the file, /PT, Check Sum /CS, Time Stamp, /FD.
From there it saves temporary files to a buffer and waits to receiver all parts before file assembly
It also checks to make sure the packet is error free by looking at the check sum.

*/
#include "FSK_ImageProcessor.h"  
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <winsock.h>
#include <math.h>

#define DATA_BUFFER_SIZE 65536 //65,536 integers is biggest gnu radio packet size, and is our max local read buffer size.
#define NPOLY 20       //Descrambler Depth
#define MINIMUM_BITS 56000 //the minimum number of bits that gaurantee that a full packet can be processed.  The larger the number, the safer, but also, the slower the performacne.
#define MAXFILES  20
#define MAXPARTS  100
#define MAXLENGTH 1200
#define FSK_PATH "C:\\EMWINTEMP\\"
//#define ERROR_LOG_PATH "C:\\EMWINTEMP\\EMWINError.log"
#define FER_LOG_PATH "C:\\EMWINTEMP\\FSK_ERROR.log"
#define VERBOSEFSK 1 //Print all messages to screen+file, or just standard messages
//#define VERBOSEFSK 0 //Print all messages to screen+file, or just standard messages

int stored_bits[DATA_BUFFER_SIZE];  //Data Buffer with FSK Demodulated Data From GnuRadio
int FSK_bits[DATA_BUFFER_SIZE];  //Descrambled/Flipped Data
int G1[20] = {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0};
int header[10] = {0,0,0,0,0,0,0,0,0,1};
int s[NPOLY] = {0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1};
int stored_bits_count = 0; //This is the number of bits stored in memory so that I can process a full packet
int AvailIndex[MAXFILES][12+21];
int PartsRx[MAXFILES][MAXPARTS];
int FSK_files[MAXFILES][MAXPARTS][MAXLENGTH+1];
int GoodFrameCount = 0;
int FrameErrorCount = 0;
int Success_NewOffset[2] = {0,0}; 
int MyChar_Offset[2] = {0,0};
int run=0;
int Consecutive_Frame_Errors = 0;
static int USE_SOCKET_GBL = 0;

SOCKET wx_server_socket; //JR added to write to a socket
SOCKET new_connection;  //JR added to write to a socket
WSADATA wsaData;
unsigned char d_data_dump[1000]; 
unsigned char FSK_bytes[8192];

//FILE		* BERpFile; 

void Initialize_FSKprocessor(int use_socket_flag)
{
     USE_SOCKET_GBL = use_socket_flag; 
     unsigned short port_number = USE_SOCKET_GBL;
    // Write Data out to a socket 
    if(USE_SOCKET_GBL>0)
    {
      // Used to open connection to the Windows networking DLLs
      int errorCode = WSAStartup(0x0101, &wsaData);
      if(errorCode != 0) {
        printf("Unable to open connection to Windows networking system.\n");
      }

      // Setup the server socket to listen on the bitrate control port
      wx_server_socket = socket(AF_INET, SOCK_STREAM, IPPROTO_TCP);
      struct sockaddr_in server_addr;
      server_addr.sin_family = AF_INET;
      server_addr.sin_port = htons(port_number);
      server_addr.sin_addr.s_addr = inet_addr("127.0.0.1");  //Local Host connection 
      if(bind(wx_server_socket, (SOCKADDR*)&server_addr, sizeof(server_addr)) != 0) {
        printf("Unable to start/bind bit-rate request server (%d). Port may already be in use.\n", WSAGetLastError());
        closesocket(wx_server_socket);
      }
      if(listen(wx_server_socket, SOMAXCONN) != 0) {
          printf("Unable to listen to bitrate-control port(%d) Unknown cause.\n", WSAGetLastError());
          closesocket(wx_server_socket);
      }

      // Accept incoming connections until told to stop:
      //while(g_server_running) {
      printf("Waiting for Wx Weather Message Client to Connect to HRIT-R Server....\n");
      new_connection = accept(wx_server_socket, NULL, NULL);

      if(new_connection == INVALID_SOCKET) {
            // If invalid, dump the socket and try again...
          printf("Invalid Socket\n");
          closesocket(new_connection);
          //continue;
       }
       else
         printf("valid socket now open\n");
     
       //recv(new_connection, (char *)data_dump, 100, 0);
       //printf("Received 40 bytes\n");
    }
}

void Uninitialize_FSKprocessor()
{
     if(wx_server_socket != NULL){  //JR socket
        closesocket(wx_server_socket);
        wx_server_socket = NULL;
     }
     if(new_connection != NULL){  //JR socket
        closesocket(new_connection);
        new_connection = NULL;
     }
     WSACleanup();  //JR socket
     //fclose(BERpFile);
}

//int ProcessFSKDataBlock(int *ReadBuffer, int next_block_size, int flip_sign, FILE * BERFILE) 
int ProcessFSKDataBlock(int *ReadBuffer, int next_block_size, int flip_sign, char * BERFileName) 
{
     int bits_processed = next_block_size;  
     int innerprod = 0;
     int block_length = 0;
     int NullCount = 0;
     int headersum = 0;
     int ii=0;
     int AdditionalNulls;
     int tempChar[100];
     int FileName[100];
     char FileNameChar[100];
     int tempCharLengthFN = 0;
     int tempCharLengthBN = 0;
     int tempCharLengthBT = 0;
     int tempCharLengthCS = 0;
     int tempCharLengthMD = 0;
     int BlockNum[100];
     int BlockNumValVec[15];
     int BlockNumInt;
     int BlockTot[100];
     int BlockTotValVec[15];
     int BlockTotInt;
     int CheckSum[100];
     int CheckSumValVec[15];
     int CheckSumInt;
     int TxCheckSum;
     int RxCheckSum;
     int MyDate[100];
     int DataStart;
     int MyData[1024];
     int MyDataLength;
     int NameCharCount;
     int FoundName = 0;
     int nn;
     int FileIndex;
     int FirstAvailable;
     int val = 0;
     int PartsFound;
     char FilePath[100];
     FILE *FSKFile;
     FILE *FER_File;
     FILE *FSK_ErrorFile;
     unsigned char binarydatatowrite[120000]; //Didn't feel like using new to allocate MAXPARTS*MAXLENGTH
     char MyText[300];
     unsigned char data_dump[1000];

     int verbose = VERBOSEFSK;
     for(int k =0; k<10; k++)
          CheckSumValVec[k] = 255;


//////////////////////////////////////////////////////////////////////////////////
//
//   BUFFER INCOMING DATA FROM GNU RADIO AND STORE INTO Stored_bits[];
//
///////////////////////////////////////////////////////////////////////////////////
     //FSK PACKETS COULD BE ARBITRALIY LARGE DUE TO IDLING, BUT WE ASSUME 60000 IS THE MAXIMUM LENGTH OF ONE PACKET
     //GNU RADIO PACKETS CAN BE BETWEEN 2 AND 65536
     //OUR LOCAL BUFFER DOES NOT BEGIN PROCESSING DATA UNTIL IT HAS 60000 BITS IN IT.

     //printf("GNU Radio  next_block_size = %i\n",next_block_size);

     if(next_block_size > 65536)
     {
         printf("\n GNU RADIO has sent a packet of data exceeding 65536, and this packet will not be processed\n");
         printf("Please increase DATA_BUFFER_SIZE in fskimageprocessor.h and recompile the application\n");
         return 0;
     }

    //Store packets into a temporary vault.  Once enough bits are stored, process the packets.
    
     //If you don't have enough bits to process a full packet, wait for more.
    if(stored_bits_count + next_block_size < MINIMUM_BITS)  
          {
               for(int k=0; k<next_block_size; k++)
                    stored_bits[stored_bits_count+k] = ReadBuffer[k];
               stored_bits_count = stored_bits_count + next_block_size;
               return next_block_size;
          }
          else  //You have enough Data. Keep just enough bits to meet 60,000 requirement, then begin processing!
          {
               for(int k=0; k<MINIMUM_BITS-stored_bits_count; k++)
                    stored_bits[stored_bits_count+k] = ReadBuffer[k];
               bits_processed = MINIMUM_BITS-stored_bits_count;  
          }

     block_length = MINIMUM_BITS;

//////////////////////////////////////////////////////////////////////////////////
//
//   DESCRAMBLE DATA AND FLIP THE BITS
//
///////////////////////////////////////////////////////////////////////////////////
  
     //Descramble Data Loop
     for(int j = 0; j<block_length; j++) 
     {
          for(int k = 0; k < NPOLY; k++)
               innerprod += G1[k]*s[k];
          FSK_bits[j] = (innerprod + stored_bits[j]) % 2;
          innerprod = 0;

          //G[1:20] = [Myinput(0) G(1:19)]  %VERIFIED
          for(int k=NPOLY; k > 0 ; k--)
             G1[k] = G1[k-1];
          G1[0] = stored_bits[j];
     } // descramble loop, j variable

     //Flip the bits
     if(flip_sign) {
          for(int j=0; j<block_length; j++) {              
            if(FSK_bits[j]) 
                   FSK_bits[j] = 0;
              else
                   FSK_bits[j] = 1;
                               
          }
     } //flip bits

     //Find Nulls
     //printf("Begin finding HEADER NULLS = %d\n",ii+1);
     while((NullCount < 6) && (ii+11 <  block_length ))
     {
          headersum = 0;
          for(int k=0;k<10;k++)
               if(FSK_bits[k+ii] == header[k])
                    headersum = headersum +  1;
          if(headersum ==10)
          {
               NullCount = NullCount + 1;
               ii = ii+10;
               while(FSK_bits[ii]==1)
                    ii=ii+1;
          }
          else
          {
               ii=ii+1;
               NullCount = 0;
          }
     }
     // if(verbose){printf("NullsFound=%i \tEnd of HEADER NULLS = %d\n",NullCount, ii+1);}
     if (NullCount ==0){
         // Couldn't find any Header. Restart
          stored_bits_count    =0;
         return 0;

     }

     NullCount = 0;
     while(ii+11 <  block_length )
     //while(1)
     {
          headersum = 0;
          for(int k=0;k<10;k++)
               if(FSK_bits[k+ii] == header[k])
                    headersum = headersum +  1;          

          if(headersum ==10)
          {
               NullCount = NullCount +1;
               ii = ii+10;
               while(FSK_bits[ii]==1)
                    ii=ii+1;
          }
          else {break;}
     }
     if ( ii+11 ==block_length){        
        printf("End of Buffer Reached \n");
        return block_length;   
        
     }
     // if(verbose){printf("NullsFound=%i \tEnd of ADDITIONAL HEADER NULLS = %d\n",NullCount, ii+1);}
   
//////////////////////////////////////////////////////////////////////////////////
//
//   PARSE PACKET:  EXTRACT FILE NAMES, BLOCK NUMBERS, TIME STAMPS, Etc.
//
///////////////////////////////////////////////////////////////////////////////////

     AdditionalNulls = NullCount;
     // '/' = 47, 'P' = 80, 'F' = 70, 'N' = 78, 'T' = 84, 'C' = 67, 'S' = 83, 'D' = 68

     // Find /PF marker [File Name]
     getAsynchChar(FSK_bits, ii, block_length,47,Success_NewOffset);
     ii=ii+Success_NewOffset[1];
     getAsynchChar(FSK_bits, ii, block_length,80,Success_NewOffset);
     ii=ii+Success_NewOffset[1];
     getAsynchChar(FSK_bits, ii, block_length,70,Success_NewOffset);
     ii=ii+Success_NewOffset[1];
     //if(verbose){printf("\t End of PF = %d\n", ii+1);}
     
     //Store File name into tempchar
     tempCharLengthFN = 0;
     for(int jj=0;jj<12;jj++)
     {
          NextChar(FSK_bits, ii, block_length, MyChar_Offset);
          tempChar[jj] = MyChar_Offset[0];
          tempCharLengthFN = tempCharLengthFN+1;
          ii = ii+MyChar_Offset[1];
     }


     //printf File Name
     if(verbose){printf("FileName = ");}
     for(int k=0; k<tempCharLengthFN;k++)
     {
          FileName[k] = tempChar[k];
          if(verbose){printf("%c",FileName[k]);}
     }
     if(verbose){printf("\n");}

     // Discard Invalid Files
     int ValidFileFound =0;
     if (  (FileName[8]==46) && (FileName[9]==84) && (FileName[10]==88) && (FileName[11]==84)){ // .TXT 
            ValidFileFound =1;
     }
     if (  (FileName[8]==46) && (FileName[9]==74) && (FileName[10]==80) && (FileName[11]==71)){ // .JPG 
            ValidFileFound =1;
     }
    if (  (FileName[8]==46) && (FileName[9]==71) && (FileName[10]==73) && (FileName[11]==70)){ // .GIF
            ValidFileFound =1;
     }
    if (  (FileName[8]==46) && (FileName[9]==90) && (FileName[10]==73) && (FileName[11]==83)){ // .ZIS 
            ValidFileFound =1;
     }

    if (ValidFileFound ==0){
        stored_bits_count    =0;
        return next_block_size;
    }
   
     
     // Find /PN marker [Block Number] 
     getAsynchChar(FSK_bits, ii, block_length,47,Success_NewOffset);
     ii=ii+Success_NewOffset[1];
     getAsynchChar(FSK_bits, ii, block_length,80,Success_NewOffset);
     ii=ii+Success_NewOffset[1];
     getAsynchChar(FSK_bits, ii, block_length,78,Success_NewOffset);
     ii=ii+Success_NewOffset[1];

     //Store Block number into tempchar
     tempCharLengthBN = 0;
     for(int jj=0;jj<5;jj++)
     {
          NextChar(FSK_bits, ii, block_length, MyChar_Offset);
          tempChar[jj] = MyChar_Offset[0];
          tempCharLengthBN = tempCharLengthBN+1;
          ii = ii+MyChar_Offset[1];
     }

     //Print File Block Number
     if(verbose){printf("Block = ");}
     nn=0;
     for(int k=0; k<tempCharLengthBN;k++)
     {
          BlockNum[k] = tempChar[k];
          if(verbose){printf("%c",BlockNum[k]);}
          if(BlockNum[k] >= 48 && BlockNum[k] <= 57)
          {
               BlockNumValVec[nn] = BlockNum[k]-48;
               nn = nn+1;
          }  
     }
     if(verbose){printf(" of ");}
     

     //Convert Block Number String into an integer
     if(nn==3)
          BlockNumInt = BlockNumValVec[0]*100 + BlockNumValVec[1]*10 + BlockNumValVec[2]*1;
     if(nn==2)
          BlockNumInt = BlockNumValVec[0]*10 + BlockNumValVec[1]*1;
     if(nn==1)
          BlockNumInt = BlockNumValVec[0]*1;

     //Find /PT Marker [Total Block Count] 
     getAsynchChar(FSK_bits, ii, block_length,47,Success_NewOffset);
     ii=ii+Success_NewOffset[1];
     getAsynchChar(FSK_bits, ii, block_length,80,Success_NewOffset);
     ii=ii+Success_NewOffset[1];
     getAsynchChar(FSK_bits, ii, block_length,84,Success_NewOffset);
     ii=ii+Success_NewOffset[1];

     //Store Total block length into tempchar
    tempCharLengthBT = 0;
     for(int jj=0;jj<5;jj++)
     {
          NextChar(FSK_bits, ii, block_length, MyChar_Offset);
          ii = ii+MyChar_Offset[1];
          if(MyChar_Offset[0]>58)
               break;
          tempChar[jj] = MyChar_Offset[0];
          tempCharLengthBT = tempCharLengthBT+1;        
     }

     //Print total block length to screen
     nn=0;
     for(int k=0; k<tempCharLengthBT;k++)
     {
          BlockTot[k] = tempChar[k];
          if(verbose){printf("%c",BlockTot[k]);}
          if(BlockTot[k] >= 48 && BlockTot[k] <= 57)
          {
               BlockTotValVec[nn] = BlockTot[k]-48;
               nn = nn+1;
          }     
     }
     if(verbose){printf("\n");}
     //Convert total block number from string to integer
     if(nn==3)
          BlockTotInt = BlockTotValVec[0]*100 + BlockTotValVec[1]*10 + BlockTotValVec[2]*1;
     if(nn==2)
          BlockTotInt = BlockTotValVec[0]*10 + BlockTotValVec[1]*1;
     if(nn==1)
          BlockTotInt = BlockTotValVec[0]*1;

     //Find /CS marker, [Check Sum]
     getAsynchChar(FSK_bits, ii, block_length,47,Success_NewOffset);
     ii=ii+Success_NewOffset[1];
     getAsynchChar(FSK_bits, ii, block_length,67,Success_NewOffset);
     ii=ii+Success_NewOffset[1];
     getAsynchChar(FSK_bits, ii, block_length,83,Success_NewOffset);
     ii=ii+Success_NewOffset[1];

     //Store Checksum value into tempchar
     tempCharLengthCS = 0;
     for(int jj=0;jj<7;jj++)
     {
          NextChar(FSK_bits, ii, block_length, MyChar_Offset);
          tempChar[jj] = MyChar_Offset[0];
          tempCharLengthCS = tempCharLengthCS+1;
          ii = ii+MyChar_Offset[1];
     }
     
     nn=0;
     for(int k=0; k<tempCharLengthCS;k++)
     {
          CheckSum[k] = tempChar[k];
          if(CheckSum[k] >= 48 && CheckSum[k] <= 57)
          {
               CheckSumValVec[nn] = CheckSum[k]-48;
               nn = nn+1;
          }          
     }
     //Convert Check Sum data from string to integer
     if(nn==7)
          CheckSumInt = CheckSumValVec[0]*1000000 + CheckSumValVec[1]*100000 + CheckSumValVec[2]*10000 + CheckSumValVec[3]*1000 + CheckSumValVec[4]*100 + CheckSumValVec[5]*10 + CheckSumValVec[6]*1;
     if(nn==6)
          CheckSumInt = CheckSumValVec[0]*100000 + CheckSumValVec[1]*10000 + CheckSumValVec[2]*1000 + CheckSumValVec[3]*100 + CheckSumValVec[4]*10 + CheckSumValVec[5]*1; 
     if(nn==5)
          CheckSumInt = CheckSumValVec[0]*10000 + CheckSumValVec[1]*1000 + CheckSumValVec[2]*100 + CheckSumValVec[3]*10 + CheckSumValVec[4]*1;
     if(nn==4)
          CheckSumInt = CheckSumValVec[0]*1000 + CheckSumValVec[1]*100 + CheckSumValVec[2]*10 + CheckSumValVec[3]*1;
     if(nn==3)
          CheckSumInt = CheckSumValVec[0]*100 + CheckSumValVec[1]*10 + CheckSumValVec[2]*1;

     //Find /FD marker [File Date]
     getAsynchChar(FSK_bits, ii, block_length,47,Success_NewOffset);
     ii=ii+Success_NewOffset[1];
     getAsynchChar(FSK_bits, ii, block_length,70,Success_NewOffset);
     ii=ii+Success_NewOffset[1];
     getAsynchChar(FSK_bits, ii, block_length,68,Success_NewOffset);
     ii=ii+Success_NewOffset[1];

     //Store File Date into tempchar
     tempCharLengthMD = 0;
     for(int jj=0;jj<20;jj++)
     {
          NextChar(FSK_bits, ii, block_length, MyChar_Offset);
          tempChar[jj] = MyChar_Offset[0];
          tempCharLengthMD = tempCharLengthMD+1;
          ii = ii+MyChar_Offset[1];
     }
     
     
     for(int k=0; k<tempCharLengthMD;k++)
     {
          MyDate[k] = tempChar[k];
          //if(verbose){printf("%c",MyDate[k]);}
     }
     //if(verbose){printf("\n");}
    // After the date, there's a
    //  SPACE=32decimal. There may be a variable number of these
    //   Carriage Return = 13 decimal
     getAsynchChar(FSK_bits, ii, block_length,13,Success_NewOffset);
     ii=ii+Success_NewOffset[1];
     NextChar(FSK_bits, ii, block_length, MyChar_Offset);
     ii = ii+MyChar_Offset[1];

     DataStart = ii;
     //printf("Start of Information Data = %d\n",DataStart);


/////////////////////////////////////////////////////////////////////////////
//
//   EXTRACT DATA AND PERFOM CHECKSUM TO FIND IF DATA HAS ERRORS
//
/////////////////////////////////////////////////////////////////////////////


     //START POPULATING DATA
     for(int k=0;k<1024;k++)
          MyData[k] = -1;
     jj=0;

     //BEGIN PROCESSING THE 1024 DATA:  BASED ON THIS BEING THE LAST BLOCK FOR A GIVEN FILE
     if(BlockNumInt == BlockTotInt) 
     {
          NullCount = 0;
          while((NullCount<6)&&(jj<1024))
          {
               NextChar(FSK_bits, ii, next_block_size, MyChar_Offset);
               ii=ii+MyChar_Offset[1];
               MyData[jj] = MyChar_Offset[0];
               jj=jj+1;
               if(MyChar_Offset[0]==0)
                    NullCount = NullCount+1;
               else
                    NullCount = 0;
          }
          MyDataLength = jj-NullCount;  //Not sure if its jj-NullCount-1;
     }
      //BEGIN PROCESSING THE 1024 DATA:  BASED ON THIS NOT BEING THE LAST BLOCK
     if(BlockNumInt != BlockTotInt)
     {     
          while(jj<1024) //j<=1024???
          {
               NextChar(FSK_bits, ii, block_length, MyChar_Offset);
               ii=ii+MyChar_Offset[1];
               MyData[jj] = MyChar_Offset[0];
               jj=jj+1;
          }
          MyDataLength = 1024;
     }

    //  EMWIN SPEC:
    // - The data, as a 1024-byte block; if the remaining data of the product
    // is less than 1024 bytes, this block is null-filled so that each packet's
    // data block is always 1024 bytes long.
    //-6 bytes of ASCII 0 (nullNUL) to clear the receiver buffer.

    
     // BEGIN PROCESSING THE 1024 DATA: ASSUMING ZIS FILE
     NullCount = 0;
     NameCharCount = 0;
     FoundName = 0;
     if(MyDataLength < 1024) //should it be MyDataLength < 1023?
     {
          //----------
          // ZIS FILES
          //----------
          if(FileName[9] == 90 && FileName[10] == 73 && FileName[11] == 83)
          {       
            // These files are processed differently
            // Step 1: Look for FILENAME.TXT in the remaining bytes
            while((NullCount < 6)&&(jj<1023)) 
            {
                 NextChar(FSK_bits, ii, next_block_size, MyChar_Offset);
                 ii=ii+MyChar_Offset[1];
                 MyData[jj] = MyChar_Offset[0];
                 jj=jj+1;
                 if(MyChar_Offset[0] == FileName[NameCharCount]) 
                 {
                      NameCharCount = NameCharCount+1;
                      if(NameCharCount==8) 
                           FoundName=1;
                 }
                 else
                      NameCharCount = 0;
                 if((FoundName==1)&&(MyChar_Offset[0]==0))
                      NullCount = NullCount + 1;
                 else
                      NullCount = 0;
            }
            MyDataLength = jj-NullCount; 
          }
     }

     //CHECK SUM ANALYSIS
     TxCheckSum = CheckSumInt;

     //Compute Check sum by adding up all ones in the datastream
     val = 0;
     if(MyDataLength < 1024)
          for (int k=0; k< MyDataLength; k++)
               val = val+MyData[k];
     else
          for (int k=0; k<1024; k++)
               val = val+MyData[k];
     
     RxCheckSum = val;
     //if(verbose){printf("Check Sum: Tx=%d, Rx=%d tMatch=%d\n", TxCheckSum, RxCheckSum, TxCheckSum==RxCheckSum);}
     
     if ((TxCheckSum!=RxCheckSum )&&(TxCheckSum!=0) ){
         // Error in Checksum
        if(verbose){printf("Check Sum: Tx=%d, Rx=%d \t Check Difference=%d \n", TxCheckSum, RxCheckSum, abs(TxCheckSum-RxCheckSum) );}
     }

///////////////////////////////////////////////////////////////////////////////
//
//  DATA MANAGEMENT:  STORE DATA INTO TEMPORARY ARRAYS UNTIL WHOLE FILE CAN BE WRITTEN
//
////////////////////////////////////////////////////////////////////////////////

     //PACKET IS ERROR FREE
     if(TxCheckSum==RxCheckSum)
     {
          Consecutive_Frame_Errors = 0;  
          GoodFrameCount = GoodFrameCount+1;
          FileIndex = -1;
          FirstAvailable = -1;
          //Check to see if current file already have blocks that have been received already
          //Or if this file is the first block. 
          for (int jj=MAXFILES-1; jj>=0; jj--) 
          {
               val=0;
               for(int k=0;k<12; k++)
                    if(FileName[k]==AvailIndex[jj][k])
                         val = val+1;
               if(val==12) //If the File Name Matches
               {
                    FileIndex = jj; 
                    break;
               }
               else if(val==0) //If the File Name doesn't match Matches
               {
                    FirstAvailable =jj; 
               }
              
          }

          if((FirstAvailable==-1) && (FileIndex == -1))
          {
               printf("Error: THERE IS NO ROOM TO STORE PARTIAL DATA!\n");
               printf("Error: ONLY %d PARTIAL FILES CAN BE STORED AT A TIME\n",MAXFILES);
               return 0;
          }

          if(FileIndex >=0){
          //Continuation of a previous file
          } 

          //New File
          else 
          {
               if (FirstAvailable >=0)
               {
                    // Space Available
                    for(int k=0; k<12; k++)
                         AvailIndex[FirstAvailable][k]= FileName[k];
                    for(int k=0; k<20; k++)
                         AvailIndex[FirstAvailable][k+12]= MyDate[k];  
                    FileIndex = FirstAvailable;
               }
               else
               {
                  FileIndex = 0;
                //Our array is full, possibly due to a bunch of incomplete
                //files. Look for the older file and throw it away
               }
          } //IF New File Found 
          

          //Copy Data into Storage Array
          for(int k = 0; k<MyDataLength;k++)
               FSK_files[FileIndex][BlockNumInt-1][k] = MyData[k];
          PartsRx[FileIndex][BlockNumInt-1] = MyDataLength;  //Make sure FileIndex is correct
     
          
          //IF THIS IS THE LAST DATA SEGMENT
          if(BlockNumInt == BlockTotInt)
          {
               PartsFound = 0;
               for(int jj =0; jj<BlockTotInt; jj++)
                    if(PartsRx[FileIndex][jj]>0)
                         PartsFound = PartsFound+1;

               //IF FILE IS COMPLETE
               if(PartsFound == BlockTotInt)
               {
                    //ZIS File Is Actually a ZIP file
                    if(FileName[9] == 90 && FileName[10] == 73 && FileName[11] == 83) //If ZIS file
                         FileName[11] = 80; //'P'

                    //OPEN FILE FOR WRITING
                    for(int k=0; k<12; k++)  //This is HARDCODED at 12 for File Name LENGTH!!!
                         FileNameChar[k] = FileName[k]&0x00FF;
                    FileNameChar[12] = '\0';
                    strcpy(FilePath,FSK_PATH); 
                    strcat(FilePath, FileNameChar); 
                    printf("File Extracted %s\n", FilePath);
                    FSKFile = fopen(FilePath,"ab");
                   
                    if(FSKFile==NULL)
                    {
                         printf("ERROR!!!!\n");
                         printf("THERE WAS AN ERROR ATTEMPTING TO OPEN %s\n",FilePath);
                         printf("Please make sure the directory currently exists!\n");
                         return 0;
                    }

                    //WRITE FILE TO DISK
                    else   
                    {
                         val=0;
                         for(int jj=0;jj<BlockTotInt; jj++)
                         {
                              for(int k=0; k<PartsRx[FileIndex][jj]; k++)  //k=1:MyDataLength for a particular block
                              {
                                   binarydatatowrite[val] = FSK_files[FileIndex][jj][k]&0x00FF; //Store data into a BYTE array
                                   val = val+1;
                              }
                              PartsRx[FileIndex][jj] = 0; 
                              for(int k=0; k<MAXLENGTH; k++)  
                                   FSK_files[FileIndex][jj][k] = 0;  
                         }
                         fwrite(binarydatatowrite, 1, val, FSKFile); //Write entire File at once
                         for(int k=0; k<33; k++)
                              AvailIndex[FileIndex][k] = 0;
                         fclose(FSKFile);
                    } //End If FILE Opened without error   
               } //End If File Is Complete
          
               //FILE IS IMCOMPLETE: DON'T WRITE FILE TO DISK
               else  
               {
                    for(int k=0; k<12; k++)  //This is HARDCODED at 12 for File Name LENGTH!!!
                         FileNameChar[k] = FileName[k]&0x00FF;
                    FileNameChar[12] = '\0';
                    printf("ERROR! %s is incomplete!\n", FileNameChar);
                    printf("Only %d of %d parts were received\n", PartsFound, BlockTotInt);
                    for(int k=0; k<33; k++)
                         AvailIndex[FileIndex][k] = 0;
                    for(int k=0; k<MAXPARTS; k++)
                         PartsRx[FileIndex][k] = 0;

                    for(int jj=0; jj< MAXPARTS; jj++)
                         for(int k=0; k<MAXLENGTH; k++)
                             FSK_files[FileIndex][jj][k] = 0;
               }
          } //End If Current Block is the Last Block
     } //End If Check Sum Passed
     
       //FILE ERROS DETECTED: LOG ERROR, DROP PACKET
     else  //CheckSum Did Not Pass
     {
          for(k=0; k<12; k++)  //This is HARDCODED at 12 for File Name LENGTH!!!
               FileNameChar[k] = FileName[k]&0x00FF;
          FileNameChar[12] = '\0';
          Consecutive_Frame_Errors++; //ReAcq Hook
          
          FrameErrorCount++;

          printf("Frame Error Count = %i \t ", FrameErrorCount);
          printf("Total Frame Count = %i \n ", FrameErrorCount + GoodFrameCount);

          
          //EV: Better to open and close within a function to avoid crashing
          FER_File = fopen(BERFileName,"a");
          if(FER_File==NULL){
               printf("Could Not Open BER log file name\n");
               exit;

          }else{          
            fprintf(FER_File, "FER = %4.3e \t ErrFrames=%i \t Frames=%i\n\n",
              float(FrameErrorCount)/ ( float(GoodFrameCount)+ float(FrameErrorCount) ),FrameErrorCount,GoodFrameCount+FrameErrorCount);

             fclose(FER_File);
          }

          sprintf(MyText, "The File %s, part %d of %d,has Errors.\nTx Check Sum = %d,   Rx Check Sum = %d\n----------------------------------------\n",
              FileNameChar, BlockNumInt, BlockTotInt, TxCheckSum, RxCheckSum);
          printf("%s\n",MyText); 

          //fwrite(MyText, 1, 300, BERFILE); 

          FER_File = fopen(BERFileName,"a");
          if(FER_File==NULL){
               printf("Could Not Open FER log file name\n");
               exit;

          }else{          
            fprintf(FER_File, MyText);
            fclose(FER_File);
          }

          //fclose(FSK_ErrorFile);

          // CheckSum was not correct
          // Delete File from Archive
          FileIndex = -1;
          FirstAvailable = -1;
          for(int jj=MAXFILES-1; jj>=0; jj--)
          {
               val=0;
               for(int k=0; k<12; k++)
                    if(FileName[k]==AvailIndex[jj][k])
                         val=val+1;
               if(val==12) //Found a match
               {
                    FileIndex = jj;
                    break;
               }
               else if(val==0)
                    FirstAvailable==jj;
          }

          if(FileIndex>=0)
          {
               printf("File With Errors\n %s. Part %d of %d failed\n", FileNameChar, BlockNumInt,BlockTotInt);
               for(int k=0; k<33; k++)
                    AvailIndex[FileIndex][k] = 0;
               for(int k=0; k< MAXPARTS; k++)
                    PartsRx[FileIndex][k] = 0;
               for(int jj=0; jj<MAXPARTS;jj++)
                    for(int k=0; k<MAXLENGTH; k++)
                         FSK_files[FileIndex][jj][k] = 0;
          }
     } //END IF CHECK SUM FAILED
     
   
     if(verbose){printf("------------------------------\n");}
     
     
     if (GoodFrameCount % 500 == 0)
     {
          printf("FSK EMWIN: Error-Free Frames=%d\t Err.Frames=%d \t", GoodFrameCount,FrameErrorCount);
          printf("FER=%e\n", float(FrameErrorCount)/ ( float(GoodFrameCount)+ float(FrameErrorCount) ));

         // FER_File = fopen(FER_LOG_PATH,"a");
         
          /*if(BERFILE != NULL) 
          fprintf(BERFILE, "FER = %4.3e \t ErrFrames=%i \t Frames=%i\n",
              float(FrameErrorCount)/ ( float(GoodFrameCount)+ float(FrameErrorCount) ),FrameErrorCount,GoodFrameCount+FrameErrorCount);
            */
              FER_File = fopen(BERFileName,"a");
          if(FER_File==NULL){
               printf("Could Not Open FER log file name\n");
               exit;

          }else{          
            fprintf(FER_File, "FER = %4.3e \t ErrFrames=%i \t Total Frames=%i\n",
              float(FrameErrorCount)/ ( float(GoodFrameCount)+ float(FrameErrorCount) ),FrameErrorCount,GoodFrameCount+FrameErrorCount);
            fclose(FER_File);
          }

        //  fclose(FER_File);

     }

      
      //printf("Last Processed Bit %d\n", ii);

      //Assume ii marker for the current bit that needs processing!  That means ii-1 is the last bit CONSUMED!
     stored_bits_count = MINIMUM_BITS-ii; 
  
     //Send FSK Data over a Socket
  if(USE_SOCKET_GBL>0){
     //Convert the Asynchronous bitsteam into a synchronous byte-stream
     int total_asynch_bits = ii;  //The total number of asynchronous bits that were processed during this function call
     int asynch_bit = 0; //Current asynchronous bit marker of FSK_bits (raw derandomized demodulated data) 
     int BB = 0; //Currenty Byte market for socket bytestream
     while(asynch_bit < total_asynch_bits)
     {
          NextChar(FSK_bits, asynch_bit, next_block_size, MyChar_Offset);
          asynch_bit=asynch_bit+MyChar_Offset[1];
          FSK_bytes[BB] = MyChar_Offset[0] & 0x00FF;
          BB=BB+1;
          //printf("%x",FSK_bytes[BB]);
     }
     int socket_byte_length = BB;
     //Send data over a socket
     send(new_connection, (char *)FSK_bytes, socket_byte_length, 0); 
  }



//Before starting over, re-seed the randomizer by storing the last 20 bits     
     for(int k=0; k<20; k++){
          G1[k] = stored_bits[ii-1-k];
     }

     //Copy the unprocessed data to the front of the array to start processing at the beginning of stored_bits during next function call. 
     for(int k=0; k<stored_bits_count; k++)
          stored_bits[k] = stored_bits[k+ii];

      
     return bits_processed;
} //END ProcessFSK
  

void getAsynchChar(int *invec, int start, int stop, int MyCharInt, int *Succ_NewOffset)
{
     int Nobits = stop-start+1;
     int ii=0;
     int BIN[8] = {128, 64, 32, 16, 8, 4, 2, 1} ;
     int MyBits[8] = {0,0,0,0,0,0,0,0};
     int tempChar_int = 0;
     int val=0;
     int zz;
     int tempOffset;


     while(ii+1<Nobits)
     {
          zz=1;
          for(int k=7; k>=0; k--)
               MyBits[7-k] = invec[start+zz+k+ii]; 
            
          zz = zz+8;
          while((invec[start+zz+ii]==1))
               zz=zz+1;
          //Bin 2 dec
          val =0;
          for(int bin=0;bin<8;bin++)
               if(MyBits[bin] ==1)
                    val = val+BIN[bin];

          tempChar_int = val;
          tempOffset = zz; 

          if(tempChar_int==MyCharInt)
          {
               ii=ii+tempOffset;
               Succ_NewOffset[0] = 1;
               break;
          }
          else
               ii=ii+tempOffset;
     }
     Succ_NewOffset[1] = ii; 
}

void NextChar(int *invec, int start, int stop, int *MyChar_Off)
{
     int ii=1;
     int MyBits[8];
     int BIN[8] = {128, 64, 32, 16, 8, 4, 2, 1} ;
     int val=0;

     for(int k=7;k>=0;k--)
          MyBits[7-k] = invec[start+ii+k];

     for(int bin=0;bin<8;bin++)
          if(MyBits[bin] ==1)
               val = val +BIN[bin];
     MyChar_Off[0] = val;

     ii=ii+8;
     while((invec[start+ii]==1))
          ii=ii+1;
     MyChar_Off[1] = ii;
}


