/*GOES Radio Comments
This is the top level file for the FSK demodulator
It calls usrp_source_c to instantiate the hardware, freq_acq, to detect
the frequency offset, and calls the fsk_rx class for demodulation and display. 
*/

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

// AeroRadio
#include <ar_throughput.h>
#include "gnuradio.h"
#include "fsk_modem.h"
//#include "freq_acq_bfsk.h"
#include "gnuradio.h"

//GNU Radio
#include <gr_single_threaded_scheduler.h>
#include <gr_block_detail.h>
#include <gr_file_sink.h>
#include <gr_file_source.h>
#include <gr_head.h>
#include <usrp1_source_c.h>
#include <gr_null_sink.h>
// General C++
#include <iostream>
#include <time.h>
using namespace std;


#include "freq_acq.h"

//#define THROUGHPUT
//#undef THROUGHPUT

//------------------------------------------------------------------------------------
//Variables for data link layer to user, FSK_ImageProcessor.cpp
int Global_Consecutive_Frame_Errors=0; //Used For Reacquisition Purposes
int Global_Consecutive_Frame_Errors_Limit=20;
int Global_Flip_Bits=1;                //Used so GUI can select Polarity of FSK Demod
//------------------------------------------------------------------------------------

int main(int argc, char* argv[])
{

    //--------------------------------------------------------------
    // Read Command Prompt Input Arguments
    int l;
    // Search for switches, one loop for each switch
    // WARNING !!!! Remember that the ETTUS Box has an embedded filter at
    // Fc/FS or so, which can really affect the input signal !
    //
    // USRP:
    //   DR     Samples Per Symbol at 9600 bps
    //   66        101.0101
    //   74         90.0901  --> Works really well
    //   98         68.0272  --> works well
    //  136         49.0196  --> Works noisy

    // AeroID:
    //   DR     Samples Per Symbol at 9600 bps
    //   50        100 sps
    //   100       50  sps
    //   200       25  sps

    //---------------------------------------------------------------------
     
    
    //-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+--+-+-+-+-+-+-+-+-+--++-+-+-+-+-+-+-+-+-+--+-+
    //--------------------------------------------------------------

    // Read Command Prompt Input Arguments

    //---------------------------------------------------------------
    //-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+--+-+-+-+-+-+-+-+-+--++-+-+-+-+-+-+-+-+-+--+-+

    //help function
    for (l = 1; l < argc; l++ ) {
        if (!strcmp(argv[l],"help")) {
             printf("\n\n\nThis Application Demodulates the FSK EMWIN-I Signal from GOES Using the Following Options\n\nflags\t\t default\t description\n------\t\t --------\t -------------\n\n-socket\t\t 0\t\t Socket port number for which EMWIN data is sent out over localhost\n");
             printf("-wr\t\t 0\t\t Flag that writes input USB data to a file temp\\usrp_log\\usrp_in.txt\n");
             printf("-rec\t\t Null\t\t Processes data from the file [FileName] instead of using the USB\n");
             printf("-repeat\t\t 1\t\t Flag that specifies if the input data file will be looped back, or processed just once\n");
             printf("-sink\t\t 0\t\t Flag that writes FSK tracked constellation complex data points to temp\\usrp_log\\timing_tracked.dat\n");
             printf("-fc\t\t 137.225e6\t Input IF Center Frequency of the EMWIN-I Signal at the input to the IF Digitizer\n");
             printf("-br\t\t 9600\t\t Baud rate of the Signal\n");
             printf("-dr\t\t 66\t\t Decimation ratio for the IF Digitizer\n");
             printf("-delta\t\t 3600\t\t Frequency separation of 2-lvl FSK carriers\n");
             printf("-roll\t\t 0.5\t\t Root Raised Cosine Receiver Pulse Shaping Filter roll-off (alpha) factor\n");
             printf("-acq\t\t 1\t\t Flag that turns the coarse frequency acquisition engine off/on\n");
             printf("-foff\t\t 0\t\t Manual Initial Frequency Offset Adjustment in Hz\n");
             printf("-fa_numfft\t 50\t\t Number of FFT averages on spectrum during frequency acquisition\n");
             printf("-fa_res\t\t 100\t\t Spectral Resolution in Hz during frequency acquisition process\n");
             printf("-fa_range\t 50000\t\t Range of frequencies to search over during frequency acquisition\n");
             printf("-fa_2\t\t 0\t\t Flag that uses the 2nd order frequency acquisition algorithm (To set 1st order, set 2nd and 4th order to 0)\n");
             printf("-fa_4\t\t 0\t\t Flag that uses the 4th order frequency acquisition algorithm (4th order requires 2nd order to be 1)\n");
             printf("-fa_fc\t\t 0\t\t Frequency Offset in Hz performed before conducting frequency acquisition\n");
             printf("-fa_delay\t 101\t\t Delay in samples of the Frequency Acquisition delay-and-multiply (squaring)algorithm\n");
           return 0;
        }
    }

    char *in_socket = "0";  //Option to dump data out to a socket
    for (l = 1; l < argc; l++ ) {
        if (!strcmp(argv[l],"-socket")) {
            in_socket                = argv[l+1];
        }
    }

    //USRP FPGA DDFS center frequency
    char *in_fc = "137.225e6";  //FSK
    for (l = 1; l < argc; l++ ) {
        if (!strcmp(argv[l],"-fc")) {
            in_fc                = argv[l+1];
        }
    }

    //Manual Frequency Offset
    char *in_foff = "0.0";
    for (l = 1; l < argc; l++ ) {
        if (!strcmp(argv[l],"-foff")) {
            in_foff                = argv[l+1];
        }
    }

    //Manual Frequency Offset fudge factor to account for oscillator offset
    char *in_foff_AID = "18200";
    for (l = 1; l < argc; l++ ) {
        if (!strcmp(argv[l],"-faid")) {
            in_foff_AID       = argv[l+1];
        }
    }

    //Flip the Polarity of the signal
    char *flip_sign = "1";
    for (l = 1; l < argc; l++ ) {
        if (!strcmp(argv[l],"-flip")) {
            flip_sign                = argv[l+1];
        }
    }

    //This flag writes USB data direclty to a file
    char *write_input = "0";
    for (l = 1; l < argc; l++ ) {
        if (!strcmp(argv[l],"-wr")) {
            write_input     = argv[l+1];
        }
    }

    //Timing Acquisition Variable for ar_fsk_demod_cf()
    char *acq_alpha = "1e-4";
    for (l = 1; l < argc; l++ ) {
        if (!strcmp(argv[l],"-falpha")) {
            acq_alpha     = argv[l+1];
        }
    }
    //Timing Acquisition Variable for ar_fsk_demod_cf()
    char *acq_beta = "0";
    for (l = 1; l < argc; l++ ) {
        if (!strcmp(argv[l],"-fbeta")) {
            acq_beta     = argv[l+1];
        }
    }

    /////FREQ ACQ PART
    char *in_num_fft_avg = "50";      // FRequency Acquisition: number off FFT averages on spectrum
    for (l = 1; l < argc; l++ ) {
        if (!strcmp(argv[l],"-fa_numfft")) {
            in_num_fft_avg= argv[l+1];
        }
    }

    char *in_resolution = "100";      // FRequency Acquisition: Spectral Granularity
    for (l = 1; l < argc; l++ ) {
        if (!strcmp(argv[l],"-fa_res")) {
            in_resolution= argv[l+1];
        }
    }

    char *in_freq_range = "50000";      // FRequency Acquisition: Range of Frequencies to search over
    for (l = 1; l < argc; l++ ) {
        if (!strcmp(argv[l],"-fa_range")) {
            in_freq_range= argv[l+1];
        }
    }

    char *in_square_law = "0";      // FRequency Acquisition: Square Law Detector
    for (l = 1; l < argc; l++ ) {
        if (!strcmp(argv[l],"-fa_2")) {
            in_square_law= argv[l+1];
        }
    }

    char *in_fourth_law = "0";      // FRequency Acquisition: Fourth Law Detector
    for (l = 1; l < argc; l++ ) {
        if (!strcmp(argv[l],"-fa_4")) {
            in_fourth_law= argv[l+1];
        }
    }

    char *in_freq_acq_fc = "0";      // FRequency Acquisition: Internal Mixer (Separate from built in Usrp Mixer!)
    for (l = 1; l < argc; l++ ) {
        if (!strcmp(argv[l],"-fa_fc")) {
            in_freq_acq_fc= argv[l+1];
        }
    }
    char *repeat    = "0";    //Option to Repeat File Source
    for (l = 1; l < argc; l++ ) {
        if (!strcmp(argv[l],"-repeat")) {
            repeat = argv[l+1];
        }
    }


     //Read data from a file source instead a USB
    char *in_rec_data_file    = "C:/temp/usrp_log/fsk_usrp_in.txt";
    int  using_rec_data       =0;
    for (l = 1; l < argc; l++ ) {
        if (!strcmp(argv[l],"-rec")) {
            using_rec_data           =1;
            in_rec_data_file         = argv[l+1];
        }
    }

    //USRP FPGA decimation rate
    char *in_decim_rate = "66";    // For 64 Mhz
    for (l = 1; l < argc; l++ ) {
        if (!strcmp(argv[l],"-dr")) {
            in_decim_rate = argv[l+1];
        }
    }

    // Obsolete
    char      *using_icy      ="0";
    double    icyFreqMult     =1.0;;
    for (l = 1; l < argc; l++ ) {
        if (!strcmp(argv[l],"-icy")) {
            using_icy= argv[l+1];
        }
    }


 //Flag to turn the Initial Coarse Frequency Acquisiton Engine on/off
    char *ACQEngine = "1";      // OFF by default
    for (l = 1; l < argc; l++ ) {
        if (!strcmp(argv[l],"-acq")) {
            ACQEngine= argv[l+1];
        }
    }

    char *in_DeltaF = "3600";      // Freq. Separation for FSK
    for (l = 1; l < argc; l++ ) {
        if (!strcmp(argv[l],"-delta")) {
            in_DeltaF= argv[l+1];
        }
    }
    char *in_bitrate = "9600";      // BITRATE
    for (l = 1; l < argc; l++ ) {
        if (!strcmp(argv[l],"-br")) {
            in_bitrate= argv[l+1];
        }
    }

       //Timing Acquisition Variable for ar_fsk_demod_cf()
    char *lock_threshold = "3.0";
    for (l = 1; l < argc; l++ ) {
        if (!strcmp(argv[l],"-lck_th")) {
            lock_threshold                = argv[l+1];
        }
    }

        if (using_rec_data==0){
        printf( "==========================\n");
        printf("  Tracking LIVE SATELLITE    \n");
        if (atoi(using_icy)==0)
            printf("  Using 64 MHZ Aerospace AID Box    \n");
        else
            printf("  Using 48 MHZ Aerospace AID Box    \n");

        printf(" Fc = %10.8e   \n",atof(in_fc));
        printf("===========================\n\n");
    }else{
        printf("===========================\n\n");
        printf("  Tracking RECORDED SATELLITE    \n");
        if (atoi(using_icy)==0)
            printf("  Using ETTUS BOX    \n");
        else
            printf("  Using AERO-ICY BOX    \n");
        printf("===========================\n\n");
    }

    //--------------------------------------------------------------
    // Container for all the blocks
    vector<gr_block_sptr> blocks;
    //------------------------------------------------------------------------
    //             --- USRP CONFIGURATION ----
    int    clock_rate       =64e6;
    int    mirror_factor    =-1;
    // unsigned int	decim_rate     = 136;   //--> Fails at 136
    unsigned int	decim_rate     = atoi(in_decim_rate);
    int			nchan		    = 1;
    int			mux			    = 1;
    int			mode		    = 0;
    int			which_board	    = 0;
    float		fc		        = atof(in_fc);
    bool        RunACQEngine = atoi(ACQEngine);
    unsigned int   NoSamples    = 1e4;
    double		foff		    = atof(in_foff)+atoi(using_icy)*atof(in_foff_AID);
    // Effective Sampling Rate = 64e6/decim_rate = 256 kbps
    //------------------------------------------------------------------------
    fsk_config config;
    config.tones.resize(2);            // Only 2 tones
    float   Fs                = 0;          // Sampling frequency [Hz]
    int     lock_after             = 4;
    int     unlock_after           = 15;

    //********************************************************************************************
    // DEFINE INPUT FILE
    char 		        *RecDataFile;
    RecDataFile         =(char *) calloc(200,sizeof(char));
    strcpy(RecDataFile,in_rec_data_file);

    //-----  DETERMINE USRP FPGA DDSF FREQUENCY Based on IF-Sampling Principle and IF Center FRrequency  ------
    double mirror_fc =fc;
    if (atoi(using_icy) ==0){ //USRP
        clock_rate=64e6;
        Fs		 = 64e6/decim_rate;
        for  (int ii=0; ii< 5;ii++){
            if (fabs(fc - ii*64e6) < mirror_fc){
                mirror_fc = fabs(fc - ii*64e6);
                mirror_factor =ii;
            }
        }
    }else{//OBSOLETE
        clock_rate=48e6;
        icyFreqMult     = 64.0/48.0;
        Fs		 = 48e6/decim_rate;
        for  (int ii=0; ii< 5;ii++){
            //printf( " ICY Mirror %i Fc= %4.3e \n", ii, fabs(fc - ii*48e6)*icyFreqMult );
            //printf( " ICY Mirror %i Fc= %4.3e \n", ii, fabs(fc - ii*48e6) );
            if (fabs(fc - ii*48e6)*icyFreqMult < mirror_fc){
                mirror_fc = fabs(fc - ii*48e6)*icyFreqMult;
                mirror_factor =ii;
            }
        }
    }
    printf( " Mirror Fc= %4.3e \n", mirror_fc );
    fc             =mirror_fc;
    //-----------------------------------------------------------------------------
    // FSK demodulator
        //              --- FSK Demodulator Values ---
    // EMWIN-I:    Fc                       = 137.225Mhz.
    //   	     Data rate                = 9.6 ksps.
    //             Frequency Separation     = 3.6 kHz.
    // Sampling at 64 MHz --> 2nd Harmonic at 9.2250 Mhz
    //             f0                       =  7.425 MHz
    //             f1                       = 11.025 MHz
    // Sampling at 48 MHz --> 2nd Harmonic at 6.7750 Mhz
    //             f0                       =  4.975 MHz
    //             f1                       =  8.575 MHz
    float     Tb                  = 1.0/atof(in_bitrate);    // bit duration [s] = 1.0417e-004 [sec]
    float     DeltaF              = atof(in_DeltaF);
    int       fsk_decim           = 1;
    float     samples_per_symbol  = Fs * Tb;
    float     Fs_decim            = Fs / fsk_decim;  // after first decimation

    //
    config.aid_box      =atoi(using_icy)==1;
    config.Fs           = Fs;
    config.tones[0]     =  0 -DeltaF;
    config.tones[1]     =  0 +DeltaF;   // Separation = 6720 Hz
    config.Tb           =  Tb;  // bit duration [s]
    config.rz           = false;
    config.double_sided = false;
    config.alpha        =atof(acq_alpha);
    config.beta         =atof(acq_beta);
    config.flip_sign    =atoi(flip_sign);
    printf("FSK: \n\t fc=%5.4e\n \t f0 = %4.3e\t  f1 = %4.3e \n \t Fs = %4.3e \t Sps = %4.3f\n\n",
           fc, config.tones[0], config.tones[1], Fs, samples_per_symbol);

   

  
    //===============================================================================
    // Frequency Acqusition
    //===============================================================================
    if (RunACQEngine==1) {

        printf( "======================\n");
        printf( "  ACQ-Engine ENABLED  \n");
        printf( "======================\n");
        freq_acq_config                    freq_acq_config0;
        freq_acq_config0.Fs                =  config.Fs;
        freq_acq_config0.delay             = 0;  //Delay by one symbol
        freq_acq_config0.Fc                = atof(in_freq_acq_fc);  //Obsolete mixer always set to 0!
        freq_acq_config0.num_fft_avg       = atof(in_num_fft_avg);
        freq_acq_config0.resolution        = atof(in_resolution);
        freq_acq_config0.freq_range        = atof(in_freq_range);
        freq_acq_config0.square_law        = (bool)atoi(in_square_law);
        freq_acq_config0.fourth_law        = (bool)atoi(in_fourth_law);
        if (freq_acq_config0.fourth_law){
            freq_acq_config0.delay        = 0; //For the moment, 4th law only works with delay = 0;
            freq_acq_config0.num_fft_avg  = 16; //For the moment, 4th law doesn't allow more than 16 averages
        }

        // USRP SETUP
        freq_acq_config0.usrp_which_board    = which_board;
        freq_acq_config0.usrp_decim_rate     = decim_rate;
        freq_acq_config0.usrp_nchan          = nchan;
        freq_acq_config0.usrp_mux            = mux;
        freq_acq_config0.usrp_mode           = mode;
        freq_acq_config0.usrp_fc             = fc;
        //freq_acq_config0.lrit_filter	     = lrit_filter_on;

        freq_acq_config0.rec_data          = using_rec_data; //JR Recorded data flag
        if (using_rec_data) 
        {
            strcpy(freq_acq_config0.filename, in_rec_data_file);
        }
        freq_acq acq(freq_acq_config0, false); // no hardware
        // Begin ACQ
        acq.run();
        Sleep(2000);
        freq_acq_result result_scalar;
        vector<freq_acq_result> result_vector;
        float acquired_carrier;
        float carrier_power;

        //IF you are using squre law detector
        if (freq_acq_config0.square_law && !freq_acq_config0.fourth_law)
        {
            //printf("Using a Square Law Frequency Acquisition Engine\n");
            // Wait for Acquistion to finish
            while (!acq.get_freqs(2,result_vector)) Sleep(1);

            float peak1 = result_vector[0].freq;
            float peak2 = result_vector[1].freq;
            //The second peak, might be still represent same peak.  Might need to grab 4 peaks, evaluate them to make sure they are distinct.
            carrier_power = min(result_vector[0].relative_power, result_vector[1].relative_power);
            if (peak1>peak2)
                acquired_carrier = (peak2-peak1)/2+peak1;
            else
                acquired_carrier = (peak1-peak2)/2+peak2;
            printf("Acquired Peak: Freq = %f, pwr = %f\n", acquired_carrier,carrier_power);
        }
        //IF you are using fourth law detector
        else if (freq_acq_config0.fourth_law)
        {
            printf("Using a Fourth Law Frequency Acquisition Engine\n");
            // Wait for Acquistion to finish
            while (!acq.get_peak_freq(result_scalar)) Sleep(1);
            acquired_carrier  = result_scalar.freq;
            carrier_power = result_scalar.relative_power;
            printf("Acquired Peak: Freq = %f, pwr = %f\n", acquired_carrier,carrier_power);
        }
        else   //1st order Frequency Acquisition Law detector
        {
            while (!acq.get_freqs(10,result_vector)) Sleep(1);

            float peak1 = result_vector[0].freq;
            float peak2=0;
            int ii=0;
            while (ii<10){
                //printf(" Freq Peak %i = %6.3e \n",ii,result_vector[ii].freq );
                if (fabs(result_vector[ii].freq -peak1) < 1.5*DeltaF ){
                    ii++;
                }else{
                    peak2 = result_vector[ii].freq;
                    ii=10;
                }
            }
            carrier_power = min(result_vector[0].relative_power, result_vector[1].relative_power);
            if (peak1>peak2)
                acquired_carrier = (peak2-peak1)/2+peak1;
            else
                acquired_carrier = (peak1-peak2)/2+peak2;
            printf("Acquired Peak: Freq = %f, pwr = %f\n", acquired_carrier,carrier_power);

        }

        //Update Frequency Offset based on acquired carrier frequency.
        foff += -acquired_carrier;
        acq.stop();
    }  //end Acq

    printf("foff = %g\n", foff);

    //===============================================================================

    printf("\nRF Spectrum Information: \n");
    printf("RF Input: \t\t f=%6.4e\n",fc/icyFreqMult + mirror_factor*clock_rate  );
    printf("RF Offset: \t\t f=%6.4e\n", foff );
    printf("RF OUTPUT: \t\t f=%6.4e\n", (fc/icyFreqMult-foff)+ mirror_factor*clock_rate );
    //--------------------------
    //   ICY BOX
    //--------------------------
    foff = foff*icyFreqMult;

   
    //--------------------------
    //   Bit Error Rate Log
    //--------------------------
    char dateStr [9];
    char timeStr [9];
    _strdate( dateStr);
    printf( "The current date is %s \n", dateStr);
    _strtime( timeStr );
    printf( "The current time is %s \n", timeStr);

    FILE		* BERpFile; 
    char 	    *BERDataFile;
    BERDataFile         =(char *) calloc(200,sizeof(char));
    strcpy(BERDataFile,"c:/EMWINTEMP/FSK_error.txt");
    BERpFile = fopen(BERDataFile , "a+" );  
    if ( (BERpFile==NULL) ) {
       printf("\n\nUnable to open a BER log file!!!!\n This run will not be logged!!!!"); 
    }
    else{
          fprintf(BERpFile,"========================================================\n");
          fprintf(BERpFile,"This is an Error Log for EMWIN-I FSK transmission\n");
          fprintf(BERpFile, "Date: %s \t\t Time: %s \n\n", dateStr,timeStr);   
          fprintf(BERpFile, "Simulation Data:\n");   
          fprintf(BERpFile, "\t Clock.Frequency: %g MHz\n",((icyFreqMult==1)?64:48));   
          fprintf(BERpFile, "\t ACQ Enable? %i \n \t Square Law=%i \t 4th Power=%i\n",RunACQEngine,atoi(in_square_law),atoi(in_fourth_law));   
          fprintf(BERpFile, "\t Frequency Offset=%4.3e\n",foff);   
          fprintf(BERpFile, "\t Tracking: Alpha=%4.3e \t Beta=%4.3e\n\n",config.alpha,config.beta);   

    }
    fclose(BERpFile);

    //config.BERFILE      = BERpFile;
    config.BERFileName  = BERDataFile;
    config.socket_flag  = atoi(in_socket);  

    printf("============================\n");
    printf(" FSK Demodulator Created \n");
    printf("===========================\n");
    // FSK demodulator
    fsk_rx         fsk_demod(blocks, config);
    //set_locking(float threshold, int lock_after, int unlock_after)
    fsk_demod.set_locking(atof(lock_threshold), lock_after, unlock_after);

    // CONNECT !
    if (using_rec_data==1){

        ///   Tracking RECORDED DATA !!!!
        unsigned int	InputDataSize	=sizeof(gr_complex);
        //gr_file_source_sptr  recorded_data_source       = gr_make_file_source(InputDataSize, RecDataFile);
         gr_file_source_sptr  recorded_data_source = gr_make_file_source(InputDataSize, RecDataFile, (bool)(atoi(repeat)));  

        connect_blocks(recorded_data_source,    0, fsk_demod.get_input_block(),    0, blocks);

#ifdef THROUGHPUT
        gr_throughput_sptr throughput =
            gr_make_throughput(sizeof(gr_complex), 2, "cout", "DISK-IN");
        connect_blocks(recorded_data_source,    0, throughput,                     0, blocks);
#endif

    }else{
        ///----------------------------
        ///   Tracking LIVE DATA !!!!
        ///----------------------------

        //---------------------- SOURCE: USRP-------------------------------------
        usrp1_source_c_sptr      usrp_source =
            usrp1_make_source_c ( which_board, decim_rate,nchan, mux,mode, 0,0,"","");
        usrp_source->set_rx_freq (0,fc+foff);
        usrp_source->set_pga (which_board,1);

        // USE LIVE DATA
        connect_blocks(usrp_source,    0, fsk_demod.get_input_block(), 0, blocks);


#ifdef THROUGHPUT
        gr_throughput_sptr throughput =
            gr_make_throughput(sizeof(gr_complex), 2, "cout", "USRP IN");
        connect_blocks(usrp_source,    0, throughput, 0, blocks);
#endif
        // --------------------- HEAD: File Sink: Input Bits
        if (atoi(write_input)==1){
            unsigned int	     InputDataSize	=sizeof(gr_complex);
            gr_file_sink_sptr   file_sink  =
                gr_make_file_sink(InputDataSize, "C:/temp/usrp_log/fsk_usrp_in.txt");
            connect_blocks(usrp_source,   0,   file_sink,  0, blocks);

        }
    }



    // --------------------------------------------------------------------------------
    printf("=====================\n");
    printf("  USRP BOX Connected  \n");
    printf("=====================\n\n");
    if (using_rec_data==1){
        printf("\tRec Data File   \t %s \n"  ,in_rec_data_file);
    }

    printf(" USRP Information: \n");
    printf("\t Decimation Rate = %i \n",decim_rate);
    printf("\t Sampling Freq. = %4.2e \n" , config.Fs );
    printf("\t Samples per Symbol = %4.2f\n" , samples_per_symbol);
    printf("\t Center Freq. = %4.2e \n" , fc);
    printf("\t Offset Freq. = %4.2f \n" , foff);
    printf("\t Projected Data-Input Rate= %4.2e [sps]\n" , samples_per_symbol*9600);
    printf("\n\n");
    printf("\t TRACKING= \n");
    printf("\t Alpha= %4.3e \t\t Beta= %4.3e \n",atof(acq_alpha),atof(acq_beta));
    printf("\t Threshold= %4.2f \t Lock After=%i \tUnLock After=%i \n" ,atof(lock_threshold), lock_after,unlock_after);
    printf("\n\n");
    printf("-------------------------------------------\n");
    //--------------------    SCHEDULER    ----------------------------------
    gr_single_threaded_scheduler_sptr  scheduler =
        start_scheduler_thread(blocks, false);
    printf( "---------   END PROCESS    -----------------\n");
    char dummy;
    scanf("%c", &dummy);


    // Free Variables
    free(in_decim_rate);
    free(in_fc);
    free(in_rec_data_file);
    free(ACQEngine);
    free(in_DeltaF);
    free(in_bitrate);
//    free(foff_file_in);
   
   printf("Closing BER-LOG file\n");
 
    return 0;
}