/*
GOES Radio Comments
This is the toplevel file.   It calls usrp_source_c to create an interface to the hardware,
freq_acq, oqpsk_rx.   This project is a purely single threaded projet.  oqpsk_rx does OQPSK
demodulation and also viterbi decoding.    
*/

#pragma once

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <ar_fir_filter_ccf.h>
#include "gnuradio.h"
#include "oqpsk_modem.h"
//#include "ControllerIF.h"
#include "freq_acq.h"  

//GNU Radio
#include <gr_single_threaded_scheduler.h>
#include <gr_block_detail.h>
#include <gr_file_sink.h>
#include <gr_file_source.h>
#include <gr_head.h>
#include <usrp1_source_c.h>

using namespace std;

int main(int argc, char* argv[])
{

    //-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+--+-+-+-+-+-+-+-+-+--++-+-+-+-+-+-+-+-+-+--+-+
    //--------------------------------------------------------------

    // Read Command Prompt Input Arguments

    //---------------------------------------------------------------
    //-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+--+-+-+-+-+-+-+-+-+--++-+-+-+-+-+-+-+-+-+--+-+
    int l;

   //help function
   for (l = 1; l < argc; l++ ) {
        if (!strcmp(argv[l],"help")) {
             printf("\n\n\nThis Application Demodulates the OQPSK EMWIN-N Signal from GOES Using the Following Options\n\nflags\t\t default\t description\n------\t\t --------\t -------------\n\n-socket\t\t 0\t\t Socket port number for which EMWIN data is sent out over localhost\n");
             printf("-wr\t\t 0\t\t Flag that writes input USB data to a file temp\\usrp_log\\usrp_in.txt\n");
             printf("-rec\t\t Null\t\t Processes data from the file [FileName] instead of using the USB\n");
             printf("-repeat\t\t 1\t\t Flag that specifies if the input data file will be looped back, or processed just once\n");
             printf("-sink\t\t 0\t\t Flag that writes OQPSK tracked constellation complex data points to temp\\usrp_log\\timing_tracked.dat\n");
             printf("-fc\t\t 137.9e6\t Input IF Center Frequency of the EMWIN-N Signal at the input to the IF Digitizer\n");
             printf("-br\t\t 17970\t\t Baud rate of the Signal\n");
             printf("-dr\t\t 254\t\t Decimation ratio for the IF Digitizer\n");
             printf("-pbw\t\t 200\t\t Phase bandwidth set as the (symbol rate)/pbw.  pbw=200 implies 17970/200 = 89.85 Hz\n");
             printf("-tbw\t\t 1000\t\t Timing loop bandwidth set as the (symbol rate)/tbw.  tbw=1000 implies 17970/1000 = 17.97 Hz\n");
             printf("-lfilter\t 1\t\t Flag that specifies if an input rejection filter is applied to the signal before demodulation\n");
             printf("-agc\t\t 10\t\t Level for the software AGC\n");
             printf("-roll\t\t 0.5\t\t Root Raised Cosine Receiver Pulse Shaping Filter roll-off (alpha) factor\n");
             printf("-acq\t\t 1\t\t Flag that turns the coarse frequency acquisition engine off/on\n");
             printf("-foff\t\t 0\t\t Manual Initial Frequency Offset Adjustment in Hz\n");
             printf("-fa_numfft\t 50\t\t Number of FFT averages on spectrum during frequency acquisition\n");
             printf("-fa_res\t\t 50\t\t Spectral Resolution in Hz during frequency acquisition process\n");
             printf("-fa_range\t 50000\t\t Range of frequencies to search over during frequency acquisition\n");
             printf("-fa_2\t\t 1\t\t Flag that uses the 2nd order frequency acquisition algorithm (To set 1st order, set 2nd and 4th order to 0)\n");
             printf("-fa_4\t\t 0\t\t Flag that uses the 4th order frequency acquisition algorithm (4th order requires 2nd order to be 1)\n");
             printf("-fa_fc\t\t 0\t\t Frequency Offset in Hz performed before conducting frequency acquisition\n");
             printf("-fa_delay\t 14\t\t Delay in samples of the Frequency Acquisition delay-and-multiply (squaring)algorithm\n");
             printf("-vperiod\t 10\t\t The number of Viterbi packets processed before verifying it is still locked\n");
             printf("-vthresh\t 0.1\t\t Output Viterbi BER to declare the FEC block is Locked\n");
             printf("-thresh\t\t 22\t\t The number of errors allowed in the CADU frame header to declare that lock is maintained\n");
           return 0;
        }
    }
    char *in_socket = "0";  //Option to dump emwin data out to a socket
    for (l = 1; l < argc; l++ ) {
        if (!strcmp(argv[l],"-socket")) {
            in_socket                = argv[l+1];
        }
    }

    char *in_qpsk_shift = "0"; //Manual fixed signal phase rotation of e^j*pi/2*qpsk_shift 
    for (l = 1; l < argc; l++ ) {
        if (!strcmp(argv[l],"-shft")) {
            in_qpsk_shift = argv[l+1];
        }
    }

     //Phase bandwidth ratio defined as Symbol_Rate/PHaseBW = BW in Hz. 
    char *PhaseBW    = "200";
    for (l = 1; l < argc; l++ ) {
        if (!strcmp(argv[l],"-pbw")) {
            PhaseBW = argv[l+1];
        }
    }

    //Timing bandwidth ratio defined as Symbol_Rate//TimeBW = BW in Hz. 
    char *TimeBW    = "1000";
    for (l = 1; l < argc; l++ ) {
        if (!strcmp(argv[l],"-tbw")) {
            TimeBW = argv[l+1];
        }
    }

    // Mode: 0 = BPSK, 1= QPSK, 2=OQPSK
    char *in_modulation = "2";
    for (l = 1; l < argc; l++ ) {
        if (!strcmp(argv[l],"-mod")) {
            in_modulation                = argv[l+1];
        }
    }

   //USRP FPGA decimation rate
    char *in_decim_rate = "254";
    for (l = 1; l < argc; l++ ) {
        if (!strcmp(argv[l],"-dr")) {
            in_decim_rate = argv[l+1];
        }
    }

 //This turns the oqpsk_rx timing tracked data output sink on/off
   char *sink_data = "0";
    for (l = 1; l < argc; l++ ) {
        if (!strcmp(argv[l],"-sink")) {
            sink_data = argv[l+1];
        }
    }

    //Set the AGC level for the oqpsk_rx class
    char *in_AGC = "10";      // AGC
    for (l = 1; l < argc; l++ ) {
        if (!strcmp(argv[l],"-agc")) {
            in_AGC= argv[l+1];
        }
    }

    //USRP FPGA DDFS center frequency
    char *in_fc = "137.9e6";
    for (l = 1; l < argc; l++ ) {
        if (!strcmp(argv[l],"-fc")) {
            in_fc                = argv[l+1];
        }
    }

    //Manual Frequency Offset
    char *in_foff = "0.0";
    for (l = 1; l < argc; l++ ) {
        if (!strcmp(argv[l],"-foff")) {
            in_foff                = argv[l+1];
        }
    }

     //Despite its name, activates Low Pass EMWIN filter
    char *in_lrit_filter_on = "1";
    for (l = 1; l < argc; l++ ) {
        if (!strcmp(argv[l],"-lfilter")) {
            in_lrit_filter_on    = argv[l+1];
        }
    }
    char *in_shaping_rolloff = "0.5"; 	     // Rolloff for RRC Pulse
    for (l = 1; l < argc; l++ ) {
        if (!strcmp(argv[l],"-roll")) {
            in_shaping_rolloff   = argv[l+1];
        }
    }

     //Flag to turn the Initial Coarse Frequency Acquisiton Engine on/off
    char *ACQEngine = "1";      // ON by default
    for (l = 1; l < argc; l++ ) {
        if (!strcmp(argv[l],"-acq")) {
            ACQEngine= argv[l+1];
        }
    }
    //Set the bit rate for the oqpsk_rx class
    char *in_bitrate = "17970";      // BITRATE
    for (l = 1; l < argc; l++ ) {
        if (!strcmp(argv[l],"-br")) {
            in_bitrate= argv[l+1];
        }
    }

    //Set the software decimation rate (inside oqpsk_rx) such that the number of samples
    //per symbol going into the tracking loops are "desired_samples_per_symbol"
    char *desired_samples_per_symbol    = "2";
    for (l = 1; l < argc; l++ ) {
        if (!strcmp(argv[l],"-sps")) {
            desired_samples_per_symbol = argv[l+1];
        }
    }

    // Write USB Input to DISK
    char *write_input = "0";
    for (l = 1; l < argc; l++ ) {
        if (!strcmp(argv[l],"-wr")) {
            write_input     = argv[l+1];
        }
    }

    //This sets the CADU Frame Synchronization error threshold in the data link layer
    char      *resynch_thresh     ="15";
    for (l = 1; l < argc; l++ ) {
        if (!strcmp(argv[l],"-thresh")) {
            resynch_thresh = argv[l+1];
        }
    }

    //Obsolete Option
    char      *using_icy      ="0";
    double    icyFreqMult=1.0;;
    //1	//Turn on required LRIT filter in order to receive LRIT
    for (l = 1; l < argc; l++ ) {
        if (!strcmp(argv[l],"-icy")) {
            using_icy= argv[l+1];
        }
    }

//Read data from a file source instead a USB
    char *in_rec_data_file    = "C:/temp0/usrp_log/oqpsk_usrp_in_dr254.txt";
    int  using_rec_data       =0;
    for (l = 1; l < argc; l++ ) {
        if (!strcmp(argv[l],"-rec")) {
            using_rec_data           =1;
            in_rec_data_file         = argv[l+1];
        }
    }

    char *repeat    = "1"; //Option to Repeat File Source
    for (l = 1; l < argc; l++ ) {
        if (!strcmp(argv[l],"-repeat")) {
            repeat = argv[l+1];
        }
    }

    //This sets the maximum error rate to determine that the Viterbi decoder is locked
    char      *vit_thresh     = "0.1"; 
    for (l = 1; l < argc; l++ ) {
        if (!strcmp(argv[l],"-vthresh")) {
            vit_thresh = argv[l+1];
        }
    }

     //This sets the frequency with wich Viterbi verifies that it is still locked
    char      *resynch_period     ="10";
    for (l = 1; l < argc; l++ ) {
        if (!strcmp(argv[l],"-vperiod")) {
            resynch_period= argv[l+1];
        }
    }


        // FRequency Acquisition: number off FFT averages on spectrum
    //The more averaging, the better the result, but the longer it takes
    char *in_num_fft_avg = "50";      
    for (l = 1; l < argc; l++ ) {
        if (!strcmp(argv[l],"-fa_numfft")) {
            in_num_fft_avg= argv[l+1];
        }
    }

    char *in_resolution = "50";      // FRequency Acquisition: Spectral Granularity
    for (l = 1; l < argc; l++ ) {
        if (!strcmp(argv[l],"-fa_res")) {
            in_resolution= argv[l+1];
        }
    }

    char *in_freq_range = "50000";      // FRequency Acquisition: Range of Frequencies to search over
    for (l = 1; l < argc; l++ ) {
        if (!strcmp(argv[l],"-fa_range")) {
            in_freq_range= argv[l+1];
        }
    }

    char *in_square_law = "1";      // FRequency Acquisition: Square Law Detector
    for (l = 1; l < argc; l++ ) {
        if (!strcmp(argv[l],"-fa_2")) {
            in_square_law= argv[l+1];
        }
    }

    char *in_fourth_law = "0";      // FRequency Acquisition: Fourth Law Detector
    for (l = 1; l < argc; l++ ) {
        if (!strcmp(argv[l],"-fa_4")) {
            in_fourth_law= argv[l+1];
        }
    }

    char *in_freq_acq_fc = "0";      // FRequency Acquisition: Internal Mixer (Separate from built in Usrp Mixer!)
    for (l = 1; l < argc; l++ ) {
        if (!strcmp(argv[l],"-fa_fc")) {
            in_freq_acq_fc= argv[l+1];
        }
    }

    //-fa_delay = 13 seems to be best with 64 MHz board, dr 222, and Werner DownConverter
    char *in_freq_acq_delay = "13";      // FRequency Acquisition: Delay for delay and square detector (dummy 9999 -> Set to samples/symbol)
    for (l = 1; l < argc; l++ ) {
        if (!strcmp(argv[l],"-fa_delay")) {
            in_freq_acq_delay= argv[l+1];
        }
    }

    if (using_rec_data==0){
        printf( "==========================\n");
        printf("  Tracking LIVE SATELLITE    \n");
        if (atoi(using_icy)==0)
            printf("  Using ETTUS BOX    \n");
        else
            printf("  Using AERO-ICY BOX    \n");
        printf("===========================\n\n");
    }else{
        printf("===========================\n\n");
        printf("  Tracking RECORDED SATELLITE    \n");
        if (atoi(using_icy)==0)
            printf("  Using ETTUS BOX    \n");
        else
            printf("  Using AERO-ICY BOX    \n");
        printf("===========================\n\n");
    }

    //--------------------------------------------------------------
    //             ---  BPSK - LRIT/EMWIN     ----
    // EMWIN-N:    Fc                     = 137.5Mhz.
    //   	        Data rate                = 293883 ksps.
    //
    //URSP BOX
    //--------
    // Sampling at 64 MHz --> 3rd Harmonic at 9.5  Mhz
    // Signal comes in at 293883 ksps.  ADC samples a 64 Msps.  Thus I hav 217.7 sps
    // decimate by 108  gives 2.0164 samples per symbol
    // decimate by 72  gives 3.0246 samples per symbol
    // decimate by 54  gives 4.03 samples per symbol
    //
    // ICY BOX
    //---------
    // Sampling at 48 MHz --> 3rd Harmonic at  6.5 Mhz
    // Signal comes in at 293883 ksps.  ADC samples a 48 Msps.  Thus I hav 163.33 sps
    // decimate by 80  gives 2.0416 samples per symbol
    // decimate by 54  gives 3.0246 samples per symbol
    // decimate by 40  gives 4.08 samples per symbol

    //              --- FSK Demodulator Values ---
    // EMWIN-I:    Fc                       = 137.225Mhz.
    //   	     Data rate                = 9.6 ksps.
    //             Frequency Separation     = 3.6 kHz.
    // Sampling at 64 MHz --> 2nd Harmonic at 9.2250 Mhz
    //             f0                       =  7.425 MHz
    //             f1                       = 11.025 MHz
    // Sampling at 48 MHz --> 2nd Harmonic at 6.7750 Mhz
    //             f0                       =  4.975 MHz
    //             f1                       =  8.575 MHz

    //**************************************************************
    //             ---  OQPSK - EMWIN-N     ----
    // EMWIN-N:    Fc                       = 137.9Mhz.
    //   	     Data rate                  = 17970ksps.
    // Sampling at 64 MHz --> 2nd Harmonic at 11.20  Mhz
    // Signal comes in at 17.970 ksps.
    //ADC samples a 64 Msps.  Thus I hav 3561.5 sps
    // decimate by 254 gives 14.01  samples per symbol
    // >>> decimate by 222 gives 16.00 samples per symbol
    // decimate by 178 gives 20.00 samples per symbol
    // decimate by 110  gives 32.37 samples per symbol
    // decimate by 88  gives 40.47 samples per symbol

    // Sampling at 48 MHz --> 3rd Harmonic at  6.1 Mhz
    // Signal comes in at 17.970 ksps.
    //ADC samples a 48 Msps.  Thus I hav 2.671 Ksps
    // decimate by 242 gives 11.03  samples per symbol
    // decimate by 222 gives 12.03 samples per symbol
    // decimate by 206 gives 12.96 samples per symbol
    // >>> decimate by 178  gives 15.003 samples per symbol
    // decimate by 116  gives 23.06 samples per symbol
    //**************************************************************



    // Container for the transmitter and receiver blocks
    vector<gr_block_sptr> oqpsk_blocks;

    // --------------------- Create the transmitter ------------------
    // USRP: RX
    int			which_board	    = 0;
    unsigned int	decim_rate	= atoi(in_decim_rate);   // 2samples-> 108, 3 samples-->72, 4 samples--> 54
    double		fc			    = atof(in_fc);		     // Center freq. for USRP
    int			lrit_filter_on	= atoi(in_lrit_filter_on);
    int			nchan		    = 1;
    int			mux			    = 1;
    int			mode		    = 0;
    double		foff		    = atof(in_foff);		// Freq offset due to Sig.Generator
    
    int         clock_rate=64e6;
    if (atoi(using_icy) ==1){
        clock_rate      =48e6;
        icyFreqMult     = 64.0/48.0;
    }


    bool        RunACQEngine	= bool(atoi(ACQEngine));
    int			NoSamples		= 250e6;  // No. USRP Samples
    int			file_sink_on	= atoi(write_input);      //0
    char 		*RecDataFile;
    RecDataFile    =(char *) calloc(200,sizeof(char));
    strcpy(RecDataFile,in_rec_data_file);

    FILE		* BERpFile; 
    char 	    *BERDataFile;
    BERDataFile         =(char *) calloc(200,sizeof(char));
    strcpy(BERDataFile,"c:/EMWINTEMP/BER_EMWINlog.txt");
    BERpFile = fopen(BERDataFile , "a+" ); 
  
    //Create am oqpsk_rx configuration file 
    oqpsk_config config;
    double mirror_fc;
    int mirror_factor;


     //Set oqpsk_rx configuration options
    config.bit_rate		     = atoi(in_bitrate);
    config.shaping_rolloff	     = atof(in_shaping_rolloff);  //unkown
    config.Fc				     = 0;
    config.file_sink_on	          = file_sink_on;
    config.modulation             = atoi(in_modulation);  // 0=BPSK, 1=QPSK,2=OQPSK
    config.samples_per_symbol_decim     = atoi(desired_samples_per_symbol);
    //config.diff_decoder           = atoi(in_diff_decoder);
    config.qpsk_shift           = atoi(in_qpsk_shift);
    config.AGC                     = atof(in_AGC);
    config.PBW                     = atof(PhaseBW);
    config.TBW                     = atof(TimeBW);
    config.resynch_thresh         = atoi(resynch_thresh);
    config.resynch_thresh         = atoi(resynch_thresh);
    config.vthresh                = atof(vit_thresh);
    config.resynch_period         = atoi(resynch_period);
    config.BERFileName            = BERDataFile;
    config.sink_data              = atoi(sink_data);
    config.socket_flag            = atoi(in_socket);  

    // SUB-SAMPLING FREQUENCIES
    //Convert the IF frequency (for example 137.5 MHz)
    //into the DDFS frequency needed to bring the signal down to baseband based
    //on the IF-sampling principle.  For 137.5, mirror_fc would be 9.5 MHz
    mirror_fc =fc;
    mirror_factor=-1;


    config.Fs		 = clock_rate/decim_rate;
    for  (int ii=0; ii< 5;ii++)
    {
        if (fabs(fc - ii*clock_rate) < mirror_fc){
            mirror_fc = fabs(fc - ii*clock_rate)*icyFreqMult;
            mirror_factor =ii;
        }
    }

    //Assign Sub-sampling Frequency to fc
    fc                            = mirror_fc;
    float     Fs                  = config.Fs;          // Sampling frequency [Hz]
    float     Tb                  = 1.0/atof(in_bitrate);
    float     samples_per_symbol  = Fs * Tb;     
    config.samples_per_symbol     = samples_per_symbol;



    // =======================================================================
    //                  Create frequency acquisition block
    // =======================================================================
    //char *foff_file_in    = "C:/temp/usrp_log/input_data\\foff.dat";
    
    int delay;  //Set the delay value within the frequency acq engine 
    if(atoi(in_freq_acq_delay)==9999)
        delay = (int)config.samples_per_symbol;  
    else
        delay = atoi(in_freq_acq_delay);

    if (RunACQEngine==1) {
        printf( "======================\n");
        printf( "  ACQ-Engine ENABLED  \n");
        printf( "======================\n");
        freq_acq_config                    freq_acq_config0;
        freq_acq_config0.Fs                =  config.Fs;
        freq_acq_config0.delay             = delay;  //Delay by one symbol
        freq_acq_config0.Fc                = atof(in_freq_acq_fc);  //Obsolete mixer always set to 0!
        freq_acq_config0.num_fft_avg       = atof(in_num_fft_avg);
        freq_acq_config0.resolution        = atof(in_resolution);
        freq_acq_config0.freq_range        = atof(in_freq_range);
        //freq_acq_config0.bandwidth         = atof(in_freq_acq_bw); //expected bandwidth of signal for 1st order estimation
        freq_acq_config0.square_law        = (bool)atoi(in_square_law);
        freq_acq_config0.fourth_law        = (bool)atoi(in_fourth_law);
        if (freq_acq_config0.fourth_law){
            freq_acq_config0.delay        = 0; //For the moment, 4th law only works with delay = 0;
            freq_acq_config0.num_fft_avg  = 16; //For the moment, 4th law doesn't allow more than 16 averages
        }

        // USRP SETUP
        freq_acq_config0.usrp_which_board    = which_board;
        freq_acq_config0.usrp_decim_rate     = decim_rate;
        freq_acq_config0.usrp_nchan          = nchan;
        freq_acq_config0.usrp_mux            = mux;
        freq_acq_config0.usrp_mode           = mode;
        freq_acq_config0.usrp_fc             = fc;
        //freq_acq_config0.lrit_filter	     = lrit_filter_on;

        freq_acq_config0.rec_data          = using_rec_data; //JR Recorded data flag
        if (using_rec_data) //JR Recorded data option
        {
            strcpy(freq_acq_config0.filename, in_rec_data_file);
        }

        //Instantiate the frequency Acquistion Class
        freq_acq acq(freq_acq_config0, false); 

        // Begin ACQ
        acq.run();
        Sleep(2000);
        freq_acq_result result_scalar;
        vector<freq_acq_result> result_vector;
        float acquired_carrier;
        float carrier_power;

        //IF you are using OQPSK based squre law
        if (freq_acq_config0.square_law && !freq_acq_config0.fourth_law)
        {
            printf("Using a Square Law Frequency Acquisition Engine\n");
            // Wait for Acquistion to finish
            while (!acq.get_freqs(2,result_vector)) Sleep(1);

            float peak1 = result_vector[0].freq;
            float peak2 = result_vector[1].freq;
            //The second peak, might be still represent same peak.  Might need to grab 4 peaks, evaluate them to make sure they are distinct.
            carrier_power = min(result_vector[0].relative_power, result_vector[1].relative_power);
            if (peak1>peak2)
                acquired_carrier = (peak2-peak1)/2+peak1;
            else
                acquired_carrier = (peak1-peak2)/2+peak2;
            printf("Acquired Peak: Freq = %f, pwr = %f\n", acquired_carrier,carrier_power);
        }
        //IF you are using OQPSK based fourth law
        else if (freq_acq_config0.fourth_law)
        {
            printf("Using a Fourth Law Frequency Acquisition Engine\n");
            // Wait for Acquistion to finish
            while (!acq.get_peak_freq(result_scalar)) Sleep(1);
            acquired_carrier  = result_scalar.freq;
            carrier_power = result_scalar.relative_power;
            printf("Acquired Peak: Freq = %f, pwr = %f\n", acquired_carrier,carrier_power);
        }
        else
        {
            int numpeaks = 250;
            printf("Using a First Order Law Frequency Acquisition Engine\n");
            // Wait for Acquistion to finish
            while (!acq.get_freqs(numpeaks,result_vector)) Sleep(1);
            float peaksum = 0;
            float powersum = 0;
            float peakavg;
            for (int k=0; k<numpeaks; k++){
                peaksum = peaksum + result_vector[k].freq;
                powersum = powersum + result_vector[k].relative_power;
            }
            acquired_carrier = peaksum/numpeaks;
            carrier_power = powersum/numpeaks;
            printf("Acquired Peak: Freq = %f, pwr = %f\n", acquired_carrier,carrier_power);
        }

        //Update Frequency Offset based on acquired carrier frequency.
        foff += -acquired_carrier;
        acq.stop();
    }  //end Acq

    printf("foff = %g\n", foff);
    Sleep(1000); //Allows user to verify the frequency Acquisition result after it is printed to screen. 

    // =======================================================================
    //                  BEGIN TRACKING & PROCESSING
    // =======================================================================
    usrp1_source_c_sptr           usrp_source;

    //------------------------------------------------------------------------------------


    // --------------------- Create the receiver ---------------------
    oqpsk_rx     oqpsk_demod(oqpsk_blocks, config);

    // ----------------- FILTER front end LRIT SATELLITE SIGNAL  ---------------------
    vector<float> usrp_lowpass;
    vector<float> icy_lowpass;
    gr_file_source_sptr recorded_data_source;
    gr_fir_filter_ccf_sptr lrit_filter;

    if (lrit_filter_on == 1){
        if (icyFreqMult==1){
            printf("\tLoading Filter Coeffs \n\n");
            if (decim_rate > 0){
                grab(usrp_lowpass, "c:/temp/usrp_log/input_data/lowpass_OPQSK_35tap_0.2.coef");
                //grab(usrp_lowpass, "c:/temp/usrp_log/input_data/lowpass_OQPSK_50tap_03.coef")    ;
            }
            lrit_filter =  gr_make_fir_filter_ccf (1, usrp_lowpass);
        }else{ //ICY BOX

            printf("Loading ICY Filter Coeffs  - NEEDS UPDATE \n");
            if (decim_rate >0 ){ // 4 samples per symbol
                grab(icy_lowpass, "c:/temp/usrp_log/input_data/lowpass_OPQSK_35taps.coef");
                //}else if(decim_rate == 80){  // 4 samples per symbol
            }else {
                grab(icy_lowpass, "c:/temp/usrp_log/input_data/lowpass_OPQSK_35taps.coef");
                //grab(icy_lowpass, "c:/temp/usrp_log/input_data/icy_80decim_40taps.coef");
            }
            lrit_filter =  gr_make_fir_filter_ccf (1, icy_lowpass);

        }

        if (using_rec_data==1){
            recorded_data_source = gr_make_file_source(sizeof(gr_complex), RecDataFile,(bool)(atoi(repeat)));
            connect_blocks(recorded_data_source,    0, lrit_filter, 0, oqpsk_blocks);
        }else{
            usrp_source   = usrp1_make_source_c ( which_board, decim_rate,nchan, mux,mode, 0,0,"","");
            usrp_source->set_rx_freq (0,fc+foff*icyFreqMult);  //48 Mhz clock must be accounted for in set_rx_freq function
            usrp_source->set_pga (which_board,1);
            connect_blocks(usrp_source,             0, lrit_filter, 0, oqpsk_blocks);
        }


        // SEND INPUT DATA TO BPSK RECEIVER
        connect_blocks(lrit_filter, 0, oqpsk_demod.get_input_block(), 0, oqpsk_blocks);

        if (file_sink_on){
            gr_block_sptr		my_head2;  //plot filtered signal
            my_head2		     = gr_make_head(sizeof(gr_complex),NoSamples);   // plot filtered signal
            gr_file_sink_sptr   file_sink2
            = gr_make_file_sink(sizeof(gr_complex), "C:/temp/usrp_log/oqpsk_in_filtered.txt");		//plot filtered signal
            connect_blocks(lrit_filter, 0, my_head2, 0, oqpsk_blocks);   //plot filtered singal
            connect_blocks(my_head2, 0, file_sink2, 0, oqpsk_blocks);    //plot filtered signal
        }

    }
    else{
        // NO FILTER
        if (using_rec_data==1){
            recorded_data_source = gr_make_file_source(sizeof(gr_complex), RecDataFile);
            connect_blocks(recorded_data_source, 0, oqpsk_demod.get_input_block(), 0, oqpsk_blocks);
        }else{
            usrp_source   = usrp1_make_source_c ( which_board, decim_rate,nchan, mux,mode, 0,0,"","");
            usrp_source->set_rx_freq (0,fc+foff*icyFreqMult);
            usrp_source->set_pga (which_board,1);
            connect_blocks(usrp_source,             0, oqpsk_demod.get_input_block(), 0, oqpsk_blocks);
        }

    }

    // --------------------- HEAD: File Sink: Input Bits
    if (file_sink_on){
        unsigned int	     InputDataSize	=sizeof(gr_complex);
        gr_block_sptr	     my_head;
        my_head			= gr_make_head(InputDataSize,NoSamples);
        gr_file_sink_sptr   file_sink  =
            gr_make_file_sink(InputDataSize, "C:/temp/usrp_log/oqpsk_usrp_in.txt");

        /// BUG: If I connect this, throughput estimator gets killed
        connect_blocks(usrp_source,   0,   my_head,  0, oqpsk_blocks);
        connect_blocks(my_head,       0,   file_sink,0, oqpsk_blocks);
    }

    // --------------------------------------------------------------------------------
    printf("==================================\n");
    printf("  SOFTWARE RADIO CONFIGURATION    \n");
    printf("==================================\n\n");
    printf("Input Parameters: \n");
    printf("Using Recorded Data ? (0,1)=\t %i \n" , using_rec_data );
    if (using_rec_data==1){
        printf("\tRec Data File   \t %s \n"  ,in_rec_data_file);
    }
    printf("Using LRIT Filter? (0,1)=\t %s\n", in_lrit_filter_on);
    printf("\n");
    if (atoi(using_icy) ==0){
        printf(" USRP Information: \n");
    }else{
        printf(" AID Information: \n");
    }

    printf("\t Decimation Rate = %i \n",decim_rate);
    printf("\t Sampling Freq. = %4.2e \n" , config.Fs );
    printf("\t Data Rate  = %5.3f \n" , config.bit_rate );
    printf("\t Signal Freq. = %g\n" , atof(in_fc));
    //printf("\t\t Center Freq. = %5.3f \n" , fc);
    //printf("\t\t Offset Freq. = %5.3f \n" , foff);
    printf("\t Pulse-Shaping Rolloff = %4.2f \n" , config.shaping_rolloff);
    printf("\t Samples per symbol = %4.2f \n",config.samples_per_symbol);
    if (atoi(in_modulation)==0){
        printf("\t MODULATION = BPSK \n");
    }else if (atoi(in_modulation)==1){
        printf("\t MODULATION = QPSK \n");
    }else if (atoi(in_modulation)==2){
        printf("\t MODULATION = O-QPSK \n");
    }
    printf("\t SINGLE-THREAD PROCESS \n" );
    printf("\n\n");
    //-------------------------------------------------------------------


    //   ----------------
    //   SINGLE THREAD
    //   ----------------


    gr_single_threaded_scheduler_sptr   scheduler =
        start_scheduler_thread(oqpsk_blocks, true);  //Used to be false

    printf( "---------   END PROCESS    -----------------\n");
    char dummy;
    scanf("%c", &dummy);
    fclose(BERpFile);
    return 0;
}



