#pragma once

#include "gnuradio_misc.h"

#include "circular_buffer.h" // Thread-safe ring buffer
#include <gr_single_threaded_scheduler.h>

//! oqpsk modem configuration.  This structure contains all the variables
//! needed to define a oqpsk modem.
typedef struct oqpsk_config {
    float Fs;                 //!< Sampling frequency at the input
    float Fc;                 //!< Subcarrier frequency
    float bit_rate;           //!< Symbol rate
    float shaping_rolloff;    //!< Rolloff factor for the SRRCF.  No filter if r >0.99
    bool double_sided;        //!< Double sided -- real output only
    // Esteban
    float       samples_per_symbol;  //The samples_per_symbol at the input to the USB
    int         samples_per_symbol_decim;   //JR The desired integer samples per symbol after decimation, assumed to be an integer
    int	        file_sink_on;
    int         modulation;
    int         qpsk_shift;
    float       AGC;  //JR Added for Viterbi Table
    float       PBW; //JR Added
    float       TBW;  //JR Added
    int         resynch_thresh; //JR added
    float       vthresh;
    int         resynch_period;
    int         dsps;
    //FILE *      BERFILE;
    char        * BERFileName;
    int         sink_data;
    int         socket_flag;
    int         socket_lrit_flag;
    int         only_sink;

    oqpsk_config() : Fs(0), Fc(0), bit_rate(0), shaping_rolloff(0),
            double_sided(false),samples_per_symbol(0)
    { }
} oqpsk_config;

using namespace std;

/*! OQPSK RECEIVER class.  This class encapsulates a number of
 *  GNU radio blocks to implement a oqpsk modulator
 */

class oqpsk_rx
{
private:
    vector<gr_block_sptr> d_blocks;
    oqpsk_config d_config;

public:
    //! Create the receiver.  See documentation on oqpsk_config
    oqpsk_rx(vector<gr_block_sptr> &blocks, const oqpsk_config &config);

    //! Return a vector of all the blocks created in this module
    vector<gr_block_sptr> &get_blocks() {
        return d_blocks;
    }

    //! Get the first block -- subcarrier mixer
    // gr_block_sptr get_input_block()
    //     {return find_block(d_blocks, "oqpsk_rx::mixer");}
    // Mixer is not really needed
    gr_block_sptr get_input_block()
    {
        return find_block(d_blocks, "oqpsk_rx::agc");  
    }

    //! Get the last block -- the slicer
    gr_block_sptr get_bits()
    {
        return find_block(d_blocks, "oqpsk_rx::slicer");
    }

    gr_block_sptr get_softout()
    {
        return find_block(d_blocks, "oqpsk_rx::slicer_real");
    }

    gr_block_sptr get_softout_QPSK()
    {
        return find_block(d_blocks, "oqpsk_rx::slicer_real_QPSK");
    }

    //! Check if the phase tracking locked
    bool get_phase_lock();
    //! Set the bandwidth of the phase tracking loop (costas) [Hz]
    void set_phase_bandwidth(float bw);
    //! Set the bandwidth of the symbol/timing tracking loop (M&M) [Hz]
    void set_timing_bandwidth(float bw);
    //! See comment in ::transponder
    //circular_buffer<gr_complex> *get_bent_pipe();

    void set_phase_bandwidth(float bw, const oqpsk_config &config);
    //! Set the bandwidth of the symbol/timing tracking loop (M&M) [Hz]
    void set_timing_bandwidth(float bw, const oqpsk_config &config);

};


;
