#ifndef INCLUDED_GR_CHECK_LFSR15_B_H
#define INCLUDED_GR_CHECK_LFSR15_B_H

#include <gr_sync_block.h>
#include <gri_lfsr_32k.h>


class gr_check_lfsr15_b;
typedef boost::shared_ptr<gr_check_lfsr15_b> gr_check_lfsr15_b_sptr;

gr_check_lfsr15_b_sptr gr_make_check_lfsr15_b ();

/*!
 * \brief sink that checks if its input stream consists of a lfsr_32k sequence.
 * \ingroup sink
 *
 * This sink is typically used along with gr_lfsr15_bource_s to test
 * the USRP using its digital loopback mode.
 */
class gr_check_lfsr15_b : public gr_sync_block
{
  friend gr_check_lfsr15_b_sptr gr_make_check_lfsr15_b ();

  enum state {
    LOAD_LFSR,		// searching for synchronization
    CHECK_BER
  };

  state			d_state;
  int			d_lfsr;
  int			d_load_count;
  
  unsigned long	d_bits;
  unsigned long	d_errs;
  double		d_total_bits;
  double		d_total_errs;


  gr_check_lfsr15_b ();
  
  char next_lfsr_output();
  static int LFSR_BER_LENGTH;
  static int LFSR_LENGTH;

 public:

  int work (int noutput_items,
	    gr_vector_const_void_star &input_items,
	    gr_vector_void_star &output_items);

  double ber () const { return d_total_errs / d_total_bits; }
};


#endif /* INCLUDED_GR_CHECK_LFSR15_B_H */
