
#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <ar_viterbi_lib.h>
#include <ar_conv_encoder.h>
#include <gr_io_signature.h>
#include <ipp.h>
#include <stdio.h>

 


ar_conv_encoder_sptr     ar_make_conv_encoder()
{
    return ar_conv_encoder_sptr (new ar_conv_encoder () );
}

// Data size is 8 bits. 
ar_conv_encoder::ar_conv_encoder ()
        : gr_block ("conv_encoder",
                    gr_make_io_signature (1, 1, sizeof (unsigned char)),
                    gr_make_io_signature (1, 1, sizeof (unsigned char)))  
        
{ }

ar_conv_encoder::~ar_conv_encoder()
{ }

void ar_conv_encoder::forecast(int noutput_items,
                          gr_vector_int &ninput_items_required)
{
   ninput_items_required[0] = ( 5000+ noutput_items  + DEPTH_TRELLIS) *0.5;   
}




int
ar_conv_encoder::general_work (int                  noutput_items,
                          gr_vector_int             &ninput_items,
                          gr_vector_const_void_star &input_items,
                          gr_vector_void_star       &output_items){

    // FYI:  DEPTH_TRELLIS=35
    unsigned char  *in_vec    = (unsigned char *)   input_items[0];
    unsigned char  *out_vec   = (unsigned char *)   output_items[0];

    int         generated_output_bits ;
    
    //Continuous encoder
    static int begin_state    =0;           
    
    int     ni                  = ninput_items[0];
    //Use Multiples of Bytes. 16 = 8 * 1/CodeRate
    int     data_length = (ni/16)*16;

    // if (data_length >60e3){
    //     data_length=16000;
    // }
    //Convert Input from 8 bits to  16 bits
    Ipp16s     *in_vec16 ;
    in_vec16      = ippsMalloc_16s(data_length);
    for (int ii=0; ii<data_length;ii++){
        in_vec16[ii]  =  in_vec[ii];
    }
    Ipp16s  *encoded_data_out ;    
    encoded_data_out             = ippsMalloc_16s(2* data_length); 
    
    // Output from Encoder  =  VitOut of size 2*generated_output_bits
    generated_output_bits       = 2*data_length - DEPTH_TRELLIS;      
    // short           *data_in	= new short [channel_length];  
             
    cnv_encd_intel(data_length, in_vec16, encoded_data_out,&begin_state) ;
       
    for (int ii=0;ii<2*data_length;ii++){
        out_vec[ii]  =encoded_data_out[ii] & 0x0001;
    }
    consume_each (data_length);
    return (generated_output_bits);
}
