
#ifndef INCLUDED_GR_FIR_FILTER_CCF_H
#define	INCLUDED_GR_FIR_FILTER_CCF_H

#include <gr_sync_decimator.h>
#include <ipp.h>

class gr_fir_filter_ccf;
typedef boost::shared_ptr<gr_fir_filter_ccf> gr_fir_filter_ccf_sptr;
gr_fir_filter_ccf_sptr gr_make_fir_filter_ccf (int decimation, const std::vector<float> &taps);

class gr_fir_ccf;

/*!
 * \brief FIR filter with gr_complex input, gr_complex output and float taps
 * \ingroup filter
 */
class gr_fir_filter_ccf : public gr_sync_decimator
{
 private:
  friend gr_fir_filter_ccf_sptr gr_make_fir_filter_ccf (int decimation, const std::vector<float> &taps);

  std::vector<float>	d_new_taps;
  bool			d_updated;
  IppsFIRState_32f *d_filt_re, *d_filt_im;
  Ipp32f *d_buf_re, *d_buf_im;

  void create_filters (const std::vector<float> &taps);

  /*!
   * Construct a FIR filter with the given taps
   */
  gr_fir_filter_ccf (int decimation, const std::vector<float> &taps);

 public:
  ~gr_fir_filter_ccf ();

  void set_taps (const std::vector<float> &taps);

  int work (int noutput_items,
		 gr_vector_const_void_star &input_items,
		 gr_vector_void_star &output_items);
};

#endif
