#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <ar_function_source_b.h>
#include <gr_io_signature.h>
#include <stdexcept>

gr_function_source_b_sptr gr_make_function_source_b()
{
  return gr_function_source_b_sptr(new gr_function_source_b());
}

gr_function_source_b::gr_function_source_b()
  : gr_sync_block ("function_source_b",
		   gr_make_io_signature (0, 0, 0),
		   gr_make_io_signature (1, 1, sizeof(unsigned char))),
    d_repeat(0), d_index(0)
{
}

int
gr_function_source_b::work(int noutput_items,
			gr_vector_const_void_star &input_items,
			gr_vector_void_star &output_items)
{
  char *out = (char *) output_items[0];

  for (int i = 0; i < noutput_items; i++) {

    if (d_index == d_data.size()) 
      if (d_hold_last)
	      out[i] = d_data[d_data.size()-1]; // Hold the last value
      else
        out[i] = -1;                      // -1 means we are out of data
	else
	  out[i] = d_data[d_index++];
    
  if (d_index == d_data.size())
	{
		// still have repeats left, or infinite
		if (d_repeat > 0 || d_repeat == -1) d_index = 0; 

		if (d_repeat > 0) d_repeat--; // one less repeat left
	}		
  }

  return noutput_items;
}


