
#ifndef INCLUDED_GR_FUNCTION_SOURCE_B_H
#define INCLUDED_GR_FUNCTION_SOURCE_B_H

#include <gr_sync_block.h>

class gri_function;

class gr_function_source_b;
typedef boost::shared_ptr<gr_function_source_b> gr_function_source_b_sptr;

gr_function_source_b_sptr gr_make_function_source_b();

/*!
 * \brief Symbol source, where the bits are explicitly set by calling a
 * member function.
 * \ingroup source
 */
class gr_function_source_b : public gr_sync_block 
{
 private:
  friend gr_function_source_b_sptr gr_make_function_source_b();
  
  int d_repeat;
  bool d_hold_last;
  std::vector<char> d_data;
  unsigned int d_index;
  
  gr_function_source_b();

 public:

  int work(int noutput_items,
	   gr_vector_const_void_star &input_items,
	   gr_vector_void_star &output_items);

  /*! Set the data to be transmitted.  This should only be called while the
   *  scheduler is not running.  Data replaces any data that may have been
   *  set previously.
   * \param data Array of bytes (0-255).  Each byte corresponds to a symbol
   * \param len Number of symbols in the array
   */
  void set_data(const char *data, int len) {d_data = std::vector<char>(data, data + len);}

  /*! Set the behavior after all the symbols are exhausted. We can either
   *  repeat from the beginning, hold the last value.  If neither is true,
   *  a default value of -1 is output.
   * \param repeat Set to 0 for one pass, an integer > 0 to specify a fixed 
   *               number of passes, or -1 for infinite repeats.
   * \param hold_last Set to true to hold the last value once all the repeat
   *                  passes are exhausted.  Only makes sense if repeat != -1
   */
  void set_repeat(int repeat, bool hold_last) {d_repeat = repeat; d_hold_last = hold_last;}
};

#endif /* INCLUDED_GR_FUNCTION_SOURCE_B_H */
