
#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <ar_integrate_dump_cc.h>
#include <gr_io_signature.h>
#include <ipp.h>

gr_integrate_dump_cc_sptr gr_make_integrate_dump_cc (int samples_per_symbol)
{
  return gr_integrate_dump_cc_sptr (new gr_integrate_dump_cc (samples_per_symbol));
}

gr_integrate_dump_cc::gr_integrate_dump_cc (int samples_per_symbol)
  : gr_sync_decimator ("integrate_dump_cc",
		       gr_make_io_signature (1, 1, sizeof (gr_complex)),
		       gr_make_io_signature (1, 1, sizeof (gr_complex)),
		       samples_per_symbol),
    d_samples_per_symbol(samples_per_symbol)
{
    d_mean_pwr = 0.0;
    d_pwr_alpha = 0.1;
}


int
gr_integrate_dump_cc::work (int noutput_items,
		   gr_vector_const_void_star &input_items,
		   gr_vector_void_star &output_items)
{
  gr_complex *in = (gr_complex *) input_items[0];
  gr_complex *out = (gr_complex *) output_items[0];

  int idx = 0;
  for (int n=0; n<noutput_items; n++)
  {
      ippsSum_32fc((Ipp32fc*)in, d_samples_per_symbol, (Ipp32fc*)& (out[n]), ippAlgHintFast);
      in += d_samples_per_symbol;
      d_mean_pwr = (1-d_pwr_alpha) * d_mean_pwr + d_pwr_alpha * norm(out[n]);
  }

  return noutput_items;
}
