#ifndef INCLUDED_GR_INTERP_FIR_FILTER_ccf_H
#define	INCLUDED_GR_INTERP_FIR_FILTER_ccf_H

#include <gr_sync_interpolator.h>
#include <ipp.h>

class gr_interp_fir_filter_ccf;
typedef boost::shared_ptr<gr_interp_fir_filter_ccf> gr_interp_fir_filter_ccf_sptr;
gr_interp_fir_filter_ccf_sptr 
    gr_make_interp_fir_filter_ccf (unsigned interpolation, 
                                       const std::vector<float> &taps);

class gr_fir_ccf;

/*!
 * \brief Interpolating FIR filter with gr_complex input, gr_complex output and float taps
 * \ingroup filter
 */
class gr_interp_fir_filter_ccf : public gr_sync_interpolator
{
 private:
  friend gr_interp_fir_filter_ccf_sptr gr_make_interp_fir_filter_ccf (unsigned interpolation, const std::vector<float> &taps);

  std::vector<float> d_taps;
  unsigned           d_interpolation;
  bool			     d_updated;
  IppsFIRState_32f  *d_filt_re, *d_filt_im;
  Ipp32f            *d_buf_re, *d_buf_im;

  /*!
   * Construct a FIR filter with the given taps
   */
  gr_interp_fir_filter_ccf (unsigned interpolation, const std::vector<float> &taps);

  void install_taps (const std::vector<float> &taps);

 public:
  ~gr_interp_fir_filter_ccf ();

  //! Set the new taps
  void set_taps (const std::vector<float> &taps);
  //! Get the currently used taps
  const std::vector<float> &get_taps() const {return d_taps;} 
  //! Get the interpolation ratio
  unsigned get_interp_ratio() {return d_interpolation;}

  int work (int noutput_items,
	    gr_vector_const_void_star &input_items,
	    gr_vector_void_star &output_items);
};

#endif
