#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <ar_mixer_cc.h>
#include <gr_io_signature.h>
#include <math.h>
#include <ipp.h>

#define M_TWOPI (2*M_PI)

unsigned long long getCPUTick(); 

gr_mixer_cc_sptr
gr_make_mixer_cc (float sampling_freq, float freq, bool real)
{
  return gr_mixer_cc_sptr (new gr_mixer_cc (sampling_freq, freq, real));
}

gr_mixer_cc::gr_mixer_cc (float sampling_freq, float freq, bool real)
  : gr_sync_block ("mixer_cc",
		   gr_make_io_signature (1, 1, sizeof (gr_complex)),
		   gr_make_io_signature (1, 1, sizeof (gr_complex))),
   d_sampling_freq(sampling_freq), d_phase(0), d_real(real)
{
    set_freq(freq);
}

int
gr_mixer_cc::work (int noutput_items,
		   gr_vector_const_void_star &input_items,
		   gr_vector_void_star &output_items)
{
  gr_complex *iptr = (gr_complex *) input_items[0];
  gr_complex *optr = (gr_complex *) output_items[0];

  // Save the complex exponential to the output array since it is
  // not used yet.  It will then be overwritten by in[i] * exp[i]
  ippsTone_Direct_32fc((Ipp32fc *)optr, noutput_items, 1.0, d_freq,
      &d_phase, ippAlgHintFast);

  if (d_real) // unusual case, should really just compute the real part
      for (int i=0; i<noutput_items; i++) ((Ipp32fc *)optr)[i].im = 0;

  ippsMul_32fc_I((Ipp32fc *)iptr, (Ipp32fc *)optr, noutput_items);

  return noutput_items;
}

float gr_mixer_cc::get_freq () const
{
    // Convert from radians / sample to Hz
    if (d_freq < 0.5)
        return d_freq * d_sampling_freq; 
    else 
        return (d_freq - 1.0) * d_sampling_freq;
}
void gr_mixer_cc::set_freq (float freq) 
{
    d_freq = freq / d_sampling_freq;
    if (d_freq < 0) d_freq = 1 + d_freq; // freq is [0..1.0]
    assert(d_freq >= 0 && d_freq <= 1.0);
}
void gr_mixer_cc::set_phase (float phase) 
{
    // Phase must be [0 - 2 pi]
    if (phase < 0) phase = M_TWOPI + phase;
    d_phase += phase;
    while (d_phase > M_TWOPI) d_phase -= M_TWOPI; // wrap 2 pi
    assert(d_phase > 0 && d_phase < M_TWOPI);
}
