#ifndef INCLUDED_GR_MIXER_CC_H
#define INCLUDED_GR_MIXER_CC_H

#include <gr_sync_block.h>
#include <gr_fxpt_nco.h>

class gr_mixer_cc;
typedef boost::shared_ptr<gr_mixer_cc> gr_mixer_cc_sptr;

//! Create a gr_mixer_cc block.
//! \param sampling_freq Sampling frequency in Hz
//! \param freq Mixer frequency in Hz
//! \param real Mix with a real waveform (cosine) when true.  Default:false
gr_mixer_cc_sptr
    gr_make_mixer_cc (float sampling_freq, float freq, bool real = false);

/*!
 * \brief output = input * exp(j*freq*t)
 * \ingroup block
 */
class gr_mixer_cc : public gr_sync_block
{
  friend gr_mixer_cc_sptr 
      gr_make_mixer_cc (float sampling_freq, float freq, bool real);

  float d_freq;
  float d_phase;
  float d_sampling_freq;

  bool d_real;

  gr_mixer_cc (float sampling_freq, float freq, bool real);

 public:
  //! Get the current frequency [Hz]
  float get_freq () const;
  //! Set a new frequency [Hz]
  void set_freq (float freq);
  //! Change the current phase. New phase = previous phase + phase.   Mostly
  //! for debugging since the phase is changing continuously when freq != 0
  void set_phase (float phase);

  int work (int noutput_items,
	    gr_vector_const_void_star &input_items,
	    gr_vector_void_star &output_items);
};

#endif
