
#ifndef INCLUDED_GR_multistage_decimator_cc_H
#define	INCLUDED_GR_multistage_decimator_cc_H

#include <gr_sync_decimator.h>
#include <vector>
#include <ipp.h>

using namespace std;

class gr_multistage_decimator_cc;
typedef boost::shared_ptr<gr_multistage_decimator_cc> 
    gr_multistage_decimator_cc_sptr;
gr_multistage_decimator_cc_sptr 
    gr_make_multistage_decimator_cc (int decimation);

/*!
 * Integer decimator with multiple decimation stages.  Decimation by a large
 * factor requires very sharp filters.  The sharp filters are very long and
 * computationally inefficient.  Instead, a large decimation ratio is 
 * broken into multiple stages, such that the product of the decimations in
 * each stage is approximately equal to the requested decimation. 
 * The resultant decimation may not be exactly equal to the requested!
 * The user should check the actual decimation using the decimation() function.
 * For example, a prime number may not be decomposed.  Each decimation stage
 * is no more than 6.  The filters are computed automatically using
 * gr_firdes::low_pass(1.0, 1.0, 0.5 / decim_ratio, 0.1);
 */
class gr_multistage_decimator_cc : public gr_sync_decimator
{
 private:
  friend gr_multistage_decimator_cc_sptr 
      gr_make_multistage_decimator_cc (int decimation);

  vector<IppsFIRState_32f *>   d_filt_re;
  vector<IppsFIRState_32f *>   d_filt_im;
  vector<vector<float> >       d_taps;
  vector<int>                  d_ratios;
  Ipp32f                       *d_buf_re, *d_buf_im;
  /*!
   * Construct a decimator
   */
  gr_multistage_decimator_cc (int decimation);

 public:
  ~gr_multistage_decimator_cc ();

  int work (int noutput_items,
		 gr_vector_const_void_star &input_items,
		 gr_vector_void_star &output_items);


  //! Get the taps for each decimation stage.  The vector length gives
  //! the number of stages.
  vector<vector<float> > &taps() {return d_taps;}
  //! Get the decimation ratio in each stage
  vector<int> &ratios() {return d_ratios;}
};

#endif
