
#ifndef INCLUDED_GR_multistage_interpolator_cc_H
#define	INCLUDED_GR_multistage_interpolator_cc_H

#include <gr_sync_interpolator.h>
#include <vector>
#include <ipp.h>

using namespace std;

class gr_multistage_interpolator_cc;
typedef boost::shared_ptr<gr_multistage_interpolator_cc> 
    gr_multistage_interpolator_cc_sptr;
gr_multistage_interpolator_cc_sptr 
    gr_make_multistage_interpolator_cc (int interpolation);

/*!
 * Integer interpolator with multiple decimation stages.  Interpolation by a 
 * large factor requires very sharp filters.  The sharp filters are very long,
 * computationally inefficient.  Instead, a large interpolation ratio is 
 * broken into multiple stages, such that the product of the interpolations in
 * each stage is approximately equal to the requested interpolation. 
 * The resultant interpolation may not be exactly equal to the requested!
 * The user should check the actual interpolation using the interpolation() 
 * function. For example, a prime number may not be decomposed.  Each 
 * interpolation stage is no more than 6.  The filters are computed using
 * gr_firdes::low_pass(1.0, 1.0, 0.5 / interp_ratio, 0.1);
 */
class gr_multistage_interpolator_cc : public gr_sync_interpolator
{
 private:
  friend gr_multistage_interpolator_cc_sptr 
      gr_make_multistage_interpolator_cc (int interpolation);

  vector<IppsFIRState_32f *>   d_filt_re;
  vector<IppsFIRState_32f *>   d_filt_im;
  vector<vector<float> >       d_taps;
  vector<int>                  d_ratios;
  Ipp32f                       *d_buf_re, *d_buf_im;
  /*!
   * Construct a interpolator
   */
  gr_multistage_interpolator_cc (int interpolation);

 public:
  ~gr_multistage_interpolator_cc ();

  int work (int noutput_items,
		 gr_vector_const_void_star &input_items,
		 gr_vector_void_star &output_items);


  //! Get the taps for each interpolation stage.  The vector length gives
  //! the number of stages.
  vector<vector<float> > &taps() {return d_taps;}
  //! Get the interpolation ratio in each stage
  vector<int> &ratios() {return d_ratios;}
};

#endif
