
#ifdef HAVE_CONFIG_H
#include "config.h"
#endif


#include <ar_qpsk_detect.h>
#include <gr_io_signature.h>
#include <ipp.h>


//#define BYTESOUT 


ar_qpsk_detect_sptr     ar_make_qpsk_detect ()
{
  return ar_qpsk_detect_sptr (new ar_qpsk_detect () );
}

ar_qpsk_detect::ar_qpsk_detect ()
  : gr_block ("qpsk_detect",
		      gr_make_io_signature (1, 1, sizeof (unsigned char)),
		      gr_make_io_signature (1, 1, sizeof (unsigned char)))               
{ }

ar_qpsk_detect::~ar_qpsk_detect()
{ }

void ar_qpsk_detect::forecast(int noutput_items, 
                              gr_vector_int &ninput_items_required)
{
  int ninputs   = 1;  
  ninput_items_required[0] = ( noutput_items  + DEPTH_TRELLIS) * 2;
}

int
ar_qpsk_detect::general_work (int noutput_items,
		     gr_vector_int &ninput_items,
		     gr_vector_const_void_star &input_items,
		     gr_vector_void_star &output_items){
		          
     // FYI:  DEPTH_TRELLIS=35		          
     float     *in_vec             =  (float *)   input_items[0];    
     static int synched            =0;  
     int       begin_state         =0;
     int       ni                  = ninput_items[0];     
     int       channel_length;	
	unsigned char  *vit_bytes     = (unsigned char *)   output_items[0];	
     //Use Multiples of Bytes. 16 = 8 * 1/CodeRate
     channel_length = int( floor(float(ni/16))*16); 
     // Warning: Viterbi Core uses Intel IPP functions that returns 16 bit shorts !, 
     //          not 8 bit unsignedchars
     short          *vit_out		     = new short [channel_length];
     static int     VitSynched          =0;
     int            consumed_bits_from_input = channel_length;      
     int            generated_output_bits ;     
       //----------
     // DECODE 
     //----------
     if (VitSynched==0){
          //We are not synched, so there's no point in decoding all bits, since we have
          // a 50% chance of being wrong
          // So we decode a small number of packets:  1000
          int  TestPkts=1000;
          if ( channel_length < TestPkts){               
               TestPkts=channel_length;
          }                           
          generated_output_bits    = TestPkts/2 - DEPTH_TRELLIS;         
          
          sdvd_intel( TestPkts, in_vec, vit_out	,&VitSynched,&begin_state); 

          //----------------------------------------------------------------------------
          // TEST SYNCH
          //------------     
          Ipp16s          *myEncodedBits;
          myEncodedBits  = ippsMalloc_16s(2* generated_output_bits);        
          //-------------------
          // ENCODE - Phase #1
          //------------------- 
          // Recall modulation here is   y=1-2*x ==> 0->1, 1->-1
		
          cnv_encd_intel(generated_output_bits, vit_out, myEncodedBits,&begin_state) ;

          int SynchThreshold    =int (floor( (generated_output_bits*2)/10.0)) ;
          int SumErrors=0;
          for (int ii=0; ii< 2* (generated_output_bits-DEPTH_TRELLIS); ii++){          
               SumErrors += ( (in_vec[ii ] < 0)!=myEncodedBits[ii+2*DEPTH_TRELLIS*(VitSynched!=0)]);
          }     
          
          printf("Sum Errors = %i \t  Threshold=%i \tBegining State=%i  \n", SumErrors,SynchThreshold,begin_state);      
         
          if (SumErrors < SynchThreshold){
               printf("---------------------------------------\n");
               printf("Viterbi Decoder: RX SIGNAL WAS SYNCHED \n");
               printf("---------------------------------------\n");
               // We are synched !!
               VitSynched     =1;       
               // Decode All PKTS
               generated_output_bits    = channel_length/2;   
               
               sdvd_intel( channel_length, in_vec, vit_out	,&VitSynched,&begin_state); 
			
               consume_each (channel_length);
          }else{
               // We are NOT synched !!          
               printf("---------------------------------------\n");
               printf("Viterbi Decoder: RX SIGNAL WAS SHIFTED \n");               
               // We need to decode our packets again since we were off by one bit
               // And our previos decoded packet was wrong
               Ipp32f          *myShiftedInput;
               myShiftedInput  = ippsMalloc_32f(channel_length-1);      
               for (int ii=0; ii<channel_length-1 ; ii++){          
                    myShiftedInput[ii] = in_vec[ii+1];
               } 
               VitSynched=2;     
               generated_output_bits    = channel_length/2;   
               // sdvd_intel( channel_length-1, in_vec, vit_out,&VitSynched,&begin_state);  
			
               //sdvd_intel( channel_length-1, in_vec,vit_out,&VitSynched,&begin_state);
               sdvd_intel( channel_length-1, myShiftedInput,vit_out,&VitSynched,&begin_state);

               consume_each (channel_length-1);          
               ippsFree(myShiftedInput);               
               printf("Viterbi Decoder: RX SIGNAL WAS CORRECTED\n");   
               printf("----------------------------------------\n");
          }
     }else{
          // -------------------------
          //   We are already synched
          // -------------------------
          generated_output_bits    = channel_length/2;
          //sdvd_intel( channel_length, in_vec, vit_out,&VitSynched,&begin_state); 
		
          sdvd_intel( channel_length, in_vec, vit_out	,&VitSynched,&begin_state);
          consume_each (channel_length);      
     }

    

     #ifdef BYTESOUT    

	     //printf("Generated OutputBits=%i \t No.Input Items=%i \t BitsUsed=Channel_length%i \n ",generated_output_bits,ni,channel_length);
	     unsigned char cumm_temp=0;
		//EV 
          //for (int nn = 1; nn < generated_output_bits+1; nn += 8){
		for (int nn = 0; nn < generated_output_bits; nn += 8){
               
               //for (int nn = 1; nn < generated_output_bits-8; nn += 8){	
               cumm_temp = 0;
			for (int mm = nn; mm < nn + 8; mm++){
				cumm_temp <<= 1;
				cumm_temp += (vit_out[mm] & 0x0001);
                    if(nn<10){			
                         //printf("[%i,%i,%i,%i]  \t ",nn,mm,nn-1>>3,cumm_temp);
                         //printf("[%i,%i,%i]  \t ",mm,vit_out[mm],cumm_temp);
                         //printf("[%i,%i,]   ",mm,vit_out[mm]);
                    }
			}
               if(nn<10){
			     //printf("--> out=%i \n ",cumm_temp);
               }
			vit_bytes[nn-1>>3] = cumm_temp; // same as out_vec[nn/8] = cumm_temp; without dividing
               //vit_bytes[nn>>3] = cumm_temp;
		}
          //------------------------------------
          // RETURN GENERATED OUTPUT BYTES  !!!
		return int(generated_output_bits/8);
		//------------------------------------
     #else          
          //EV The +1 offset is (for the time being) considered
          // as, mmm,... , magic !
          if (VitSynched  ==1){       
               // Originally Synched, so no shifting
               for (int nn = 0; nn < generated_output_bits; nn ++){	
                    //vit_bytes[nn]=(vit_out[nn] & 0x0001) ;  // --> Looses PKTS
                    vit_bytes[nn]=(vit_out[nn+1] & 0x0001) ;  // 
               }                 
          }else if(VitSynched  ==2){
               // Not Synched --> Synched
               for (int nn = 0; nn < generated_output_bits; nn ++){	
                    vit_bytes[nn+1]=(vit_out[nn+1] & 0x0001) ;
               }                 
          }
          return int(generated_output_bits);     
		
     #endif  
}
