#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <ar_ring_buffer_sink.h>
#include <gr_io_signature.h>

template <class T>
gr_ring_buffer_sink<T>::gr_ring_buffer_sink(bool allow_overflow, int size)
  : gr_sync_block ("gr_ring_buffer_sink_b",
		   gr_make_io_signature(1, 1, sizeof(T)),
		   gr_make_io_signature(0, 0, 0)),
    d_allow_overflow(allow_overflow), d_overflow(false)
{
    if (size == 0) size = GR_RING_BUFFER_SINK_SIZE;
    buffer = new circular_buffer<T>(size, false); 
}

gr_ring_buffer_sink_b_sptr
gr_make_ring_buffer_sink_b (bool allow_overflow, int size)
{
  return gr_ring_buffer_sink_b_sptr (new gr_ring_buffer_sink<char> (allow_overflow, size));
}

gr_ring_buffer_sink_c_sptr
gr_make_ring_buffer_sink_c (bool allow_overflow, int size)
{
  return gr_ring_buffer_sink_c_sptr (new gr_ring_buffer_sink<gr_complex> (allow_overflow, size));
}

template <class T>
gr_ring_buffer_sink<T>::~gr_ring_buffer_sink ()
{
    delete buffer;
}

// Copy data from the input buffer to the output buffer
template <class T>
int gr_ring_buffer_sink<T>::work (int noutput_items,
			    gr_vector_const_void_star &input_items,
			    gr_vector_void_star &output_items)
{ 
  T *in = (T *) input_items[0];

  // Copy no more than the requested number of items, and obviously,
  // not more than we have room for
  unsigned int data_len = min(buffer->n_avail_write_items(), noutput_items);
  if (data_len)
    buffer->enqueue(in, data_len);

  if (d_allow_overflow)
  {
      if (data_len < noutput_items)
      {
          if (!d_overflow) printf("gr_ring_buffer_sink_b(%s)::Overflow\n", d_name.c_str());
          d_overflow = true;
      }
      return noutput_items; // always consume all the input
  } else {
    return data_len;
  }
}