#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <ar_ring_buffer_sink2.h>
#include <gr_io_signature.h>

template <class T>
gr_ring_buffer_sink2<T>::gr_ring_buffer_sink2(int size)
  : gr_sync_block ("gr_ring_buffer_sink2_b",
		   gr_make_io_signature(1, 1, sizeof(T)),
		   gr_make_io_signature(0, 0, 0))
{
    if (size == 0) size = GR_RING_BUFFER_SINK_SIZE; 

    ready2stop = false;
	buffer = new circular_buffer<T>(size, true, true); 

    

}

gr_ring_buffer_sink2_b_sptr
gr_make_ring_buffer_sink2_b (int size)
{
  return gr_ring_buffer_sink2_b_sptr (new gr_ring_buffer_sink2<char> (size));
}

gr_ring_buffer_sink2_c_sptr
gr_make_ring_buffer_sink2_c (int size)
{
  return gr_ring_buffer_sink2_c_sptr (new gr_ring_buffer_sink2<gr_complex> (size));
}

gr_ring_buffer_sink2_r_sptr
gr_make_ring_buffer_sink2_r (int size)
{
  return gr_ring_buffer_sink2_r_sptr (new gr_ring_buffer_sink2<float> (size));
}


template <class T>
gr_ring_buffer_sink2<T>::~gr_ring_buffer_sink2 ()
{
    printf("Delete SINK Ring Buffer \n");
    delete buffer;
}

// Copy data from the input buffer to the output buffer
template <class T>
int gr_ring_buffer_sink2<T>::work (int noutput_items,
			    gr_vector_const_void_star &input_items,
			    gr_vector_void_star &output_items)
{ 
  T *in = (T *) input_items[0];

  unsigned int data_len = noutput_items;
  
  //if (data_len && (ready2stop==false))
  ///  buffer->enqueue(in, data_len);


  //THis is a hack, but slows down performance by a factor of ~ 2
  if (data_len&& (ready2stop==false)){
      if (buffer->n_avail_write_items() > data_len){
        buffer->enqueue(in, data_len);
      }
      else
          return 0;
  }

  return data_len;
}

//EV

template <class T>
void gr_ring_buffer_sink2<T>::gr_buffer_prepare2stop ()
{
    //printf("----> SINK Ring Buffer: Set Flag \n");
    ready2stop = true;
}

//-----------------------------------------------
// Thanks Alex Utter !!!! --> If we don't do this 
// we get a linking error !
// Create code for commonly used types
template class gr_ring_buffer_sink2<char>;
template class gr_ring_buffer_sink2<int>;
template class gr_ring_buffer_sink2<float>;
template class gr_ring_buffer_sink2<gr_complex>;
//-----------------------------------------------