
#ifndef INCLUDED_gr_ring_buffer_sink2_H
#define INCLUDED_gr_ring_buffer_sink2_H

#include <gr_sync_block.h>
#include "circular_buffer.h" // Thread-safe ring buffer

template <class T> class gr_ring_buffer_sink2;
typedef gr_ring_buffer_sink2<char> gr_ring_buffer_sink2_b;
typedef gr_ring_buffer_sink2<float> gr_ring_buffer_sink2_r;
typedef gr_ring_buffer_sink2<gr_complex> gr_ring_buffer_sink2_c;

typedef boost::shared_ptr<gr_ring_buffer_sink2_b> gr_ring_buffer_sink2_b_sptr;
typedef boost::shared_ptr<gr_ring_buffer_sink2_c> gr_ring_buffer_sink2_c_sptr;
typedef boost::shared_ptr<gr_ring_buffer_sink2_r> gr_ring_buffer_sink2_r_sptr;

//! Create a gr_ring_buffer_sink2.  Define a unique function for each data type
//! allow_overflow If true, the buffer always accept all the input
//! and never blocks.  If data overflows, it is discarded.
//! \param size Set the buffer size.  If zero, defaults to 16384 (2^14) entries
gr_ring_buffer_sink2_b_sptr gr_make_ring_buffer_sink2_b(int size = 0);

gr_ring_buffer_sink2_c_sptr gr_make_ring_buffer_sink2_c(int size = 0);

gr_ring_buffer_sink2_r_sptr gr_make_ring_buffer_sink2_r(int size = 0);


/*!
 * A general-purpose buffer to take data from the GNU radio environment  
 * to 'C++' world
 */
template <class T>
class gr_ring_buffer_sink2 : public gr_sync_block
{
  friend gr_ring_buffer_sink2_b_sptr 
      gr_make_ring_buffer_sink2_b(int size);

  friend gr_ring_buffer_sink2_c_sptr
      gr_make_ring_buffer_sink2_c(int size);

  friend gr_ring_buffer_sink2_r_sptr
      gr_make_ring_buffer_sink2_r(int size);

 private:
  circular_buffer<T> *buffer;

  //EV
  volatile bool ready2stop;
  
 protected:
  gr_ring_buffer_sink2(int size);

 public:	
  ~gr_ring_buffer_sink2();
  //EV
  //void  gr_buffer_prepare2stop() {ready2stop = true;}
  void  gr_buffer_prepare2stop();
  
  int work (int noutput_items,
		    gr_vector_const_void_star &input_items,
		    gr_vector_void_star &output_items);

  //! Return the buffer to the C++ world
  circular_buffer<T> *get_buffer() {return buffer;}

  
};

#define GR_RING_BUFFER_SINK_SIZE  (1<<14) // Default buffer size

#endif /* INCLUDED_gr_ring_buffer_sink2_H */
