#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <ar_ring_buffer_source.h>
#include <gr_io_signature.h>

template <class T>
gr_ring_buffer_source<T>::gr_ring_buffer_source(bool allow_underflow, int size)
  : gr_sync_block ("gr_ring_buffer_source_b",
		   gr_make_io_signature(0, 0, 0),
		   gr_make_io_signature(1, 1, sizeof(d_default_data))),
    d_allow_underflow(allow_underflow), d_underflow(false)
{
    if (size == 0) size = GR_RING_BUFFER_SOURCE_SIZE;
    buffer = new circular_buffer<T>(size, false); 
}

gr_ring_buffer_source_b_sptr
gr_make_ring_buffer_source_b (bool allow_underflow, int size)
{
  return gr_ring_buffer_source_b_sptr (new gr_ring_buffer_source<char> (allow_underflow, size));
}

gr_ring_buffer_source_c_sptr
gr_make_ring_buffer_source_c (bool allow_underflow, int size)
{
  return gr_ring_buffer_source_c_sptr (new gr_ring_buffer_source<gr_complex> (allow_underflow, size));
}

template <class T>
gr_ring_buffer_source<T>::~gr_ring_buffer_source ()
{
    delete buffer;
}

// Copy data from the input buffer to the output buffer
template <class T>
int gr_ring_buffer_source<T>::work (int noutput_items,
			    gr_vector_const_void_star &input_items,
			    gr_vector_void_star &output_items)
{ 
  T *out = (T *) output_items[0];

  // Copy no more than the requested number of items, and 
  // not more than we have 
  unsigned int data_len = min(buffer->n_avail_read_items(), noutput_items);
  if (data_len)
    buffer->dequeue(out, &data_len);

  if (d_allow_underflow)
  {
      if (data_len < noutput_items)
      {
          if (!d_underflow) printf("gr_ring_buffer_source(%s)::underflow\n", d_name.c_str());
          d_underflow = true;
          for(int i=data_len; i<noutput_items; ++i) out[i] = d_default_data;
      }
      return noutput_items; // always consume all the input
  } else {
    return data_len;
  }
}