#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <ar_ring_buffer_source2.h>
#include <gr_io_signature.h>

template <class T>
gr_ring_buffer_source2<T>::gr_ring_buffer_source2(circular_buffer<T> *input_buffer)
  : gr_sync_block ("gr_ring_buffer_source2_b",
		   gr_make_io_signature(0, 0, 0),
		   gr_make_io_signature(1, 1, sizeof(T)))
{
    ready2stop = false;
	buffer = input_buffer;
}

gr_ring_buffer_source2_b_sptr
gr_make_ring_buffer_source2_b (circular_buffer<char> *input_buffer)
{
  return gr_ring_buffer_source2_b_sptr (new gr_ring_buffer_source2<char> (input_buffer));
}

gr_ring_buffer_source2_c_sptr
gr_make_ring_buffer_source2_c (circular_buffer<gr_complex> *input_buffer)
{
  return gr_ring_buffer_source2_c_sptr (new gr_ring_buffer_source2<gr_complex> (input_buffer));
}

gr_ring_buffer_source2_r_sptr
gr_make_ring_buffer_source2_r (circular_buffer<float> *input_buffer)
{
  return gr_ring_buffer_source2_r_sptr (new gr_ring_buffer_source2<float> (input_buffer));
}



template <class T>
gr_ring_buffer_source2<T>::~gr_ring_buffer_source2 ()
{
    // EV:  WHy is this commented ????

    //printf("Delete Source Ring Buffer \n");
    //delete buffer;
}

// Copy data from the input buffer to the output buffer
template <class T>
int gr_ring_buffer_source2<T>::work (int noutput_items,
			    gr_vector_const_void_star &input_items,
			    gr_vector_void_star &output_items)
{ 
  T *out = (T *) output_items[0];

  unsigned int data_len = noutput_items;

  //EV: This is a hack, but slows down performance by a factor of ~ 2
  if (data_len&& (ready2stop==false)){
      if (buffer->n_avail_read_items() > data_len){
        buffer->dequeue(out, &data_len);
      }else
          return 0;
  }

  return data_len;
}

template <class T>
void gr_ring_buffer_source2<T>::gr_buffer_prepare2stop ()
{
    //printf("SOURCE Ring Buffer: Prepare to stop \n");
    ready2stop = true;
}

//-----------------------------------------------
// Thanks Alex Utter !!!! --> If we don't do this 
// we get a linking error !
// Create code for commonly used types
template class gr_ring_buffer_source2<char>;
template class gr_ring_buffer_source2<int>;
template class gr_ring_buffer_source2<float>;
template class gr_ring_buffer_source2<gr_complex>;
//-----------------------------------------------