
#ifndef INCLUDED_gr_ring_buffer_source2_H
#define INCLUDED_gr_ring_buffer_source2_H

#include <gr_sync_block.h>
#include "circular_buffer.h" // Thread-safe ring buffer

template <class T> class gr_ring_buffer_source2;
typedef gr_ring_buffer_source2<char> gr_ring_buffer_source2_b;
typedef boost::shared_ptr<gr_ring_buffer_source2_b> gr_ring_buffer_source2_b_sptr;
typedef gr_ring_buffer_source2<gr_complex> gr_ring_buffer_source2_c;
typedef boost::shared_ptr<gr_ring_buffer_source2_c> gr_ring_buffer_source2_c_sptr;

typedef gr_ring_buffer_source2<float> gr_ring_buffer_source2_r;
typedef boost::shared_ptr<gr_ring_buffer_source2_r> gr_ring_buffer_source2_r_sptr;

//! Create a gr_ring_buffer_source2.  Define a unique function for each data type
//! \param allow_underflow If true, the buffer always generates as much input
//! as requested by the scheduler.  If more data is requested than available,
//! the output data is padded with 'default_value'
//! \param size Set the buffer size.  If zero, defaults to 16384 (2^14) entries
gr_ring_buffer_source2_b_sptr gr_make_ring_buffer_source2_b(circular_buffer<char> *input_buffer);

gr_ring_buffer_source2_c_sptr gr_make_ring_buffer_source2_c(circular_buffer<gr_complex> *input_buffer);

gr_ring_buffer_source2_r_sptr gr_make_ring_buffer_source2_r(circular_buffer<float> *input_buffer);
/*!
 * A general-purpose buffer to take data from the 'C++' world to GNU radio 
 * environment
 */
template <class T>
class gr_ring_buffer_source2 : public gr_sync_block
{
  friend gr_ring_buffer_source2_b_sptr 
      gr_make_ring_buffer_source2_b(circular_buffer<char> *input_buffer);

  friend gr_ring_buffer_source2_c_sptr
      gr_make_ring_buffer_source2_c(circular_buffer<gr_complex> *input_buffer);
  
  friend gr_ring_buffer_source2_r_sptr
      gr_make_ring_buffer_source2_r(circular_buffer<float> *input_buffer);

 private:
  circular_buffer<T> *buffer;
  //EV
  volatile  bool ready2stop;
 protected:
  gr_ring_buffer_source2(circular_buffer<T> *input_buffer);

 public:
	
  ~gr_ring_buffer_source2();
   
   //EV
   //void gr_buffer_prepare2stop(){ready2stop = true;}   
   void gr_buffer_prepare2stop();   
   

  int work (int noutput_items,
		    gr_vector_const_void_star &input_items,
		    gr_vector_void_star &output_items);

  //! Return the buffer to the C++ world
  circular_buffer<T> *get_buffer() {return buffer;}
};

#define GR_RING_BUFFER_SOURCE_SIZE  (1<<14) // Default buffer size

#endif /* INCLUDED_gr_ring_buffer_source2_H */
