
#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include "ar_running_average.h"
#include <gr_io_signature.h>
#include <ipp.h>

gr_running_average_ff_sptr gr_make_running_average_ff (int average_length)
{
  return gr_running_average_ff_sptr (new gr_running_average<float> (average_length));
}

gr_running_average_cc_sptr gr_make_running_average_cc (int average_length)
{
  return gr_running_average_cc_sptr (new gr_running_average<gr_complex> (average_length));
}

template <class T>
gr_running_average<T>::gr_running_average<T> (int average_length)
  : gr_sync_block ("running_average",
		       gr_make_io_signature (1, 1, sizeof (gr_complex)),
		       gr_make_io_signature (1, 1, sizeof (gr_complex))),
    d_average_length(average_length), d_acc(0)
{
    set_history(d_average_length+1);
}


template <class T>
int
gr_running_average<T>::work (int noutput_items,
		   gr_vector_const_void_star &input_items,
		   gr_vector_void_star &output_items)
{
    T *in = (T *) input_items[0] + d_average_length;
    T *out = (T *) output_items[0];

    noutput_items -= d_average_length; // can't use the last symbol's worth
    noutput_items = std::max(0, noutput_items); // if not enough, just 0

    for (int n=0; n<noutput_items; n++)
    {
        d_acc += in[n] - in[n-d_average_length];
        out[n] = d_acc;
    }

    return noutput_items;
}
