
#ifndef INCLUDED_GR_running_average_H
#define	INCLUDED_GR_running_average_H

#include <gr_sync_block.h>

template <class T> class gr_running_average;
typedef boost::shared_ptr<gr_running_average<float> > gr_running_average_ff_sptr;
typedef boost::shared_ptr<gr_running_average<gr_complex> > gr_running_average_cc_sptr;
//! Create a gr_running_average for float input and output
gr_running_average_ff_sptr gr_make_running_average_ff (int average_length);
//! Create a gr_running_average for complex input and output
gr_running_average_cc_sptr gr_make_running_average_cc (int average_length);


/*!
 * Compute the running average (actually the running sum) over the past N 
 * samples. y(n) = y(n-1) + x(n) - x(n-N)
 */
template <class T>
class gr_running_average : public gr_sync_block
{
 private:
  friend gr_running_average_ff_sptr gr_make_running_average_ff (int average_length);
  friend gr_running_average_cc_sptr gr_make_running_average_cc (int average_length);

  T d_acc;
  int d_average_length;

  gr_running_average(int average_length);
 public:

  int work (int noutput_items,
		 gr_vector_const_void_star &input_items,
		 gr_vector_void_star &output_items);

};

#endif
