   
#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <ar_snr_estimator_cf.h>
#include <gr_io_signature.h>
#include <cmath>

gr_snr_estimator_cf_sptr
gr_make_snr_estimator_cf(const std::vector<gr_complex> &symbol_map)
{
  return gr_snr_estimator_cf_sptr(new gr_snr_estimator_cf(symbol_map));
}

gr_snr_estimator_cf::gr_snr_estimator_cf(const std::vector<gr_complex> &symbol_map)
  : gr_sync_block("snr_estimator_cf",
		  gr_make_io_signature(1, 1, sizeof(gr_complex)),
		  gr_make_io_signature(0, 1, sizeof(float))),
  d_symbol_map(symbol_map), d_mean_abs(0), d_var_abs(0), d_alpha(0.01)
{
    // Compute the mean of the constellation point magnitudes
    d_map_magnitude = 0;
    for (int i=0; i<d_symbol_map.size(); i++) 
        d_map_magnitude += abs(d_symbol_map[i]);
    d_map_magnitude /= d_symbol_map.size();
}

gr_snr_estimator_cf::~gr_snr_estimator_cf()
{
}

float gr_snr_estimator_cf::get_snr_estimate()
{
    return 10*log10(d_map_magnitude*d_map_magnitude/d_var_abs);
}

int
gr_snr_estimator_cf::work (int noutput_items,
		   gr_vector_const_void_star &input_items,
		   gr_vector_void_star &output_items)
{
    gr_complex *in = (gr_complex *) input_items[0];
    // There may be an output port, or not.
    float *out = (output_items.size()) ? (float *)output_items[0] : 0;

    for (int i = 0; i< noutput_items; i++)
    {
        // Scale the input magnitude to match the nominal constellation
        d_mean_abs = (1-d_alpha) * d_mean_abs + d_alpha * abs(in[i]);

        in[i] = in[i] / d_mean_abs * d_map_magnitude;

        // Find the point in the map closest to the received point
        float min_delta = 1e20;
        for (int j=0; j<d_symbol_map.size(); j++)
        {
            float delta = norm(d_symbol_map[j] - in[i]);
            if (delta < min_delta) min_delta = delta;
        }

        // Compute the error variance
        d_var_abs  = (1-d_alpha) * d_var_abs  + d_alpha * min_delta;
        
        if (out) out[i] = get_snr_estimate();
    }
    return noutput_items;
}
