/* -*- c++ -*- */
#ifndef INCLUDED_GR_snr_estimator_ff_H
#define INCLUDED_GR_snr_estimator_ff_H

#include <gr_sync_block.h>
#include <string>

using namespace std;

class gr_snr_estimator_ff;
typedef boost::shared_ptr<gr_snr_estimator_ff> gr_snr_estimator_ff_sptr;

//! Create a gr_snr_estimator_ff block.  See ::gr_snr_estimator_ff.
//! Set the nominal symbol map.  BPSK only for now!!!!! BUGBUG
gr_snr_estimator_ff_sptr 
    gr_make_snr_estimator_ff(const std::vector<float> &symbol_map);

//! Estimate the slicer Eb/No based on the soft decisions.  The estimate
//! is based on 1 sample/symbol and is given by:
//! mean(hard_decision(signal))^2 / var(signal - hard_decision(signal) )
//! The output is in dB -- 10 log10(EbNo)
//! The EbNo is averaged by computing the average of the mean and the variance
//! using : mean = (1-alpha) * mean + alpha*mean, and likewise for variance.
//!
//! This block may be used as a sink (get estimates by calling the 
//! gr_snr_estimator_ff::get_snr_estimate() function, or have downstream
//! blocks attached to it.
class gr_snr_estimator_ff : public gr_sync_block
{
  friend gr_snr_estimator_ff_sptr 
      gr_make_snr_estimator_ff(const std::vector<float> &symbol_map);

  float d_mean_abs, d_var_abs;
  float d_alpha;
  float d_map_magnitude;
  std::vector<float> d_symbol_map;

  gr_snr_estimator_ff(const std::vector<float> &symbol_map);

public:
    ~gr_snr_estimator_ff();

    //! Get the current EbNo estimate [dB]
    float get_snr_estimate();
    //! Set the averaging constant, alpha
    void set_averaging(float alpha) {d_alpha = alpha;}

    int work (int noutput_items,
	    gr_vector_const_void_star &input_items,
	    gr_vector_void_star &output_items);
};
 

#endif /* INCLUDED_GR_snr_estimator_ff_H */
