/* -*- c++ -*- */
#ifndef INCLUDED_GR_THROUGHPUT_H
#define INCLUDED_GR_THROUGHPUT_H

#include <gr_sync_block.h>
#include <string>

using namespace std;

class gr_throughput;
typedef boost::shared_ptr<gr_throughput> gr_throughput_sptr;

//! Create a gr_throughput block.
//! \param update_rate Set the output frequency for the average throughput [s]
//! \param filename Set the output target.  Set to 'cout' to print to screen
//! \param label Prefix the throughput with this label.  Useful if multiple
//! gr_throughput blocks are running
gr_throughput_sptr gr_make_throughput(size_t itemsize, double update_rate, 
				   const char *filename, const char *label);

//! Estimates the throughput in samples/sec and reports every update_rate
class gr_throughput : public gr_sync_block
{
  friend gr_throughput_sptr gr_make_throughput(size_t itemsize,
			double update_rate, const char *filename, const char *label);
  double		d_update_rate;
  unsigned		d_total_samples;
  unsigned		d_prev_time;
  FILE          *d_outfile;
  string		d_label;

  gr_throughput(size_t itemsize, double update_rate, 
				const char *filename, const char *label);

public:
  ~gr_throughput();

  int work (int noutput_items,
	    gr_vector_const_void_star &input_items,
	    gr_vector_void_star &output_items);
};
 

#endif /* INCLUDED_GR_THROUGHPUT_H */
