
#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <ar_viterbi_lib.h>
#include <ar_viterbi.h>
#include <gr_io_signature.h>
#include <ipp.h>
#include <stdio.h>

// Don't output bytes !!
// Output Bits and then connect to a bits-to-bytes converter


gr_viterbi_sptr     gr_make_viterbi(int modulation)
{
    return gr_viterbi_sptr (new gr_viterbi (modulation) );
}

gr_viterbi_sptr     gr_make_viterbi()
{
    return gr_viterbi_sptr (new gr_viterbi (2) );
}

gr_viterbi::gr_viterbi (int modulation)
        : gr_block ("viterbi",
                    gr_make_io_signature (1, 1, sizeof (float)),
                    gr_make_io_signature (1, 1, sizeof (unsigned char)))  ,
        d_modulation(modulation)
{ }

gr_viterbi::~gr_viterbi()
{ }

void gr_viterbi::forecast(int noutput_items,
                          gr_vector_int &ninput_items_required)
{
    ninput_items_required[0] = ( noutput_items  + DEPTH_TRELLIS) * 2;     
}




int
gr_viterbi::general_work (int                       noutput_items,
                          gr_vector_int             &ninput_items,
                          gr_vector_const_void_star &input_items,
                          gr_vector_void_star       &output_items){

    // FYI:  DEPTH_TRELLIS=35
    float     *in_vec             =  (float *)   input_items[0];
    int       begin_state         =0;
    int       begin_state_tmp     =0;
    int       ni                  = ninput_items[0];
    int       channel_length;
    unsigned char  *vit_bytes     = (unsigned char *)   output_items[0];
    //Use Multiples of Bytes. 16 = 8 * 1/CodeRate
    channel_length = (ni/16)*16;
    // Warning: Viterbi Core uses Intel IPP functions that returns 16 bit shorts !,
    //          not 8 bit unsignedchars
    short          *vit_out		     = new short [channel_length];
    static int     VitSynched          =0;    
    int            generated_output_bits ;
    //----------
    // DECODE
    //----------
    static int swipe=0;
    swipe++;
    if (swipe>10000){
        swipe=10000;
    }

    static int print_offset=0;

//  Uncomment to prevent Phase testing

    if (VitSynched==0){
        //We are not synched, so there's no point in decoding all bits, since we have
        // a 50% chance of being wrong
        // So we decode a small number of packets:  1000
        int  TestPkts=5000;
        if ( channel_length < TestPkts){
            TestPkts=channel_length;
        }
        // Discard the Initial Bits for testing
        int StartTest               = 12000;
        generated_output_bits       = TestPkts/2 - DEPTH_TRELLIS;        
        if (StartTest + generated_output_bits > channel_length){
            StartTest = channel_length - generated_output_bits;
        }
       
        printf("StartTest=%i \t TestBits= %i OUTBits=%i \t\t ChannLength=%i \n",
            StartTest,generated_output_bits,generated_output_bits, channel_length);
          
        // Input has length  = channel_length
        Ipp32f          *myShiftedInput;
        myShiftedInput  = ippsMalloc_32f(channel_length);

        // Generated Output Bits has half the length of the channel
               
        Ipp16s          *myEncodedBits;
        // This is a temporary vector, used to encode the previously decoded test bits
        myEncodedBits  = ippsMalloc_16s(2* generated_output_bits);

        //----------------------------------------------------------------------------
        // TEST SYNCH
        //------------
        if (d_modulation==2){ // The other possibility is the correct one

            for (int ii=0; ii<TestPkts ; ii++){
                myShiftedInput[ii] = in_vec[ii+StartTest];                
            }   
            //sdvd_intel( TestPkts, in_vec, vit_out	,&VitSynched,&begin_state);
            //sdvd_intel( TestPkts, in_vec, vit_out	,&VitSynched,&begin_state_tmp);
            sdvd_intel( TestPkts, myShiftedInput, vit_out	,&VitSynched,&begin_state_tmp);
            // vit_out has length= TestPkts/2

            //----------------------
            // ENCODE - No Rotation
            //----------------------
            // Recall modulation here is   y=1-2*x ==> 0->1, 1->-1
            //  generated_output_bits = Bits to encode
            begin_state_tmp     =0;           
            // Input for Encoder    = generated_output_bits
            // Output from Encoder  =  VitOut of size 2*generated_output_bits
            generated_output_bits       = TestPkts/2 - DEPTH_TRELLIS;      
            cnv_encd_intel(generated_output_bits, vit_out, myEncodedBits,&begin_state_tmp) ;
            int SynchThreshold    =int (floor( (generated_output_bits*2)/10.0)) ;           
            int SumTmp=0;
            for (int ii=0; ii<  2*generated_output_bits; ii++){ // Compare Encoded PKTS
                SumTmp += ( (myShiftedInput[ii ] < 0)!=myEncodedBits[ii]);
                
                #ifdef DEBUG0
                if ( (myShiftedInput[ii ] < 0)!=myEncodedBits[ii]){
                    printf("AB<-(A,B) ERROR  %i : in=%f->%i  \t Encoded=%i \n", 
                        ii,myShiftedInput[ii],myShiftedInput[ii]<0, myEncodedBits[ii]);
                    }
                #endif

            }
            printf("AB<-(A,B) Sum Errors = %i \t  Threshold=%i \t BER=%4.3e \tBegining State=%i  \n",
                   SumTmp,SynchThreshold,float(SumTmp)/float(2*(generated_output_bits-DEPTH_TRELLIS)),begin_state_tmp);

            if (SumTmp < SynchThreshold){
                printf("-----------------------------------------\n");
                printf("Viterbi Decoder: NO PHASE SHIFT REQUIRED \n");
                printf("-----------------------------------------\n");
                // We are synched !!
                
                // Decode All PKTS
                generated_output_bits    = channel_length/2; // This is already an even number
                sdvd_intel( channel_length, in_vec, vit_out	,&VitSynched,&begin_state);
                for (int nn = 0; nn < generated_output_bits-DEPTH_TRELLIS; nn ++){
                    vit_bytes[nn]=(vit_out[nn] & 0x0001) ;
                }
                VitSynched     =1;
                ippsFree(myEncodedBits);
                consume_each (channel_length);
                return generated_output_bits-DEPTH_TRELLIS;

            }else{
                // We are NOT synched !!
                printf("---------------------------------------\n");
                printf("Viterbi Decoder: SHIFT 180(deg) \n");
                // We need to decode our packets again since we were off by one bit
                // And our previos decoded packet was wrong        
                //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
                //Ipp32f          *myShiftedInput;
                //myShiftedInput  = ippsMalloc_32f(channel_length-1);
                for (int ii=0; ii<channel_length-1 ; ii++){
                    myShiftedInput[ii] = in_vec[ii+1];                   
                }            
                // DECODE WHOLE VECTOR
                generated_output_bits    = channel_length/2-1;
                //sdvd_intel( channel_length-1, in_vec,vit_out,&VitSynched,&begin_state);
                sdvd_intel( channel_length-1, myShiftedInput,vit_out,&VitSynched,&begin_state);

                
                //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
                // DEBUGGIN                       
                myEncodedBits  = ippsMalloc_16s(channel_length);
                SumTmp=0;   
                begin_state_tmp     =0;  
                // Input for Encoder    = generated_output_bits
                // Output from Encoder  =  VitOut of size 2*generated_output_bits
                cnv_encd_intel(generated_output_bits, vit_out, myEncodedBits,&begin_state_tmp) ;
                for (int ii=0; ii< 2* (generated_output_bits-DEPTH_TRELLIS); ii++){
                    SumTmp += ( (myShiftedInput[ii] < 0)!=myEncodedBits[ii]);
                }
                printf("AB<-(B,A) Errors = %i \t Tot.Bits=%i \t  Threshold=%i \t BER=%4.3e \t Begining State=%i  \n",
                   SumTmp, 2* (generated_output_bits-DEPTH_TRELLIS),SynchThreshold, float(SumTmp)/float(2*(generated_output_bits-DEPTH_TRELLIS)), begin_state_tmp );
                //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~   
                printf("----------------------------------------\n");
                
                VitSynched     =1;
                ippsFree(myEncodedBits);
                for (int nn = 0; nn <generated_output_bits; nn ++){
                    vit_bytes[nn]=(vit_out[nn] & 0x0001) ;
                }   
                consume_each (channel_length-1);
                ippsFree(myShiftedInput);
                return generated_output_bits-DEPTH_TRELLIS;

            }
        }//else modulation ==4
    

    }else{ 
        // ALREADY SYNCHED
        // -----------------------------------------------------------------
        //   NOW We are synched
        // -----------------------------------------------------------------
        generated_output_bits    = channel_length/2;
        Ipp32f                   *myShiftedInput;       

        if (VitSynched==1){    //        	AB<-(A,B)
            //NO ROTATION
            sdvd_intel( channel_length, in_vec, vit_out	,&VitSynched,&begin_state);
        }
    }  

    for (int nn = 0; nn <generated_output_bits; nn ++){
        vit_bytes[nn]=(vit_out[nn] & 0x0001) ;
    }

    consume_each (channel_length);
    return (generated_output_bits);


}
