
#include "ar_viterbi_lib.h"


void cnv_encd_intel(long input_len, Ipp16s *in_array, Ipp16s *out_array,int *begin_state) {

     long t, tt;                //  bit time, symbol time
     int j, k;                  //  loop variables
     int sr_head;               //  index to the first elt in the sr
     int p, q;                  //  the upper and lower xor gate outputs

     //  allocate space for the zero-padded input data array
     IppStatus                 st;     
     //  zero-pad the end of the data
     Ipp16s                    *uncoded_data_intel;
       
     uncoded_data_intel       = ippsMalloc_16s(input_len + M*TERMINATE_TRELLIS  );
     st = ippsZero_16s(uncoded_data_intel, input_len + M*TERMINATE_TRELLIS ); 
     st = ippsCopy_16s(in_array, uncoded_data_intel, input_len  );
   
     //--------------------------------------------------------
     //  Initialize the shift register   
     
     // This is still the faster way     
     int shift_reg[K];          //  the encoder shift register
     if( begin_state[0]==0){
          for (j = 0; j < K; j++) {
               shift_reg[j] = 0;
          }        
     }else{
          int shift_regTMP[M];    
          deci2bin( begin_state[0], M, shift_regTMP);          
          
          shift_reg[0]=0;          
          for (int jj = 0; jj < M; jj++) {
               //shift_reg[jj+1] = shift_regTMP[M-jj-1];
               shift_reg[jj+1] = shift_regTMP[jj];
          }

     }
     
     //int *shift_reg;  
     //shift_reg = (int*) calloc (K,sizeof(int));
     //
     // FOR SUCH A SMALL VECTOR, NORMAL Types could be faster 
     //Ipp16s          shift_reg[K];
     //st              =ippsZero_16s(shift_reg,K);
     //--------------------------------------------------------     
     //  To try to speed things up a little, the shift register will be operated
     // as a circular buffer, so it needs at least a head pointer. It doesn't
     // need a tail pointer, though, since we won't be taking anything out of
     // it--we'll just be overwriting the oldest entry with the new data.
     sr_head = 0;

     //  initialize the channel symbol output index
     tt = 0;

     //  Now start the encoding process
     //  compute the upper and lower mod-two adder outputs, one bit at a time
     for (t = 0; t < input_len + TERMINATE_TRELLIS*M; t++) {

          // shift_reg[sr_head] = *( unencoded_data + t );
          
         shift_reg[sr_head] = uncoded_data_intel[t];
          
          p = 0;
          q = 0;
          for (j = 0; j < K; j++) {
               k = (j + sr_head) % K;
               p ^= shift_reg[k] & g[0][j];
               q ^= shift_reg[k] & g[1][j];
          }

          // write the upper and lower xor gate outputs as channel symbols
          *(out_array + tt) = p;
          tt = tt + 1;
          *(out_array + tt) = q;
          tt = tt + 1;
          sr_head -= 1;    // equivalent to shifting everything right one place
          if (sr_head < 0){ // but make sure we adjust pointer modulo K
               sr_head = M;
          }
     }
     //  free the dynamically allocated array   
     ippsFree(uncoded_data_intel);  
}



void sdvd_intel(long channel_length, Ipp32f *channel_output_vector, Ipp16s *decoder_output_matrix,int *VitSynched,int *begin_state) {

     int i, j, l, ll;                              // loop variables
     long t;                                       // time
     
     Ipp16s memory_contents[K];                        // input + conv. encoder sr     
     static Ipp32s accum_err_metric0[NOSTATES];           // accumulated error metrics
     static Ipp32s accum_err_metric1[NOSTATES];           // accumulated error metrics
     Ipp32s MyMin;
     int    MyMinIndx;
     
     int        step, branch_metric, sh_col; // misc variables
     int       sh_ptr;
     
     static int sh_ptr_previous=0;
     
     IppStatus           st;    
  
     Ipp32s *channel_output_matrix;                // ptr to input matrix
     channel_output_matrix = ippsMalloc_32s( channel_length );        
     FILE	     *pFile;       
     // **************************************************************************

     // Instantiate Vars to Zero or MAXINT
     static Ipp16s state_history[NOSTATES][DEPTH_TRELLIS + 1];   // state history table
     static Ipp16s state_sequence[DEPTH_TRELLIS+ 1];             // state sequence list
    
     
     if( VitSynched[0]==0){
          st   = ippsZero_16s(*state_history, NOSTATES*(DEPTH_TRELLIS + 1));
          st   = ippsZero_16s(state_sequence, (DEPTH_TRELLIS+ 1));         
          st   = ippsZero_32s(accum_err_metric0, NOSTATES);
          st   = ippsSet_32s(MAXINT, accum_err_metric1, NOSTATES);    
          sh_ptr_previous =0;
          // printf('Maxint=%i \n', 32000);
     }else{
          // Consider this for the future, not needed now
          /*
          for (int ii=0;ii< NOSTATES;ii++){
               accum_err_metric0[ii] = accum_err_metric0[ii]- 10000;
          }
          */         
          
          // If we are synched, begin state, shoul be 
          // the 1st state where previous iteration left
          begin_state[0]= state_sequence[0] ;      
     }   
     
                             

     // now we're going to rearrange the channel output so it has N rows,
     // and N/2 columns where each row corresponds to a channel symbol for
     // a given bit and each column corresponds to an encoded bit
     channel_length = channel_length / N;

     // interesting to compare performance of fixed vs adaptive quantizer
     //  init_quantizer_intel(quantizer_tabl);
     // This guy uses a funky quantization here !!!

     // quantize the channel output--convert float to short integer
     // channel_output_matrix = reshape(channel_output, N, channel_length)
     for (t = 0; t < (channel_length * N); t += N) {
          for (i = 0; i < N; i++){
          *(channel_output_matrix + (t / N) + (i * channel_length) ) =
          soft_quant_intel( *(channel_output_vector + (t + i) ), quantizer_table );

          }
     } // end t for-loop

     // End of setup.
     // **************************************************************************
     
     
     //-----------------
     // DECODE
     //-----------------
     // Start decoding of channel outputs with forward
     //  traversal of trellis!  Stop just before encoder-flushing bits.
     
     // RECALL , at this point due to the line that says:
     //             channel_length = channel_length / N;
     // channel_length = No of Info Bits 
     //
     
     for (t = 0; t < channel_length ; t++) {

           // Consider all states fromt he begining
          step=1;

          // we're going to use the state history array as a circular buffer so
          // we don't have to shift the whole thing left after each bit is
          // processed so that means we need an appropriate pointer

          // set up the state history array pointer for this time t
          sh_ptr = (int) ( ( sh_ptr_previous+ t + 1 ) % (DEPTH_TRELLIS + 1) );
  
          //=====================================================================================

          // repeat for each possible state
          for (j = 0; j < NOSTATES; j+= step) {
               // repeat for each possible convolutional encoder output N-tuple
               for (l = 0; l < N; l++) {
                    if(l==0){                 
                         branch_metric =                           
                              abs( *( channel_output_matrix + t  ) - binary_outputV00[j] ) +
                              abs( *( channel_output_matrix + channel_length + t ) - binary_outputV01[j] );                                   
                    }else{                        
                         branch_metric =  
                              abs( *( channel_output_matrix + t  ) - binary_outputV10[j] ) +
                              abs( *( channel_output_matrix + channel_length + t ) - binary_outputV11[j] );
                    }
                    // now choose the surviving path--the one with the smaller accumlated error metric...
                    if ( accum_err_metric1[ nextstate[j][l] ]  > accum_err_metric0[j] + branch_metric ) {
                         // save an accumulated metric value for the survivor state
                         accum_err_metric1[ nextstate[j][l] ] = accum_err_metric0[j] + branch_metric;
                         // update the state_history array with the state number of
                         // the survivor
                         state_history[ nextstate[j][l] ] [sh_ptr] = j;
                    } // end of if-statement
               } // end of 'l' for-loop
          } // end of 'j' for-loop -- we have now updated the trellis       

          // for all rows of accum_err_metric, move col 2 to col 1 and flag col 2
          st = ippsMove_32s(accum_err_metric1,accum_err_metric0,NOSTATES);
          st = ippsSet_32s(MAXINT, accum_err_metric1, NOSTATES);          

          // now start the traceback, if we've filled the trellis
         if (VitSynched[0]==0){
              if (t >= DEPTH_TRELLIS - 1) {
                    // initialize the state_sequence vector--probably unnecessary
                    st = ippsZero_16s(state_sequence, (DEPTH_TRELLIS+ 1));
     
                    // find the element of state_history with the min. accum. error metric
                    // since the outer states are reached by relatively-improbable runs
                    // of zeroes or ones, search from the top and bottom of the trellis in
                    // st = ippsMin_32s( accum_err_metric0,NOSTATES,&MyMin );
                    st = ippsMinIndx_32s( accum_err_metric0,NOSTATES,&MyMin, &MyMinIndx );
                    // now pick the starting point for traceback
                    state_sequence[DEPTH_TRELLIS] = MyMinIndx;               
     
                    // now work backwards from the end of the trellis to the oldest state
                    // in the trellis to determine the optimal path. The purpose of this
                    // is to determine the most likely state sequence at the encoder
                    //  based on what channel symbols we received.
                    for (j = DEPTH_TRELLIS; j > 0; j--) {
                         sh_col = j + ( sh_ptr - DEPTH_TRELLIS );
                         if (sh_col < 0){
                              sh_col = sh_col + DEPTH_TRELLIS + 1;
                         }
                         state_sequence[j - 1] = state_history[ state_sequence[j] ] [sh_col];
                    } // end of j for-loop
     
                    // now figure out what input sequence corresponds to the state sequence
                    //in the optimal path

                    *(decoder_output_matrix + t - DEPTH_TRELLIS + 1) =
                         (nextstate[state_sequence[0]][1]==state_sequence[1] );
                         
                    if (t == DEPTH_TRELLIS - 1 ) {
                         // Save wining state
                         begin_state[0]= state_sequence[0] ;  
                    }        
         
               } // end of if-statement
          
          }else{                           
               // initialize the state_sequence vector--probably unnecessary
               st = ippsZero_16s(state_sequence, (DEPTH_TRELLIS+1));

               // find the element of state_history with the min. accum. error metric
               // since the outer states are reached by relatively-improbable runs
               // of zeroes or ones, search from the top and bottom of the trellis in
               // st = ippsMin_32s( accum_err_metric0,NOSTATES,&MyMin );
               st = ippsMinIndx_32s( accum_err_metric0,NOSTATES,&MyMin, &MyMinIndx );
               // now pick the starting point for traceback
               state_sequence[DEPTH_TRELLIS] = MyMinIndx;   
                           
               // now work backwards from the end of the trellis to the oldest state
               // in the trellis to determine the optimal path. The purpose of this
               // is to determine the most likely state sequence at the encoder
               //  based on what channel symbols we received.
               for (j = DEPTH_TRELLIS; j > 0; j--) {
                    sh_col = j + ( sh_ptr - DEPTH_TRELLIS );
                    if (sh_col < 0){
                         sh_col = sh_col + DEPTH_TRELLIS + 1;
                    }
                    state_sequence[j - 1] = state_history[ state_sequence[j] ] [sh_col];
               } // end of j for-loop
         
               decoder_output_matrix [ t+1 ] =
                    (nextstate[state_sequence[0]][1]==state_sequence[1] );     
           
          }                  
     } // end of 't' for-loop
     
     // **************************************************************************

     // free the dynamically allocated array storage area
     ippsFree(channel_output_matrix);

     // VitSynched[0] +=1;    // We are synched now !  
     sh_ptr_previous = sh_ptr;    
     
     return;
} // end of function sdvd


// ********************* END OF Soft Decision Viterbi Decoder FUNCTION *******************************                            

/* this quantizer assumes that the mean channel_symbol value is +/- 1,
and translates it to an integer whose mean value is +/- 32 to address
the lookup table "quantizer_table". Overflow protection is included.
*/
int soft_quant_intel(float channel_symbol,Ipp16s *quantizer_table) {
     int x;

     x = (int) ( 128 * channel_symbol );
     if (x < -128) x = -128;
     if (x > 127) x = 127;
     return(quantizer_table[x + 128]);
}

/* this function converts a decimal number to a binary number, stored
   as a vector MSB first, having a specified number of bits with leading
   zeroes as necessary */
void deci2bin(int d, int size, int *b) {
    int i;
 
    for(i = 0; i < size; i++)
        b[i] = 0;
 
    b[size - 1] = d & 0x01;
 
    for (i = size - 2; i >= 0; i--) {
        d = d >> 1;
        b[i] = d & 0x01;
    }
}


void sdvd_test_intel(long channel_length, Ipp32f *channel_output_vector, Ipp16s *decoder_output_matrix,int *VitSynched,int *begin_state) {

     int i, j, l, ll;                              // loop variables
     long t;                                       // time
     
     Ipp16s memory_contents[K];                        // input + conv. encoder sr     
     static Ipp32s accum_err_metric0[NOSTATES];           // accumulated error metrics
     static Ipp32s accum_err_metric1[NOSTATES];           // accumulated error metrics
     Ipp32s MyMin;
     int    MyMinIndx;
     
     int        step, branch_metric, sh_col; // misc variables
     int       sh_ptr;
     
     static int sh_ptr_previous=0;
     
     IppStatus           st;    
  
     Ipp32s *channel_output_matrix;                // ptr to input matrix
     channel_output_matrix = ippsMalloc_32s( channel_length );        
     FILE	     *pFile;       
     // **************************************************************************

     // Instantiate Vars to Zero or MAXINT
     static Ipp16s state_history[NOSTATES][DEPTH_TRELLIS + 1];   // state history table
     static Ipp16s state_sequence[DEPTH_TRELLIS+ 1];             // state sequence list
    
     
     if( VitSynched[0]==0){
          st   = ippsZero_16s(*state_history, NOSTATES*(DEPTH_TRELLIS + 1));
          st   = ippsZero_16s(state_sequence, (DEPTH_TRELLIS+ 1));         
          st   = ippsZero_32s(accum_err_metric0, NOSTATES);
          st   = ippsSet_32s(MAXINT, accum_err_metric1, NOSTATES);    
          sh_ptr_previous =0;
          // printf('Maxint=%i \n', 32000);
     }else{
          // Consider this for the future, not needed now
          // This will prevent the array from blowing up on the long run
          for (int ii=0;ii< NOSTATES;ii++){
               accum_err_metric0[ii] = accum_err_metric0[ii]- 10000;
          }                  
          
          // If we are synched, begin state, shoul be 
          // the 1st state where previous iteration left
          begin_state[0]= state_sequence[0] ;      
     }   
     
                             

     // now we're going to rearrange the channel output so it has N rows,
     // and N/2 columns where each row corresponds to a channel symbol for
     // a given bit and each column corresponds to an encoded bit
     channel_length = channel_length / N;

     // interesting to compare performance of fixed vs adaptive quantizer
     //  init_quantizer_intel(quantizer_tabl);
     // This guy uses a funky quantization here !!!

     // quantize the channel output--convert float to short integer
     // channel_output_matrix = reshape(channel_output, N, channel_length)
     for (t = 0; t < (channel_length * N); t += N) {
          for (i = 0; i < N; i++){
          *(channel_output_matrix + (t / N) + (i * channel_length) ) =
          soft_quant_intel( *(channel_output_vector + (t + i) ), quantizer_table );

          }
     } // end t for-loop

     // End of setup.
     // **************************************************************************
     
     
     //-----------------
     // DECODE
     //-----------------
     // Start decoding of channel outputs with forward
     //  traversal of trellis!  Stop just before encoder-flushing bits.
     
     // RECALL , at this point due to the line that says:
     //             channel_length = channel_length / N;
     // channel_length = No of Info Bits 
     //
     
     for (t = 0; t < channel_length ; t++) {

           // Consider all states fromt he begining
          step=1;

          // we're going to use the state history array as a circular buffer so
          // we don't have to shift the whole thing left after each bit is
          // processed so that means we need an appropriate pointer

          // set up the state history array pointer for this time t
          sh_ptr = (int) ( ( sh_ptr_previous+ t + 1 ) % (DEPTH_TRELLIS + 1) );
  
          //=====================================================================================

          // repeat for each possible state
          for (j = 0; j < NOSTATES; j+= step) {
               // repeat for each possible convolutional encoder output N-tuple
               for (l = 0; l < N; l++) {
                    if(l==0){ 
						branch_metric =                           
                              abs(   channel_output_matrix[t]					- binary_outputV00[j] ) +
                              abs(   channel_output_matrix[channel_length + t ] - binary_outputV01[j] );          
                    }else{                        
                         branch_metric =  
                              abs( channel_output_matrix[t]						- binary_outputV10[j] ) +
                              abs( channel_output_matrix[channel_length + t ]	- binary_outputV11[j] );
                    }
                    // now choose the surviving path--the one with the smaller accumlated error metric...
                    if ( accum_err_metric1[ nextstate[j][l] ]  > accum_err_metric0[j] + branch_metric ) {
                         // save an accumulated metric value for the survivor state
                         accum_err_metric1[ nextstate[j][l] ] = accum_err_metric0[j] + branch_metric;
                         // update the state_history array with the state number of
                         // the survivor
                         state_history[ nextstate[j][l] ] [sh_ptr] = j;
                    } // end of if-statement
               } // end of 'l' for-loop
          } // end of 'j' for-loop -- we have now updated the trellis       

          // for all rows of accum_err_metric, move col 2 to col 1 and flag col 2
          st = ippsMove_32s(accum_err_metric1,accum_err_metric0,NOSTATES);
          st = ippsSet_32s(MAXINT, accum_err_metric1, NOSTATES);          

          // now start the traceback, if we've filled the trellis
         if (VitSynched[0]==0){
              if (t >= DEPTH_TRELLIS - 1) {
                    // initialize the state_sequence vector--probably unnecessary
                    st = ippsZero_16s(state_sequence, (DEPTH_TRELLIS+ 1));
     
                    // find the element of state_history with the min. accum. error metric
                    // since the outer states are reached by relatively-improbable runs
                    // of zeroes or ones, search from the top and bottom of the trellis in
                    // st = ippsMin_32s( accum_err_metric0,NOSTATES,&MyMin );
                    st = ippsMinIndx_32s( accum_err_metric0,NOSTATES,&MyMin, &MyMinIndx );
                    // now pick the starting point for traceback
                    state_sequence[DEPTH_TRELLIS] = MyMinIndx;               
     
                    // now work backwards from the end of the trellis to the oldest state
                    // in the trellis to determine the optimal path. The purpose of this
                    // is to determine the most likely state sequence at the encoder
                    //  based on what channel symbols we received.
                    for (j = DEPTH_TRELLIS; j > 0; j--) {
                         sh_col = j + ( sh_ptr - DEPTH_TRELLIS );
                         if (sh_col < 0){
                              sh_col = sh_col + DEPTH_TRELLIS + 1;
                         }
                         state_sequence[j - 1] = state_history[ state_sequence[j] ] [sh_col];
                    } // end of j for-loop
     
                    // now figure out what input sequence corresponds to the state sequence
                    //in the optimal path

                    //*(decoder_output_matrix + t - DEPTH_TRELLIS + 1) =
                    //     (nextstate[state_sequence[0]][1]==state_sequence[1] );
					decoder_output_matrix[t - DEPTH_TRELLIS + 1] = 
						(nextstate[state_sequence[0]][1]==state_sequence[1]);
                         
                    if (t == DEPTH_TRELLIS - 1 ) {
                         // Save wining state
                         begin_state[0]= state_sequence[0] ;  
                    }        
         
               } // end of if-statement
          
          }else{                           
               // initialize the state_sequence vector--probably unnecessary
               st = ippsZero_16s(state_sequence, (DEPTH_TRELLIS+1));

               // find the element of state_history with the min. accum. error metric
               // since the outer states are reached by relatively-improbable runs
               // of zeroes or ones, search from the top and bottom of the trellis in
               // st = ippsMin_32s( accum_err_metric0,NOSTATES,&MyMin );
               st = ippsMinIndx_32s( accum_err_metric0,NOSTATES,&MyMin, &MyMinIndx );
               // now pick the starting point for traceback
               state_sequence[DEPTH_TRELLIS] = MyMinIndx;   
                           
               // now work backwards from the end of the trellis to the oldest state
               // in the trellis to determine the optimal path. The purpose of this
               // is to determine the most likely state sequence at the encoder
               //  based on what channel symbols we received.
               for (j = DEPTH_TRELLIS; j > 0; j--) {
                    sh_col = j + ( sh_ptr - DEPTH_TRELLIS );
                    if (sh_col < 0){
                         sh_col = sh_col + DEPTH_TRELLIS + 1;
                    }
                    state_sequence[j - 1] = state_history[ state_sequence[j] ] [sh_col];
               } // end of j for-loop
         
               decoder_output_matrix [ t+1 ] =
                    (nextstate[state_sequence[0]][1]==state_sequence[1] );     
           
          }                  
     } // end of 't' for-loop
     
     // **************************************************************************

     // free the dynamically allocated array storage area
     ippsFree(channel_output_matrix);

     // VitSynched[0] +=1;    // We are synched now !  
     sh_ptr_previous = sh_ptr;    
     
     return;
} // end of function sdvd


// ********************* END OF Soft Decision Viterbi Decoder FUNCTION *******************************                            