#pragma once
#include <limits.h>
#include <float.h>
#include <math.h>
#include <conio.h>
#include <stdio.h>
#include <stdlib.h>
#include <string>
#include <iomanip>
#include <fstream>
#include <iostream>
//Intel
#include "ippdefs.h"
#include "ipp.h"
// Timer
// #include "timer.h"

#define	MAXINT (2147483647)

//=====================================================================
// Define some constants 

// Constraint length 
#define K (7)       

// * 2^(K - 1) -- change as required */
#define NOSTATES (64)      

// n is 2^1 = 2 for rate 1/2  
#define  N (2)

// m (memory length) = K - 1  
#define M (K - 1)  

// little degradation in performance achieved by limiting trellis depth
//     to K * 5--interesting to experiment with smaller values and measure
//   the resulting degradation.  
#define DEPTH_TRELLIS (K * 5)

// Flag for Trellis Termination
#define TERMINATE_TRELLIS (0)
//=====================================================================
// TEST BENCH 
#define MSG_LEN (10000)  
#define LOESN0 (1.0)       /* minimum Es/No at which to test */
#define HIESN0 (1.2)       /* maximum Es/No at which to test */
#define ESN0STEP (0.2)     /* Es/No increment for test driver */
//=====================================================================


void cnv_encd_intel(long input_len, Ipp16s *in_array, Ipp16s *out_array,int *begin_state);
void addnoise_intel(float es_ovr_n0, long channel_len, Ipp16s *in_array, Ipp32f *out_array);
void sdvd_intel(long channel_length,
            Ipp32f *channel_output_vector, Ipp16s *decoder_output_matrix, int *VitSynched, int *begin_state) ;

void sdvd_test_intel(long channel_length,
            Ipp32f *channel_output_vector, Ipp16s *decoder_output_matrix, int *VitSynched, int *begin_state) ;

int soft_quant_intel(float channel_symbol,Ipp16s *quantizer_table);

void deci2bin(int d, int size, int *b);

static int Pow2[8]={1,2,4,8,16,32,64,128};
/* ----------------------------- EMWIN CODE ------------------------------ */
// Convolutional Code : EMWIN Code
//   IMP                     = [171,133];
//   K                       = 7;

static int g[2][K] = {{1, 1, 1, 1,  0, 0, 1},  /* 171 */
                      {1, 0, 1, 1,  0, 1, 1}}; /* 133 */

//static Ipp16s nextstate[64][2] ={
static int nextstate[64][2] ={
     {0 , 32 },    {0 , 32},
     {1 , 33},     {1 , 33},
     {2 , 34},     {2 , 34},
     {3 , 35},     {3 , 35},
     {4 , 36},     {4 , 36},
     {5 , 37},     {5 , 37},
     {6 , 38},     {6 , 38},
     {7 , 39},     {7 , 39},
     {8 , 40},     {8 , 40},
     {9 , 41},     {9 , 41},
     {10 , 42},    {10 , 42},
     {11 , 43},    {11 , 43},
     {12 , 44},    {12 , 44},
     {13 , 45},    {13 , 45},
     {14 , 46},    {14 , 46},
     {15 , 47},    {15 , 47},
     {16 , 48},    {16 , 48},
     {17 , 49},    {17 , 49},
     {18 , 50},    {18 , 50},
     {19 , 51},    {19 , 51},
     {20 , 52},    {20 , 52},
     {21 , 53},    {21 , 53},
     {22 , 54},    {22 , 54},
     {23 , 55},    {23 , 55},
     {24 , 56},    {24 , 56},
     {25 , 57},    {25 , 57},
     {26 , 58},    {26 , 58},
     {27 , 59},    {27 , 59},
     {28 , 60},    {28 , 60},
     {29 , 61},    {29 , 61},
     {30 , 62},    {30 , 62},
     {31 , 63},    {31 , 63}
};

static Ipp16s binary_outputV00[64]={0,7,0,7,0,7,0,7,7,0,7,0,7,0,7,0,7,0,7,0,7,0,7,0,0,7,0,7,0,7,0,7,7,0,7,0,7,0,7,0,0,7,0,7,0,7,0,7,0,7,0,7,0,7,0,7,7,0,7,0,7,0,7,0};
static Ipp16s binary_outputV01[64]={0,7,7,0,0,7,7,0,7,0,0,7,7,0,0,7,7,0,0,7,7,0,0,7,0,7,7,0,0,7,7,0,0,7,7,0,0,7,7,0,7,0,0,7,7,0,0,7,7,0,0,7,7,0,0,7,0,7,7,0,0,7,7,0};
static Ipp16s binary_outputV10[64]={7,0,7,0,7,0,7,0,0,7,0,7,0,7,0,7,0,7,0,7,0,7,0,7,7,0,7,0,7,0,7,0,0,7,0,7,0,7,0,7,7,0,7,0,7,0,7,0,7,0,7,0,7,0,7,0,0,7,0,7,0,7,0,7};
static Ipp16s binary_outputV11[64]={7,0,0,7,7,0,0,7,0,7,7,0,0,7,7,0,0,7,7,0,0,7,7,0,7,0,0,7,7,0,0,7,7,0,0,7,7,0,0,7,0,7,7,0,0,7,7,0,0,7,7,0,0,7,7,0,7,0,0,7,7,0,0,7};

static Ipp8u output[64][2] ={
     {0 ,3 },     {3 ,0 },
     {1 ,2 },     {2 ,1 },	
     {0 ,3 },     {3 ,0 },	
     {1 ,2 },     {2 ,1 },	
     {3 ,0 },     {0 ,3 },	
     {2 ,1 },     {1 ,2 },	
     {3 ,0 },	   {0 ,3 },	
     {2 ,1 },	   {1 ,2 },	
     {3 ,0 },	   {0 ,3 },	
     {2 ,1 },	   {1 ,2 },	
     {3 ,0 },	   {0 ,3 },	
     {2 ,1 },	   {1 ,2 },	
     {0 ,3 },	   {3 ,0 },	
     {1 ,2 },	   {2 ,1 },	
     {0 ,3 },	   {3 ,0 },	
     {1 ,2 },	   {2 ,1 },	
     {2 ,1 },	   {1 ,2 },	
     {3 ,0 },	   {0 ,3 },	
     {2 ,1 },	   {1 ,2 },	
     {3 ,0 },	   {0 ,3 },	
     {1 ,2 },	   {2 ,1 },	
     {0 ,3 },	   {3 ,0 },	
     {1 ,2 },	   {2 ,1 }, 	
     {0 ,3 }, 	   {3 ,0 },	
     {1 ,2 },	   {2 ,1 },	
     {0 ,3 },	   {3 ,0 },	
     {1 ,2 },	   {2 ,1 },	
     {0 ,3 },	   {3 ,0 },	
     {2 ,1 },	   {1 ,2 },	
     {3 ,0 },	   {0 ,3 },	
     {2 ,1 },	   {1 ,2 },	
     {3 ,0 },	   {0 ,3 }};


//OLD Table --> Not efficient
     /*
static Ipp16s quantizer_table[256]= 
    {7,     7,     7,     7,     7,     7,     7,     7,     7,     7,     7,     7,     7,     7,     7,     7,     7,     7,
     7,     7,     7,     7,     7,     7,     7,     7,     7,     7,     7,     7,     7,     7,     7,     7,     7,     7,
     7,     7,     7,     7,     7,     7,     7,     7,     7,     7,     7,     7,     7,     7,     7,     7,     7,     7,
     7,     7,     7,     7,     7,     7,     7,     7,     7,     7,     7,     7,     7,     7,     7,     7,     7,     7,
     7,     7,     7,     7,     7,     7,     7,     7,     7,     7,     7,     7,     7,     7,     7,     7,     7,     7,
     7,     7,     7,     7,     7,     7,     7,     6,     6,     6,     6,     6,     6,     6,     6,     6,     6,     5,
     5,     5,     5,     5,     5,     5,     5,     5,     5,     4,     4,     4,     4,     4,     4,     4,     4,     4,
     4,     4,     3,     3,     3,     3,     3,     3,     3,     3,     3,     3,     3,     2,     2,     2,     2,     2,
     2,     2,     2,     2,     2,     1,     1,     1,     1,     1,     1,     1,     1,     1,     1,     0,     0,     0,
     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
     0,     0,     0,     0};
*/

//New Table
static Ipp16s quantizer_table[256]= 
    {7,     7,     7,     7,     7,     7,     7,     7,     7,     7,     7,     7,     7,     7,     7,     7,     7,     7,
     7,     7,     7,     7,     7,     7,     7,     7,     7,     7,     7,     7,     7,     7,     6,     6,     6,     6,
     6,     6,     6,     6,     6,     6,     6,     6,     6,     6,     6,     6,     6,     6,     6,     6,     6,     6,
     6,     6,     6,     6,     6,     6,     6,     6,     6,     6,     5,     5,     5,     5,     5,     5,     5,     5,
     5,     5,     5,     5,     5,     5,     5,     5,     5,     5,     5,     5,     5,     5,     5,     5,     5,     5,
     5,     5,     5,     5,     5,     5,     4,     4,     4,     4,     4,     4,     4,     4,     4,     4,     4,     4,
     4,     4,     4,     4,     4,     4,     4,     4,     4,     4,     4,     4,     4,     4,     4,     4,     4,     4,
     4,     4,     3,     3,     3,     3,     3,     3,     3,     3,     3,     3,     3,     3,     3,     3,     3,     3,
     3,     3,     3,     3,     3,     3,     3,     3,     3,     3,     3,     3,     3,     3,     3,     3,     2,     2,
     2,     2,     2,     2,     2,     2,     2,     2,     2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
     2,     2,     2,     2,     2,     2,     2,     2,     2,     2,     2,     2,     1,     1,     1,     1,     1,     1,
     1,     1,     1,     1,     1,     1,     1,     1,     1,     1,     1,     1,     1,     1,     1,     1,     1,     1,
     1,     1,     1,     1,     1,     1,     1,     1,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
     0,     0,     0,     0};
