
#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <ar_viterbi_lib.h>
#include <ar_viterbi_wrap.h>
#include <gr_io_signature.h>
#include <ipp.h>

#define BYTESOUT (1)

//#define MINPKT (3000)
// #define TLEN   (30)

gr_viterbi_wrap_sptr 
    gr_make_viterbi_wrap ()
{
  return gr_viterbi_wrap_sptr (new gr_viterbi_wrap ( ));
}

gr_viterbi_wrap::gr_viterbi_wrap ()
  : gr_block ("viterbi_wrap",
		      gr_make_io_signature (1, 1, sizeof (float)),
		      gr_make_io_signature (1, 1, sizeof (short)) )
{
//  set_output_multiple(MINPKT);
       
}

gr_viterbi_wrap::~gr_viterbi_wrap()
{ }

// The forcast is intentionally incorrect.  We know that it takes
// ninput_items_required = samples_per_symbol*noutput_items.  However,
// the scheduler will not give us any data until it has enough for at
// least one output.  If samples_per_symbol is large, and vlen is
// also large, the total number of required samples may exceed the size
// of the inter-block buffers.  Thus, we only ask for one input for
// every output, but don't generate outputs for every input.
void gr_viterbi_wrap::forecast(int noutput_items, 
                                    gr_vector_int &ninput_items_required)
{
     /*
  unsigned ninputs = ninput_items_required.size ();

  for (unsigned i = 0; i < ninputs; i++) {
    ninput_items_required[i] = ( noutput_items  + TLEN) * 2;
  }
  */

  int ninputs   = 1;
  
  ninput_items_required[0] = ( noutput_items  + DEPTH_TRELLIS) * 2;

  //printf("ninput_items_required = %i\n",ninput_items_required[0]);

}

int
gr_viterbi_wrap::general_work (int noutput_items,
		     gr_vector_int &ninput_items,
		     gr_vector_const_void_star &input_items,
		     gr_vector_void_star &output_items){
		          
     // FYI:  DEPTH_TRELLIS=35		          
     float     *in_vec             =  (float *)   input_items[0];    
     static int synched            =0;  
     int       begin_state         =0;
     int       ni                  = ninput_items[0];     
     int       channel_length;
	short     *out_vec            = (short *)   output_items[0];     
	unsigned char  *vit_bytes     = (unsigned char *)   output_items[0];

	if(BYTESOUT==0){		
	     // USe Floor: We need to fill in "complete" bytes. If there are remaining 
	     // bits, use them next time
		channel_length = int( floor(float(ni/2))*2);  		
	}else{		
		channel_length = int( floor(float(ni/16))*16);  		 
		
	}
     short          *vit_out		     = new short [channel_length];
     static int     VitSynched          =0;
     int            consumed_bits_from_input = channel_length;
     int            generated_output_bits    = channel_length/2 - DEPTH_TRELLIS*(VitSynched==0);     
  
     //----------
     // DECODE 
     //----------

     if (VitSynched==0){
          //We are not synched, so there's no point in decoding all bits, since we have
          // a 50% chance of being wrong
          // So we decode a small number of packets:  1000
          int  TestPkts=1000;
          if ( channel_length < TestPkts){               
               TestPkts=channel_length;
          }                           
          generated_output_bits    = TestPkts/2 - DEPTH_TRELLIS;
          
	     if(BYTESOUT==0){
				sdvd_intel( TestPkts, in_vec, out_vec	,&VitSynched,&begin_state); 				
		}else{
				sdvd_intel( TestPkts, in_vec, vit_out	,&VitSynched,&begin_state); 
		}
          //----------------------------------------------------------------------------
          // TEST SYNCH
          //------------     
          Ipp16s          *myEncodedBits;
          myEncodedBits  = ippsMalloc_16s(2* generated_output_bits);        
          //-------------------
          // ENCODE - Phase #1
          //------------------- 
          // Recall modulation here is   y=1-2*x ==> 0->1, 1->-1
		if(BYTESOUT==0){
		     cnv_encd_intel(generated_output_bits, out_vec, myEncodedBits,&begin_state) ;								
		}else{
			cnv_encd_intel(generated_output_bits, vit_out, myEncodedBits,&begin_state) ;
		}          
          int SynchThreshold    =int (floor( (generated_output_bits*2)/10.0)) ;
          int SumErrors=0;
          for (int ii=0; ii< 2* (generated_output_bits-DEPTH_TRELLIS); ii++){          
               SumErrors += ( (in_vec[ii ] < 0)!=myEncodedBits[ii+2*DEPTH_TRELLIS*(VitSynched!=0)]);
          }     
          
          // printf("Sum Errors = %i \t  Threshold=%i \tBegining State=%i  \n", SumErrors,SynchThreshold,begin_state);      
         
          if (SumErrors < SynchThreshold){
               printf("---------------------------------------\n");
               printf("Viterbi Decoder: RX SIGNAL WAS SYNCHED \n");
               printf("---------------------------------------\n");
               // We are synched !!
               VitSynched     =1;       
               // Decode All PKTS
               generated_output_bits    = channel_length/2;   
               // sdvd_intel( channel_length, in_vec, vit_out	,&VitSynched,&begin_state); 
               
			if(BYTESOUT==0){
			     sdvd_intel( channel_length, in_vec, out_vec	,&VitSynched,&begin_state); 				
			}else{
				sdvd_intel( channel_length, in_vec, vit_out	,&VitSynched,&begin_state); 
			}			   
			consume_each (channel_length);
          }else{
               // We are NOT synched !!          
               printf("---------------------------------------\n");
               printf("Viterbi Decoder: RX SIGNAL WAS SHIFTED \n");               
               // We need to decode our packets again since we were off by one bit
               // And our previos decoded packet was wrong
               Ipp32f          *myShiftedInput;
               myShiftedInput  = ippsMalloc_32f(channel_length-1);      
               for (int ii=0; ii<channel_length-1 ; ii++){          
                    myShiftedInput[ii] = in_vec[ii+1];
               } 
               VitSynched=1;     
               generated_output_bits    = channel_length/2;   
               // sdvd_intel( channel_length-1, in_vec, vit_out,&VitSynched,&begin_state);  
			if(BYTESOUT==0){
			     sdvd_intel( channel_length-1, in_vec,out_vec,&VitSynched,&begin_state); 				
			}else{                         
				sdvd_intel( channel_length-1, in_vec,vit_out,&VitSynched,&begin_state);                      
			}
               consume_each (channel_length-1);          
               ippsFree(myShiftedInput);               
               printf("Viterbi Decoder: RX SIGNAL WAS CORRECTED\n");   
               printf("----------------------------------------\n");
          }
     }else{
          // -------------------------
          //   We are already synched
          // -------------------------
          generated_output_bits    = channel_length/2;
          //sdvd_intel( channel_length, in_vec, vit_out,&VitSynched,&begin_state); 
		if(BYTESOUT==0){
		     sdvd_intel( channel_length, in_vec, out_vec	,&VitSynched,&begin_state); 				
		}else{
               //printf("Here 00 \n");
			sdvd_intel( channel_length, in_vec, vit_out	,&VitSynched,&begin_state); 
               //printf("Here 01 \n");
		}
          consume_each (channel_length);      
     }

	if(BYTESOUT==1){        
	     //printf("Generated OutputBits=%i \t No.Input Items=%i \t BitsUsed=Channel_length%i \n ",generated_output_bits,ni,channel_length);
	     unsigned char cumm_temp=0;
		for (int nn = 1; nn < generated_output_bits+1; nn += 8){
		//for (int nn = 1; nn < generated_output_bits-8; nn += 8){	
               cumm_temp = 0;
			for (int mm = nn; mm < nn + 8; mm++){
				cumm_temp <<= 1;
				cumm_temp += (vit_out[mm] & 0x0001);
				//printf("[%i,%i,%i]  \t ",nn,mm,nn-1>>3);
			}
			//printf("\n ");
			vit_bytes[nn-1>>3] = cumm_temp; // same as out_vec[nn/8] = cumm_temp; without dividing
		}
          
          
          //------------------------------------
          // RETURN GENERATED OUTPUT BYTES  !!!
		return int(generated_output_bits/8);
		//------------------------------------
          
          //printf("Done !!!!! 01 \n");		
	}else{
		return generated_output_bits;
	}  
}
