
#ifndef   INCLUDED_GR_viterbi_wrap_H
#define	INCLUDED_GR_viterbi_wrap_H

#include <gr_sync_decimator.h>
#include <ipp.h>

class gr_viterbi_wrap;
typedef boost::shared_ptr<gr_viterbi_wrap> gr_viterbi_wrap_sptr;
//! Create an integrate and dump block.  The output is given by
//! Sum[x(1:samples_per_symbol].  One output is generated every
//! samples_per_symbol inputs.  If the input is a vector, the 
//! summation is done accross vector inputs.
//! \param samples_per_symbol Number of inputs to accumulate per output
//! \param vlen Length of the input vector.  Defaults to 1
gr_viterbi_wrap_sptr 
    gr_make_viterbi_wrap ();


/*!
 * Simple integrate and dump block.  See gr_make_viterbi_wrap for details
 */
// It would be more elegant to derive this block from gr_sync_decimator since
// we know the reduction ratio.  However, the input is then always in chunks of
// samples_per_symbol * vlen entries.  This may be a huge number if we are 
// averaging vectors (e.g. FFT outputs).  Thus, we derive from a gr_block.
class gr_viterbi_wrap : public gr_block
{
 private:
  friend gr_viterbi_wrap_sptr 
      gr_make_viterbi_wrap ();
  
  /*
  float d_mean_pwr;
  float d_pwr_alpha;
  int d_samples_per_symbol;
  int d_vlen;
  int d_sample_count;
  float *d_acc;
*/
  gr_viterbi_wrap();
 public:
  ~gr_viterbi_wrap();
  
  void forecast(int noutput_items, gr_vector_int &ninput_items_required);
  
  int general_work (int noutput_items,
		    gr_vector_int &ninput_items,
		    gr_vector_const_void_star &input_items,
		    gr_vector_void_star &output_items);

  /*
  //! Returns the mean value of the integrated samples. Mean is computed
  //! as an IIR filter: mean[k] = (1-alpha) *mean[k-1]+alpha*integrated[k]
  float get_mean_pwr() {return d_mean_pwr;}
  //! Set the averaging constant
  void  set_pwr_alpha(float pwr_alpha) {d_pwr_alpha = pwr_alpha;}
  */
};

#endif
