// Modified by Esteban Valles 3/24/08

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif
#include <ari_mmse_fir_interpolator_cc.h>
#include <gr_fir_util.h>
#include <gr_fir_ccf.h>
#include <assert.h>
#include <cmath>
#include "interpolator_taps.h"

#define  USE_INTEL
#undef  USE_INTEL
ari_mmse_fir_interpolator_cc::ari_mmse_fir_interpolator_cc ()
{
     #ifdef USE_INTEL
          // IppsFIRState_32f *FIRState; 
          ippsFIRInitAlloc_32f(&FIRStateRe,taps[0],NTAPS,NULL);
          ippsFIRInitAlloc_32f(&FIRStateIm,taps[0],NTAPS,NULL);
     #else
          // Traditional GNU Radio Filtering
          filters.resize (NSTEPS + 1);  
          for (int i = 0; i < NSTEPS + 1; i++){
               std::vector<float> t (&taps[i][0], &taps[i][NTAPS]);
               filters[i] = gr_fir_util::create_gr_fir_ccf (t);
          }
     #endif
}

ari_mmse_fir_interpolator_cc::~ari_mmse_fir_interpolator_cc ()
{
     #ifdef USE_INTEL
          ippsFIRFree_32f(FIRStateRe);
          ippsFIRFree_32f(FIRStateIm);
     #else
          for (int i = 0; i < NSTEPS + 1; i++)
          delete filters[i];     
     #endif
}

unsigned
ari_mmse_fir_interpolator_cc::ntaps () const
{
  return NTAPS;
}

unsigned
ari_mmse_fir_interpolator_cc::nsteps () const
{
  return NSTEPS;
}

gr_complex
ari_mmse_fir_interpolator_cc::interpolate (const gr_complex input[], float mu)
{
  int	imu = (int) rint (mu * NSTEPS);

  //EV   
  static int        prev_imu=-1;  // Initialize it with an out of range value.
  //----------------------------------------
  #ifdef USE_INTEL     
     IppStatus         status;      
     Ipp32fc           r;
  #else
     gr_complex r;  
  #endif
  // Debug
  if (imu <0 || imu > NSTEPS)  {
      assert(0);
  }
  assert (imu >= 0);
  assert (imu <= NSTEPS);

  if (imu != prev_imu){     
      #ifdef USE_INTEL
       // Load New Taps
          status = ippsFIRSetTaps_32f( taps[imu],FIRStateRe);
          status = ippsFIRSetTaps_32f( taps[imu],FIRStateIm);
     #endif  
     prev_imu = imu;
  }
  #ifdef USE_INTEL    
     status = ippsFIROne_32f( input[0].real(),&r.re, FIRStateRe);
     status = ippsFIROne_32f( input[0].imag(),&r.im, FIRStateIm);
     
    /*
     //-----------------------------------------------------
     // Debug Mode
     status = ippsFIRSetTaps_32f( taps[0],FIRStateRe);
     status = ippsFIRSetTaps_32f( taps[0],FIRStateIm);
     status = ippsFIROne_32f( 1.0,&fir_out_re,FIRStateRe);
     status = ippsFIROne_32f( 0.1,&fir_out_im,FIRStateIm);
     r      = gr_complex(fir_out_re,fir_out_im);
     // in=gr_complex(1.0,1.0);
     //r      =filters[0]->filter (input);
     static int iter=0;
     printf("%i \t Out=[%f,%f] \n",iter,fir_out_re,fir_out_im);
     iter +=1;
     //-----------------------------------------------------
     */
    // r         =filters[imu]->filter (input);
     return *((gr_complex *) &r);
 #else 
     r         =filters[imu]->filter (input);
     return r;
 #endif


  
}
