#ifndef _GRI_MMSE_FIR_INTERPOLATOR_CC_H_
#define _GRI_MMSE_FIR_INTERPOLATOR_CC_H_

#include <gr_complex.h>
#include <vector>


//EV: Intel
#include "ippdefs.h"
#include "ipp.h"

class gr_fir_ccf;

/*!
 * \brief Compute intermediate samples between signal samples x(k*Ts)
 *
 * This implements a Mininum Mean Squared Error interpolator with 8 taps.
 * It is suitable for signals where the bandwidth of interest B = 1/(4*Ts)
 * Where Ts is the time between samples.
 *
 * Although mu, the fractional delay, is specified as a float, it is actually
 * quantized.  0.0 <= mu <= 1.0.  That is, mu is quantized in the interpolate 
 * method to 32nd's of a sample.
 */

class ari_mmse_fir_interpolator_cc {
public:
  ari_mmse_fir_interpolator_cc ();
  ~ari_mmse_fir_interpolator_cc ();

  unsigned ntaps () const;
  unsigned nsteps () const;

  IppsFIRState_32f *FIRStateRe, *FIRStateIm; 

  /*!
   * \brief compute a single interpolated output value.
   * \p input must have ntaps() valid entries.
   * input[0] .. input[ntaps() - 1] are referenced to compute the output value.
   *
   * \p mu must be in the range [0, 1] and specifies the fractional delay.
   *
   * \returns the interpolated input value.
   */
  gr_complex interpolate (const gr_complex input[], float mu);

protected:
  std::vector<gr_fir_ccf *>	filters;
};


#endif /* _GRI_MMSE_FIR_INTERPOLATOR_CC_H_ */
