/*!
Interpolation and decimation change the rate of the input data.  An integer
rate change is typically accomplished by filtering and removing/adding
samples.  The filter bandwidth is on the order of 1/rate change factor.
For large rate change factors, the bandwidth is very small, requiring very
sharp filters.  These filters are very computationally intensive (a lot of
taps).  It is much more efficient to break up the large factor into a few
smaller ones, such that the product of the smaller changes is equal to the
desired factor.  Unfortunately, many numbers cannot be factored into 
multiple small factors.  It is often acceptable to use a rate change factor
that is close, but not exactly equal to the desired factor.  A fractional
rate change block can then be used to obtain the exact factor.  
This function returns an optimal decomposition of the desired factor, n,
into multiple stages of no more than 'max_factor.'  The decomposition is
guaranteed to have the product of the factors <= n.  Experimental results
show that the error is less than 15% for n<1000, max_factor=6

NOTE: the implementation is far from optimized, but the results are optimal
*/
#include <vector>
std::vector<int> ari_rate_change_factor(int n, int max_factor);