// This file contains useful general-purpose stuff that does not belong 
// in any other obvious place.
#pragma once
#include <vector>
#include <string>
#include <gr_runtime_types.h>

using namespace std;
void dump(const vector<float> &data, const string &filename);
void grab(vector<float> &data, const string &filename);
gr_block_sptr find_block(const vector<gr_block_sptr> &blocks, 
                         const std::string name);

// Note: template functions have to be defined in header files, or you will
// get link errors
template <typename T>
T* find_block_ptr(const vector<gr_block_sptr> &blocks, const std::string name)
{
    gr_block_sptr block = find_block(blocks, name);
    return (T *)block.get();
}

template <typename T>
vector<boost::shared_ptr<T> > find_blocks(const vector<gr_block_sptr> &blocks,
                                          const std::string name)
{
    vector<boost::shared_ptr<T> > result;

    for (vector<gr_block_sptr>::const_iterator b=blocks.begin(); b<blocks.end(); ++b)
        if ((*b)->name() == name) 
            result.push_back(boost::static_pointer_cast<T, gr_basic_block>(*b) );

    if (result.size() == 0)
    {
        printf("Could not find any blocks : %s\n", name.c_str());
        assert(0);
    }

    return result;    
}

template <typename T>
boost::shared_ptr<T> cast_block(gr_block_sptr &block)
{
    return boost::static_pointer_cast<T, gr_basic_block> (block);
}