
#ifndef INCLUDED_TCP_DEBUG_SINK_H
#define INCLUDED_TCP_DEBUG_SINK_H

#include "tcp_debug_constants.h"

#include <gr_sync_block.h>
#include "circular_buffer.h" // Thread-safe ring buffer
#include <omnithread.h>
#include <windows.h> // For Windows networking
#include <winsock.h>
//#include <sys/socket.h> // Cross-platform networking
//#include <arpa/inet.h>  // ???


class tcp_debug_sink;
typedef boost::shared_ptr<tcp_debug_sink> tcp_debug_sink_sptr;

tcp_debug_sink_sptr make_tcp_debug_sink(const char* name, size_t itemsize, int typecode);

//! A sink class (similar to gr_file_sink) that allows a separate
//  program to connect via TCP/IP and request blocks of data.
//  It sits idle (discarding any incoming data) until a connection
//  is made (see tcp_debug_server).  Once a connection is established,
//  the client can request that all data be forwarded, request a single
//  packet's worth of data, pause all transfers, or close the connection.
//  This provides a very lightweight realtime debugging tool, with
//  almost zero overhead if it remains unused.
class tcp_debug_sink : public gr_sync_block
{
  friend tcp_debug_sink_sptr make_tcp_debug_sink(const char* name, size_t itemsize, int typecode);

 private:
  size_t     d_itemsize;
  SOCKET     d_socket;       // Connection to client, or else NULL
  omni_mutex d_socket_mutex;
  int        d_count;        // -1 for infinite, 0 for pause, otherwise N more

 protected:
  tcp_debug_sink(const char* name, size_t itemsize, int typecode);

 public:
  ~tcp_debug_sink();

  int work (int noutput_items,
		    gr_vector_const_void_star &input_items,
		    gr_vector_void_star &output_items);

  //! Accept a new incoming connection from tcp_debug_server
  void handoff_socket(SOCKET sock);
  void close_socket();
};


#endif /* INCLUDED_TCP_DEBUG_SINK_H */
