
#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <ar_multiply_const_cc.h>
#include <gr_io_signature.h>
#include <ipp.h>


ar_multiply_const_cc_sptr
ar_make_multiply_const_cc (gr_complex k)
{
  return ar_multiply_const_cc_sptr (new ar_multiply_const_cc (k));
}

ar_multiply_const_cc::ar_multiply_const_cc (gr_complex k)
  : gr_sync_block ("multiply_const_cc",
		   gr_make_io_signature (1, 1, sizeof (gr_complex)),
		   gr_make_io_signature (1, 1, sizeof (gr_complex))),
    d_k (k)
{
}

int
ar_multiply_const_cc::work (int noutput_items,
		   gr_vector_const_void_star &input_items,
		   gr_vector_void_star &output_items)
{
    /*
  gr_complex *iptr = (gr_complex *) input_items[0];
  gr_complex *optr = (gr_complex *) output_items[0];

 
  int	size = noutput_items;

  while (size >= 8){
    *optr++ = *iptr++ * d_k;
    *optr++ = *iptr++ * d_k;
    *optr++ = *iptr++ * d_k;
    *optr++ = *iptr++ * d_k;
    *optr++ = *iptr++ * d_k;
    *optr++ = *iptr++ * d_k;
    *optr++ = *iptr++ * d_k;
    *optr++ = *iptr++ * d_k;
    size -= 8;
  }

  while (size-- > 0)
    *optr++ = *iptr++ * d_k;
  */
  
  // USE INTEL IPP
  
  gr_complex *iptr = (gr_complex *) input_items[0];
  gr_complex *optr = (gr_complex *) output_items[0];

  Ipp32fc MyConstant;

  MyConstant.re = real(d_k);
  MyConstant.im = imag(d_k);

  ippsMulC_32fc((Ipp32fc*) iptr, MyConstant,(Ipp32fc*) optr, noutput_items);
  
  return noutput_items;
}
