
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif
#include <ar_noise_source_c.h>
#include <gr_io_signature.h>
#include <stdexcept>


gr_noise_source_c_sptr 
gr_make_noise_source_c (gr_noise_type_t type, float ampl, long seed)
{
  return gr_noise_source_c_sptr (new gr_noise_source_c (type, ampl, seed));
}


gr_noise_source_c::gr_noise_source_c (gr_noise_type_t type, float ampl, long seed)
  : gr_sync_block ("noise_source_c",
		   gr_make_io_signature (0, 0, 0),
		   gr_make_io_signature (1, 1, sizeof (gr_complex))),
    d_type (type),
    d_ampl (ampl),
    d_seed (seed),
    d_update(true), // Update the first time work is called
    d_uniform(0),
    d_gaussian(0)
{
}

void gr_noise_source_c::update_sources()
{
    if (d_uniform) ippsRandUniformFree_32f(d_uniform);
    ippsRandUniformInitAlloc_32f(&d_uniform, -d_ampl, d_ampl, d_seed);
    if (d_gaussian) ippsRandGaussFree_32f(d_gaussian);
    ippsRandGaussInitAlloc_32f(&d_gaussian, 0, d_ampl, d_seed);
}

int
gr_noise_source_c::work (int noutput_items,
		   gr_vector_const_void_star &input_items,
		   gr_vector_void_star &output_items)
{
  gr_complex *out = (gr_complex *) output_items[0];
  // Change the source if the amplitude or type have changed.  This is faster than
  // using a separate multiply function to generate the scaled vector
  if (d_update)
  {
      update_sources();
      d_update = false;
  }
  // IPP does not have functions to generate complex valued noise.  Generate 
  // real-valued, but twice the length.
  switch (d_type){

  case GR_UNIFORM:
    ippsRandUniform_32f((Ipp32f*)out, noutput_items*2, d_uniform);
    
    break;

  case GR_GAUSSIAN:
    ippsRandGauss_32f((Ipp32f*)out, noutput_items*2, d_gaussian);
    break;

  default:
    throw std::runtime_error ("invalid type");
  }

  return noutput_items;
}


