
#ifndef INCLUDED_GR_NOISE_SOURCE_C_H
#define INCLUDED_GR_NOISE_SOURCE_C_H

#include <gr_sync_block.h>
#include <gr_noise_type.h>
#include <ipp.h>


class gr_noise_source_c;
typedef boost::shared_ptr<gr_noise_source_c> gr_noise_source_c_sptr;

//! Create a complex noise generator 
//! \param type uniform or gaussian
//! \param ampl [-ampl .. +ampl] for uniform, standard deviation for gaussian
gr_noise_source_c_sptr 
gr_make_noise_source_c (gr_noise_type_t type, float ampl, long seed = 3021);

/*!
 * \brief random number source
 * \ingroup source
 */
class gr_noise_source_c : public gr_sync_block {
  friend gr_noise_source_c_sptr 
  gr_make_noise_source_c (gr_noise_type_t type, float ampl, long seed);

  gr_noise_type_t	        d_type;
  float			            d_ampl;
  long                      d_seed;
  bool                      d_update;
  IppsRandUniState_32f      *d_uniform;
  IppsRandGaussState_32f    *d_gaussian;

  gr_noise_source_c (gr_noise_type_t type, float ampl, long seed = 3021);
  
  void update_sources();
 public:
  //! Set the noise type
  //! \param type uniform or gaussian
  void set_type (gr_noise_type_t type) {d_type = type; d_update = true;}
  //! Set the noise level
  //! \param ampl [-ampl .. +ampl] for uniform, standard deviation for gaussian
  void set_amplitude (float ampl) {d_ampl = ampl; d_update = true;}

  virtual int work (int noutput_items,
			 gr_vector_const_void_star &input_items,
			 gr_vector_void_star &output_items);
};


#endif
