#ifndef INCLUDED_GR_SIG_SOURCE_C_H
#define INCLUDED_GR_SIG_SOURCE_C_H

#include <gr_sync_block.h>
#include <gr_sig_source_waveform.h>
#include <gr_fxpt_nco.h>

class gr_sig_source_c;
typedef boost::shared_ptr<gr_sig_source_c> gr_sig_source_c_sptr;

/*!
 * \brief signal generator with gr_complex output.
 * \ingroup source
 */

class gr_sig_source_c : public gr_sync_block {
  friend gr_sig_source_c_sptr 
  gr_make_sig_source_c (double sampling_freq, gr_waveform_t waveform,
		       double frequency, double ampl, gr_complex offset);

  double		d_sampling_freq;
  gr_waveform_t		d_waveform;
  double		d_frequency;
  double		d_ampl;
  gr_complex		d_offset;
  float         d_phase, d_freq; // for IPP functions


  gr_sig_source_c (double sampling_freq, gr_waveform_t waveform,
	  double wave_freq, double ampl, gr_complex offset);

 public:
  virtual int work (int noutput_items,
		    gr_vector_const_void_star &input_items,
		    gr_vector_void_star &output_items);

  // ACCESSORS
  double sampling_freq () const { return d_sampling_freq; }
  gr_waveform_t waveform () const { return d_waveform; }
  double frequency () const { return d_frequency; }
  double amplitude () const { return d_ampl; }
  gr_complex offset () const { return d_offset; }

  // MANIPULATORS
  void set_sampling_freq (double sampling_freq);
  void set_waveform (gr_waveform_t waveform);
  void set_frequency (double frequency);
  void set_amplitude (double ampl);
  void set_offset (gr_complex offset);
};

gr_sig_source_c_sptr
gr_make_sig_source_c (double sampling_freq, gr_waveform_t waveform,
		     double wave_freq, double ampl, gr_complex offset = 0);


#endif
